//
//  AdminCell.m
//  butterflyradio
//
//  Created by Denny Kwon on 8/14/12.
//
//

#import "AdminCell.h"
#import <QuartzCore/QuartzCore.h>


@implementation AdminCell
@synthesize stationImage;
@synthesize nameLabel;
@synthesize categoryLabel;
@synthesize btnPremiumTrack;
@synthesize delegate;
@synthesize backgrounds;
@synthesize btnTitles;
@synthesize cellHeight;

- (id)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier
{
    self = [super initWithStyle:style reuseIdentifier:reuseIdentifier];
    if (self) {

        self.backgrounds = @[@"bg_button_station.png", @"bg_button_station_admins.png", @"bg_button_station_info.png", @"bg_button_station_garbage.png"];

        self.btnTitles = @{@"bg_button_station.png":@"tracks", @"bg_button_station_admins.png":@"admins", @"bg_button_station_garbage.png":@"delete", @"bg_button_station_info.png":@"details"};

        self.selectionStyle = UITableViewCellSelectionStyleNone;
        UIImage *backgroundImage = [UIImage imageNamed:@"bg_cell_station.png"];
        self.contentView.backgroundColor = [UIColor colorWithPatternImage:backgroundImage];
        self.cellHeight = backgroundImage.size.height;
    }
    return self;
}

- (void)dealloc
{
    self.btnTitles = nil;
    self.backgrounds = nil;
    self.stationImage = nil;
    self.nameLabel = nil;
    self.btnPremiumTrack = nil;
    [super dealloc];
}



- (void)setup
{
    CGRect frame = CGRectMake(19.0f, 13.0f, 80.0f, 80.0f);
    UIView *background = [[UIView alloc] initWithFrame:frame];
    background.backgroundColor = [UIColor darkGrayColor];
    background.layer.masksToBounds = YES;
    [self.contentView addSubview:background];
    [background release];
    
    frame.origin.x += 1.0f;
    frame.origin.y -= 1.0f;
    
    stationImage = [[UIImageView alloc] initWithFrame:frame];
    self.stationImage.layer.masksToBounds = YES;
    self.stationImage.backgroundColor = [UIColor lightGrayColor];
    self.stationImage.layer.borderColor = [[UIColor darkGrayColor] CGColor];
    self.stationImage.layer.borderWidth = 0.5f;
    self.stationImage.image = [UIImage imageNamed:@"Icon@2x.png"];
    [self.contentView addSubview:self.stationImage];
    
    frame = stationImage.frame;
    nameLabel = [[UILabel alloc] initWithFrame:CGRectMake(frame.origin.x+frame.size.width+10, frame.origin.y, 200, 20)];
    nameLabel.textColor = [UIColor darkGrayColor];
    nameLabel.font = [UIFont fontWithName:kFont size:16.0f];
    nameLabel.backgroundColor = [UIColor clearColor];
    [self.contentView addSubview:nameLabel];
    
    frame = nameLabel.frame;
    frame.origin.y += frame.size.height;
    frame.size.height = 15.0f;
    categoryLabel = [[UILabel alloc] initWithFrame:frame];
    categoryLabel.textColor = [UIColor grayColor];
    categoryLabel.font = [UIFont fontWithName:kFont size:13.0f];
    categoryLabel.backgroundColor = [UIColor clearColor];
    [self.contentView addSubview:categoryLabel];
    
    CGFloat w = 30.0f;
    self.btnPremiumTrack = [UIButton buttonWithType:UIButtonTypeCustom];
    [self.btnPremiumTrack setBackgroundImage:[UIImage imageNamed:@"icon_microphone.png"] forState:UIControlStateNormal];
    self.btnPremiumTrack.frame = CGRectMake(self.contentView.frame.size.width-w-20.0f, categoryLabel.frame.origin.y+5, w, w);
    [self.btnPremiumTrack addTarget:self action:@selector(btnPremiumTapped:) forControlEvents:UIControlEventTouchUpInside];
    [self.contentView addSubview:self.btnPremiumTrack];
    
    UIImage *img_btn_station = [UIImage imageNamed:[backgrounds objectAtIndex:0]];
    frame = CGRectMake(13.0f, 99.5f, 0.97*img_btn_station.size.width, img_btn_station.size.height);

    // Set up buttons along bottom:
    for (int i=0; i<self.backgrounds.count; i++) {
        NSString *background = [self.backgrounds objectAtIndex:i];
        UIButton *btn = [self detailButton:frame backgroundKey:background];
        [self.contentView addSubview:btn];
        frame.origin.x += frame.size.width;
    }
}

- (UIButton *)detailButton:(CGRect)frame backgroundKey:(NSString *)background
{
    UIButton *btn = [UIButton buttonWithType:UIButtonTypeCustom];
    btn.adjustsImageWhenHighlighted = YES;
    btn.contentVerticalAlignment = UIControlContentVerticalAlignmentBottom;
    [btn setTitle:[self.btnTitles objectForKey:background] forState:UIControlStateNormal];
    [btn setTitleColor:[UIColor grayColor] forState:UIControlStateNormal];
    btn.titleLabel.font = [UIFont fontWithName:kFont size:9.0f];
    [btn addTarget:self action:@selector(buttonTapped:) forControlEvents:UIControlEventTouchUpInside];
    btn.frame = frame;
    [btn setBackgroundImage:[UIImage imageNamed:background] forState:UIControlStateNormal];
    return btn;
}

- (void)fillImage:(UIImage *)img
{
    
    if (![stationImage.image isEqual:img]){
        self.stationImage.image = img;
        
        [UIView animateWithDuration:0.08f
                              delay:0.0f
                            options:UIViewAnimationOptionCurveLinear
                         animations:^{
                             stationImage.transform = CGAffineTransformMakeScale(1.1, 1.1);
                         }
                         completion:^(BOOL finished){
                             
                             [UIView animateWithDuration:0.08f
                                                   delay:0.0f
                                                 options:UIViewAnimationOptionCurveLinear
                                              animations:^{
                                                  stationImage.transform = CGAffineTransformMakeScale(0.95, 0.95);
                                              }
                                              completion:^(BOOL finished){
                                                  [UIView animateWithDuration:0.08
                                                                        delay:0.0f
                                                                      options:UIViewAnimationOptionCurveLinear
                                                                   animations:^{
                                                                       stationImage.transform = CGAffineTransformIdentity;
                                                                   }
                                                                   completion:NULL];
                                                  
                                              }];
                             
                             
                         }];
    }
}

- (void)buttonTapped:(UIButton *)btn
{
    if (btn.adjustsImageWhenHighlighted)
        [delegate buttonAction:self.tag title:btn.titleLabel.text];
}

- (void)btnPremiumTapped:(UIButton *)btn
{
    NSLog(@"btnPremiumTapped:");
    [self.delegate postPremiumTrack:self.tag];
}

+ (CGFloat)rowHeight
{
    return 140.0f;
}

- (void)setSelected:(BOOL)selected animated:(BOOL)animated
{
    [super setSelected:selected animated:animated];

    // Configure the view for the selected state
}

@end
