//
//  Host.m
//  butterflyradio
//
//  Created by Denny Kwon on 6/3/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "BRHost.h"

static NSString *emailKey = @"hostEmail";

@implementation BRHost
@synthesize email;
@synthesize emailHost;
@synthesize name;
@synthesize stations;
@synthesize loggedIn;
@synthesize guest;
@synthesize appHost;
@synthesize req;
@synthesize premiumThreads;

- (id)init
{
    self = [super init];
    if (self){
        NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
        NSString *savedEmail = [defaults objectForKey:emailKey];
        if (savedEmail==nil){
            savedEmail = @"none";
        }
        
        self.email = savedEmail;
        self.tickets = 0;
        loggedIn = FALSE;
        NSLog(@"HOST EMAIL == %@", email);
        self.premiumThreads = [NSArray array];
    }
    return self;
}


- (void)populate:(NSDictionary *)info
{
    for (NSString *key in [info allKeys]){
        if ([key isEqualToString:@"name"]){
            self.name = [info objectForKey:key];
        }
        if ([key isEqualToString:@"email"]){
            self.email = [info objectForKey:key];
            self.loggedIn = TRUE;
            
            NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
            [defaults setObject:self.email forKey:emailKey];
            [defaults synchronize];
        }
        if ([key isEqualToString:@"emailhost"]){
            self.emailHost = [info objectForKey:key];
        }
        if ([key isEqualToString:@"tickets"]){
            self.tickets = [[info objectForKey:key] intValue];
        }
        if ([key isEqualToString:@"premium threads"]){
            self.premiumThreads = [info objectForKey:key];
        }
        
        if ([key isEqualToString:@"shared stations"]){ //all stations where the host is an admin.
            NSArray *s = [info objectForKey:key];
            if ([s count]==0){ self.stations = [NSArray arrayWithObject:@"none"]; }
            else{
                NSMutableArray *a = [NSMutableArray array];
                for (NSDictionary *info in s){ 
                    Station *st = [[Station alloc] init];
                    [st populate:info];
                    if (self.guest==TRUE){
                        if ([st.admins containsObject:self.appHost]){ //only add stations that the host shares with current app host
                            [a addObject:st];
                        }
                    }
                    else {
                        [a addObject:st];
                    }
                    [st release];
                }
                self.stations = a;
            }
        }
    }
}

- (void)abortRequest
{
    if (self.req){
        req.delegate = nil;
        [req cancel];
        self.req = nil;
    }
}

- (void)refresh
{
    [self abortRequest];
    
//    NSString *url = [NSString stringWithFormat:@"http://www.butterflyradio.com/api/host?email=%@", self.email];
    NSString *url = [NSString stringWithFormat:@"http://%@/api/host?email=%@", kUrl, self.email];
    self.req = [BRNetworkOp operationWithAddress:url parameters:nil];
    [self.req setHttpMethod:@"GET"];
    self.req.delegate = self;
    [self.req sendRequest];
}

- (void)addPremiumThread:(BRTrack *)track
{
    [self abortRequest];

    NSString *url = [NSString stringWithFormat:@"http://%@/api/host?email=%@&action=premium&thread=%@&tickets=%d", kUrl, self.email, track.thread, track.value];
    self.req = [BRNetworkOp operationWithAddress:url parameters:nil];
    [self.req setHttpMethod:@"PUT"];
    self.req.delegate = self;
    [self.req sendRequest];
}

- (void)requestData:(NSArray *)pkg //returns [address, data]
{
    if (pkg==nil)
        return;
    
    id json = [self.req parseJsonResponse];
    if (!json)
        return;
    
    NSDictionary *d = [json objectForKey:@"results"];
    NSLog(@"%@", [d description]);
    NSString *confirmation = [d objectForKey:@"confirmation"];
    
    if ([confirmation isEqualToString:@"found"]==YES || [confirmation isEqualToString:@"success"]==YES){
        NSDictionary *info = [d objectForKey:@"host"];
        [self populate:info];
    }
    
    [[NSNotificationCenter defaultCenter] postNotification:[NSNotification notificationWithName:kRefreshNotification object:nil]];
}


- (void)dealloc
{
    self.appHost = nil;
    self.premiumThreads = nil;
    self.req = nil;
    [email release];
    [emailHost release];
    [name release];
    [stations release];
    [super dealloc];
}

@end
