//
//  CreateStationViewController.m
//  butterflyradio
//
//  Created by Denny Kwon on 6/8/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.


#import "CreateStationViewController.h"

@implementation UITextField (UITextFieldCategory)
+ (UITextField *)textFieldwithFrame:(CGRect)frame placeholder:(NSString *)p
{
    UITextField *textField = [[UITextField alloc] initWithFrame:frame];
    textField.contentVerticalAlignment = UIControlContentVerticalAlignmentCenter;
    textField.borderStyle = UITextBorderStyleNone;
    textField.placeholder = p;
    textField.font = [UIFont fontWithName:@"Heiti SC" size:14.0f];
    return [textField autorelease];
}
@end


@interface CreateStationViewController ()
@property (retain, nonatomic) NSArray *categories;
@end

@implementation CreateStationViewController
@synthesize nameField;
@synthesize categoryLabel;
@synthesize tagsField;
@synthesize delegate;
@synthesize categoriesPicker;

- (id)init
{
    self = [super init];
    if (self) {
        [self configure];
    }
    return self;
}

- (id)initWithManager:(ButterflyManager *)mgr
{
    self = [super initWithManager:mgr];
    if (self) {
        [self configure];
    }
    return self;
}

- (void)configure
{
    self.title = @"New Station";
    self.hidesBottomBarWhenPushed = YES;
    self.categories = kCategories;
}

- (void)dealloc
{
    self.delegate = nil;
    self.categoriesPicker = nil;
    self.categories = nil;
    self.nameField = nil;
    [categoryLabel release];
    [tagsField release];
    [super dealloc];
}



- (void)loadView
{
    UIView *view = [self baseViewWithColor:[UIColor colorWithRed:0.95f green:0.95f blue:0.95f alpha:1.0f]];
    CGRect frame = view.frame;
    view.tag = 1111;
	
    DetailBackground *nameBg = [DetailBackground backgroundWithFrame:CGRectMake(0, 0, 320, 35)];
    [nameBg.btnLabel setTitle:@"Name" forState:UIControlStateNormal];
    self.nameField = [UITextField textFieldwithFrame:CGRectMake(80, 5, 225, 25) placeholder:@"name"];
    nameField.borderStyle = UITextBorderStyleNone;
    nameField.delegate = self;
    [nameBg addSubview:nameField];
    [view addSubview:nameBg];
    
    DetailBackground *tagsBg = [DetailBackground backgroundWithFrame:CGRectMake(0, 35, 320, 35)];
    [tagsBg.btnLabel setTitle:@"Tags" forState:UIControlStateNormal];
    self.tagsField = [UITextField textFieldwithFrame:CGRectMake(80, 5, 225, 25) placeholder:@"tags (seprated by commas)"];
    tagsField.borderStyle = UITextBorderStyleNone;
    tagsField.delegate = self;
    [tagsBg addSubview:tagsField];
    [view addSubview:tagsBg];

    
    DetailBackground *categoryBg = [DetailBackground backgroundWithFrame:CGRectMake(0, 70, 320, 35)];
    [categoryBg.btnLabel setTitle:@"Category" forState:UIControlStateNormal];
    categoryLabel = [[UILabel alloc] initWithFrame:CGRectMake(80, 5, 225, 25)];
    categoryLabel.userInteractionEnabled = YES;
    categoryLabel.font = [UIFont fontWithName:kFont size:14.0f];
    categoryLabel.text = @"misc";
    categoryLabel.backgroundColor = [UIColor clearColor];
    [categoryBg addSubview:categoryLabel];
    [view addSubview:categoryBg];

    
    UIImage *shadow = [UIImage imageNamed:@"dropShadow.png"];
    UIImageView *dropShadow = [[UIImageView alloc] initWithImage:shadow];
    dropShadow.frame = CGRectMake(0, categoryBg.frame.origin.y+categoryBg.frame.size.height, shadow.size.width, shadow.size.height);
    [view addSubview:dropShadow];
    [dropShadow release];


    categoriesPicker = [[UIPickerView alloc] initWithFrame:CGRectMake(0, frame.size.height, frame.size.width, 150)];
    self.categoriesPicker.showsSelectionIndicator = YES;
    self.categoriesPicker.delegate = self;
    self.categoriesPicker.dataSource = self;
    [view addSubview:self.categoriesPicker];
    
    self.view = view;
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
    [textField resignFirstResponder];
    if (self.categoriesPicker.frame.origin.y < self.view.frame.size.height)
        [self slidePicker:self.view.frame.size.height];

    return TRUE;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    [self setUpBackButton:self action:@selector(exit)];
    [self setupLoadingIndicator];

    UIBarButtonItem *btnCreateStation = [[UIBarButtonItem alloc] initWithTitle:@"create" style:UIBarButtonItemStyleBordered target:self action:@selector(create)];
    btnCreateStation.tintColor = [UIColor greenColor];
    self.navigationItem.rightBarButtonItem = btnCreateStation;
    [btnCreateStation release];
}

- (void)exit
{
    [self.navigationController popViewControllerAnimated:YES];
}

- (void)selectCategory
{
    NSLog(@"selectCategory");
    [self.nameField resignFirstResponder];
    [self.tagsField resignFirstResponder];

    [self slidePicker:self.view.frame.size.height-categoriesPicker.frame.size.height];
}


- (void)create
{
    NSLog(@"CREATE: %@", self.butterflyMgr.appHost);
    if ([nameField.text length]==0 || [tagsField.text length]==0){
        [self showAlert:@"Missing Value" message:@"Please complete all fields."];
        return;
    }
    
    NSString *url = [NSString stringWithFormat:@"http://%@/api/station", kUrl];

    NSString *hidden = @"no";
    if (self.butterflyMgr.host.guest) //set stations as initially hidden when coming from a third party app.
        hidden = @"yes";
    
    NSMutableDictionary *params = [NSMutableDictionary dictionaryWithObjectsAndKeys:hidden, @"hidden", nameField.text, @"name", self.butterflyMgr.host.email, @"email", tagsField.text, @"tags", categoryLabel.text, @"category", @"create", @"action", nil];

    if (self.butterflyMgr.host.guest){
        NSLog(@"SEND APPROVAL REQUEST TO %@", self.butterflyMgr.appHost);
        NSString *appHost = [NSString stringWithFormat:@"%@", self.butterflyMgr.appHost];
        [params setObject:appHost forKey:@"appHost"];
    }
    
    if (self.request!=nil){
        [self.request cancel];
        self.request.delegate = nil;
        [self.request release];
    }

    self.request = [BRNetworkOp operationWithAddress:url parameters:params];
    self.request.delegate = self;
    [self.request sendRequest];
    [self.loading show];
}

- (void)requestData:(NSArray *)pkg //returns [address, data]
{
    if (pkg==nil)
        return;
    
    id json = [self.request parseJsonResponse];
    if (!json)
        return;
    
    [self.loading hide];
    NSDictionary *d = [json objectForKey:@"results"];
    NSLog(@"%@ - %@", [[self class] description], [d description]);
    NSString *success = [d objectForKey:@"success"];
    if (![success isEqualToString:@"yes"])
        return;
    
    NSDictionary *hostInfo = [d objectForKey:@"host"];
    [self.butterflyMgr.host populate:hostInfo];
    
    NSDictionary *stationInfo = [d objectForKey:@"station"];
    if (stationInfo){
        Station *newStation = [Station stationWithInfo:stationInfo];
        if ([self.butterflyMgr.host.stations containsObject:@"none"]){
            self.butterflyMgr.host.stations = @[newStation];
        }
        else{
            NSMutableArray *stations = [NSMutableArray arrayWithArray:self.butterflyMgr.host.stations];
            [stations addObject:newStation];
            self.butterflyMgr.host.stations = stations;
        }
    }
    
    [self.navigationController popViewControllerAnimated:YES];
}

- (void)slidePicker:(CGFloat)pos
{
    CGRect frame = self.categoriesPicker.frame;
    frame.origin.y = pos;
    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDuration:0.4];
    self.categoriesPicker.frame = frame;
    [UIView commitAnimations];
}

#pragma mark -  UIPickerViewDataSource
- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView
{
    return 1;
}

- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component
{
    return [self.categories count];
}

- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component
{
    return [self.categories objectAtIndex:row];
}

- (void)pickerView:(UIPickerView *)pickerView didSelectRow:(NSInteger)row inComponent:(NSInteger)component
{
    NSString *category = [self.categories objectAtIndex:row];
    categoryLabel.text = category;
    [self slidePicker:self.view.frame.size.height];
}


#pragma mark - UIResponder Touch Methods
-(void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
    [super touchesEnded:touches withEvent:event];

	UITouch *tchd = [touches anyObject];
	if (tchd.view.tag==1111){
		[nameField resignFirstResponder];
		[tagsField resignFirstResponder];
	}
    
    if (self.categoriesPicker.frame.origin.y < self.view.frame.size.height){
        [self slidePicker:self.view.frame.size.height];
        return;
    }
    
    if ([tchd.view isEqual:categoryLabel]){
        (categoriesPicker.frame.origin.y > 254.0f) ? [self slidePicker:254.0f] : [self slidePicker:self.view.frame.size.height];
    }


}

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
	
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
	
}

- (void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event
{
	
}





- (void)viewDidUnload
{
    [super viewDidUnload];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}


@end
