//
//  HostViewController.m
//  butterflyradio
//
//  Created by Denny Kwon on 6/3/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "HostViewController.h"
#import "BRStationViewController.h"
#import "BRRecordViewController.h"
#import "BRPurchaseViewController.h"

@interface HostViewController ()
@property (nonatomic) CGPoint currentOffset;
@property (nonatomic) int deleteIndex;
@property (nonatomic) BOOL goToBtfly;
@property (retain, nonatomic) UIBarButtonItem *btnTickets;
@property (retain, nonatomic) BRTextBox *descriptionBox;
@end


@implementation HostViewController
@synthesize theTableview;

- (id)init
{
    self = [super init];
    if (self) {
        [self configure];
    }
    return self;
}

- (id)initWithManager:(ButterflyManager *)mgr
{
    self = [super initWithManager:mgr];
    if (self) {
        [self configure];
    }
    return self;
}

- (void)configure
{
    isLoading = FALSE;
    self.deleteIndex = -1;
    self.title = @"Stations";
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(refresh)
                                                 name:kRefreshNotification
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(refresh)
                                                 name:kThumbnailReadyNotification
                                               object:nil];
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self
                                                    name:kRefreshNotification
                                                  object:nil];
    
    [[NSNotificationCenter defaultCenter] removeObserver:self
                                                    name:kThumbnailReadyNotification
                                                  object:nil];
    
    self.btnTickets = nil;
    self.descriptionBox = nil;
    [theTableview release];
    [super dealloc];
}

- (void)loadView
{
    UIView *view = [self baseViewWithColor:[UIColor colorWithPatternImage:[UIImage imageNamed:@"bg_magicalmint.png"]]];

    CGRect frame = view.frame;
    frame.origin.y = 0;
    
    theTableview = [[UITableView alloc] initWithFrame:frame style:UITableViewStylePlain];
    self.theTableview.autoresizingMask = view.autoresizingMask;
    self.theTableview.backgroundColor = [UIColor clearColor];
    self.theTableview.separatorStyle = UITableViewCellSeparatorStyleNone;
    self.theTableview.dataSource = self;
    self.theTableview.delegate = self;
    self.theTableview.showsVerticalScrollIndicator = NO;
    [view addSubview:self.theTableview];
    [self addPullToRefreshHeader:self.theTableview];
    
    self.descriptionBox = [[[BRTextBox alloc] initWithFrame:CGRectMake(20.0f, 180.0f, frame.size.width-40.0f, 10.0f)] autorelease];
    self.descriptionBox.lblText.text = [NSString stringFromFile:@"intro.txt"];
    self.descriptionBox.lblTitle.text = @"Welcome";
    [view addSubview:self.descriptionBox];
    
    
    self.btnTickets = [self barButtonWithText:[NSString stringWithFormat:@"tickets: %d", self.butterflyMgr.host.tickets] tag:1002];
    NSArray *barButtons = @[[self flex], [self barButtonWithText:@"log out" tag:1000], [self flex], [self barButtonWithText:@"new station" tag:1001], [self flex], self.btnTickets, [self flex]];
    
    CGFloat height = 56.0f;
    self.bottomToolbar = [BRToolbar toolbar:CGRectMake(0, frame.size.height-height+7, frame.size.width, height) withItems:barButtons];
    [view addSubview:self.bottomToolbar];

    self.view = view;
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];

    [self.theTableview deselectRowAtIndexPath:[theTableview indexPathForSelectedRow] animated:YES];
    [self.theTableview reloadData];
    
    if ([self.butterflyMgr.host.stations containsObject:@"none"]==TRUE || [self.butterflyMgr.host.stations count]==0){
        self.theTableview.scrollEnabled = NO;
        self.goToBtfly = FALSE;
        self.descriptionBox.hidden = NO;
        return;
    }
    
    self.theTableview.scrollEnabled = YES;
    self.descriptionBox.hidden = YES;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    [self setupNowPlayingButton];
    [self setUpBackButton:self action:@selector(exit)];
    [self setupLoadingIndicator:1 verticalOffset:160.0f];
    [self.btnTickets setTitle:[NSString stringWithFormat:@"tickets: %d", self.butterflyMgr.host.tickets]];

    if ([self.butterflyMgr.host.stations containsObject:@"none"]==TRUE || [self.butterflyMgr.host.stations count]==0){
        self.theTableview.scrollEnabled = NO;
        self.goToBtfly = FALSE;
        self.descriptionBox.hidden = NO;
        return;
    }
    
    self.theTableview.scrollEnabled = YES;
    self.descriptionBox.hidden = YES;
}


#pragma mark -
- (void)exit
{
    [self dismissViewControllerAnimated:YES completion:NULL];
}

- (void)barButtonTapped:(UIBarButtonItem *)btn
{
    NSLog(@"barButtonTapped: %d", btn.tag);
    if (btn.tag==1000){ // log out
        [self logout];
        return;
    }
    
    if (btn.tag==1001){ // create new station
        [self createStation];
        return;
    }

    if (btn.tag==1002){
        [self launchPurchaseSequence];
        return;
    }
}

- (void)logout
{
    NSArray *cookies = [NSHTTPCookieStorage sharedHTTPCookieStorage].cookies;
    for(NSHTTPCookie *cookie in cookies) {
        [[NSHTTPCookieStorage sharedHTTPCookieStorage] deleteCookie:cookie];
    }
    
    self.butterflyMgr.host.loggedIn = NO;
    UIViewController *root = [self.navigationController.viewControllers objectAtIndex:0];
    ([root isEqual:self]) ? [self dismissViewControllerAnimated:YES completion:NULL] : [self.navigationController popViewControllerAnimated:YES];
}


- (void)createStation
{
    CreateStationViewController *create = [[CreateStationViewController alloc] initWithManager:self.butterflyMgr];
    [self.navigationController pushViewController:create animated:YES];
    [create release];
}


- (void)updateInfo
{
    //this is override from superlcass. it is called by pull to refresh spinner 
    NSLog(@"%@ - updateInfo", [[self class] description]);
    [self.butterflyMgr.host refresh];
}

- (void)refresh
{
    NSLog(@"%@ - REFRESH!!", [[self class] description]);
    if (isLoading)
        [self stopLoading:theTableview];

    [theTableview reloadData];
    self.descriptionBox.hidden = ([self.butterflyMgr.host.stations containsObject:@"none"]==TRUE || [self.butterflyMgr.host.stations count]==0)? NO : YES;
    
    if (!self.butterflyMgr.host.guest){
        NSLog(@"%@ - REFRESH!!", [[self class] description]);
//        [self updateInfo];
        return;
    }
    
    self.goToBtfly = TRUE;
    NSString *msg = [NSString stringWithFormat:@"Your station has been submitted to the %@ App. If approved, it will appear on the app home page shortly. If you would like to begin managing the station now, you may do so through the Butterfly Radio iPhone App.", self.butterflyMgr.appName];
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Station Submitted" message:msg delegate:self cancelButtonTitle:@"ok" otherButtonTitles:@"Butterfly Radio", nil];
    [alert show];
    [alert release];
}

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    NSLog(@"alertView clickedButtonAtIndex: %d", buttonIndex);
    if (buttonIndex==0){
        if (self.deleteIndex >= 0){
            if (self.request){
                self.request.delegate = nil;
                [self.request cancel];
                self.request = nil;
            }
            
            Station *s = (Station *)[self.butterflyMgr.host.stations objectAtIndex:self.deleteIndex];
            
            if ([s.host isEqualToString:self.butterflyMgr.host.email]==TRUE){ //standard delete
                NSString *url = [NSString stringWithFormat:@"http://%@/api/station/%@", kUrl, s.unique_id];
                NSLog(@"URL = %@", url);
                self.request = [BRNetworkOp operationWithAddress:url parameters:nil];
                [self.request setHttpMethod:@"DELETE"];
            }
            else { // remove self.host from admin list, do not fully delete station. This would happen if the school is removing a substation from its list.  the station itself is not erased but it no longer shows up on the school list.
                
                /* REQUIRED PARAMS:
                 String name = req.getParameter("name");
                 String email = req.getParameter("email");
                 String tags = req.getParameter("tags");
                 String category = req.getParameter("category");
                 String action = req.getParameter("action");
                 */
                
                NSString *url = [NSString stringWithFormat:@"http://%@/api/station", kUrl];
                NSLog(@"URL = %@", url);
                NSDictionary *params = [NSDictionary dictionaryWithObjectsAndKeys:@"removeAdmin", @"action", self.butterflyMgr.host.email, @"admin", s.unique_id, @"station", s.name, @"name", [s tagsString], @"tags", s.category, @"category", s.host, @"email", nil];
                
                //req = [[BRNetworkOp alloc] initWithAddress:url parameters:params];
                self.request = [BRNetworkOp operationWithAddress:url parameters:params];
            }
            
            
            self.request.delegate = self;
            [self.request sendRequest];
            [self.loading show];
        }
    }
    else {
        if (self.goToBtfly==TRUE)
            [[UIApplication sharedApplication] openURL:[NSURL URLWithString:@"http://itunes.apple.com/us/app/butterfly-radio/id532051737?mt=8"]];

    }
}

- (void)requestData:(NSArray *)pkg //returns [address, data]
{
    if (!pkg)
        return;
    
    [self.loading hide];
    NSDictionary *jsonResponse = [self.request parseJsonResponse];
    NSLog(@"%@ - requestData: %@", [[self class] description], [jsonResponse description]);
    
    NSString *address = [pkg objectAtIndex:0];
    if ([address hasPrefix:[NSString stringWithFormat:@"http://%@/api/station", kUrl]]==TRUE){
        self.request.delegate = nil;
        self.request = nil;
        
        NSDictionary *results = [jsonResponse objectForKey:@"results"];
        NSDictionary *updatedHostInfo = [results objectForKey:@"host"];
        if (updatedHostInfo){
            [self.butterflyMgr.host populate:updatedHostInfo];
            
            NSArray *stations = [updatedHostInfo objectForKey:@"stations"];
            NSMutableArray *updatedStations = [NSMutableArray array];
            if (!stations){
                [self.theTableview reloadData];
                return;
            }
            
            if ([stations containsObject:@"none"]==NO){
                for (Station *s in self.butterflyMgr.host.stations){
                    for (NSString *strStation in stations) {
                        if ([strStation rangeOfString:s.unique_id].location != NSNotFound){
                            [updatedStations addObject:s];
                            break;
                        }
                    }
                }
                self.butterflyMgr.host.stations = updatedStations;
                [self.theTableview reloadData];
                return;
            }
            
            [updatedStations addObject:@"none"];
            self.butterflyMgr.host.stations = updatedStations;
            self.descriptionBox.hidden = NO;
            self.theTableview.scrollEnabled = NO;
            [self.theTableview reloadData];
        }
    }
    
    if (jsonResponse==nil){
        [self.request sendRequest];
        return;
    }
    
    NSDictionary *d = [jsonResponse objectForKey:@"results"];
    NSLog(@"TEST - %@", [d description]);
    NSString *confirmation = [d objectForKey:@"confirmation"];
    if ([confirmation isEqualToString:@"found"]){
        NSDictionary *hostInfo = [d objectForKey:@"host"];
        [self.butterflyMgr.host populate:hostInfo];
        [theTableview reloadData];
    }
    
    self.theTableview.scrollEnabled = ([self.butterflyMgr.host.stations containsObject:@"none"]) ? NO : YES;
}


- (void)deleteContent:(int)index
{
    self.deleteIndex = index - 1000;
    self.goToBtfly = FALSE;
    
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Are You Sure?" message:@"This will permanently remove the station from your profile." delegate:self cancelButtonTitle:@"yes" otherButtonTitles:@"no", nil];
    [alert show];
    [alert release];
}



#pragma mark - StationDelegate
- (void)imageReady:(NSString *)addr
{
    [theTableview reloadData];
}

- (void)stationInfoReady:(NSString *)stationId
{
    
}



#pragma mark - UITableViewStuff
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    if ([self.butterflyMgr.host.stations count]==0)
        return 0;

    if ([self.butterflyMgr.host.stations containsObject:@"none"]==TRUE)
        return 0;
    
    return self.butterflyMgr.host.stations.count;
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *cellID = @"ID";
    AdminCell *cell = (AdminCell *)[theTableview dequeueReusableCellWithIdentifier:cellID];
    if (cell==nil){
        cell = [[[AdminCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:cellID] autorelease];
        [cell setup];
        cell.delegate = self;
    }
    
    if ([self.butterflyMgr.host.stations containsObject:@"none"]==TRUE){
        cell.nameLabel.text = @"no stations";
        return cell;
    }

    Station *station = (Station *)[self.butterflyMgr.host.stations objectAtIndex:indexPath.row];
    int tag = (indexPath.row+1000);
    cell.tag = tag;
    cell.nameLabel.text = station.name;
    cell.categoryLabel.text = station.category;
    
    if (station.thumbnail==nil){
        [station fetchThumbnail:240];
        return cell;
    }
    
    
    [cell fillImage:station.thumbnail];
    return cell;

}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return [AdminCell rowHeight];
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    NSLog(@"tableView didSelectRowAtIndexPath:");
    Station *station = (Station *)[self.butterflyMgr.host.stations objectAtIndex:indexPath.row];
    if (!station){
        [theTableview deselectRowAtIndexPath:[theTableview indexPathForSelectedRow] animated:YES];
        return;
    }
    
    StationDetailsViewController *details = [[StationDetailsViewController alloc] initWithManager:self.butterflyMgr];
    details.station = station;
    [self.navigationController pushViewController:details animated:YES];
    [details release];
}



#pragma mark - AdminCellDelegate
- (void)buttonAction:(int)tag title:(NSString *)title
{
    NSLog(@"buttonAction:%d title:%@", tag, title);
    if ([title isEqualToString:@"delete"]){
        [self deleteContent:tag];
        return;
    }


    tag -= 1000;
    
    if ([self.butterflyMgr.host.stations containsObject:@"none"]==TRUE){
        [theTableview deselectRowAtIndexPath:[theTableview indexPathForSelectedRow] animated:YES];
        return;
    }
    
    
    Station *station = (Station *)[self.butterflyMgr.host.stations objectAtIndex:tag];
    if ([title isEqualToString:@"details"]){
        StationDetailsViewController *details = [[StationDetailsViewController alloc] initWithManager:self.butterflyMgr];
        details.station = station;
        [self.navigationController pushViewController:details animated:YES];
        [details release];
        return;
    }
    
    int mode;
    if ([title isEqualToString:@"tracks"]){
        mode = 0;
        if ([station.tracks containsObject:@"none"]==TRUE || [station.tracks count]==0){
            [self showAlert:@"No Tracks" message:@"This station has no tracks."];
            return;
        }
    }
    
    if ([title isEqualToString:@"news"]){
        mode = 1;
        if ([station.articles containsObject:@"none"]==TRUE || [station.articles count]==0){
            [self showAlert:@"No Articles" message:@"This station has no articles."];
            return;
        }
    }
    
    if ([title isEqualToString:@"admins"])
        mode = 2;

    
    StationAdminViewController *admin = [[StationAdminViewController alloc] initWithManager:self.butterflyMgr];
    admin.host = self.butterflyMgr.host;
    admin.mode = mode;
    admin.title = station.name;
    admin.station = station;
    
    [self.navigationController pushViewController:admin animated:YES];
    [admin release];
}

- (void)postPremiumTrack:(int)tag
{
    tag -= 1000;
    if (tag < 0)
        return;
    
    Station *station = (Station *)[self.butterflyMgr.host.stations objectAtIndex:tag];
    if (!station)
        return;

    NSLog(@"postPremiumTrack: %@", station.name);
    BRRecordViewController *submitTrack = [[BRRecordViewController alloc] initWithManager:self.butterflyMgr];
    submitTrack.station = station;
    submitTrack.premium = YES;
    UINavigationController *nav = [[UINavigationController alloc] initWithRootViewController:submitTrack];
    nav.navigationBar.barStyle = UIBarStyleBlack;
    [submitTrack release];
    
    nav.modalTransitionStyle = UIModalTransitionStyleFlipHorizontal;
    [self presentModalViewController:nav animated:YES];
    [nav release];

}



#pragma mark - UIScrollViewDelegate
- (void)scrollViewDidScroll:(UIScrollView *)scrollView
{
    [super scrollViewDidScroll:scrollView];
    
    double delta = self.theTableview.contentOffset.y - self.currentOffset.y;
    
    if (delta==0) // weird things happen when delta is 0 so just ignore it
        return;
    
    if (delta > 0){ // scrolling up
        [self.navigationController setNavigationBarHidden:YES animated:YES];
        
        CGRect frame = self.bottomToolbar.frame;
        frame.origin.y = self.view.frame.size.height;
        self.bottomToolbar.frame = frame;
        return;
    }
    
    [self.navigationController setNavigationBarHidden:NO animated:YES];
    
    CGRect frame = self.bottomToolbar.frame;
    frame.origin.y = self.view.frame.size.height-frame.size.height+7;
    self.bottomToolbar.frame = frame;
}

- (void)scrollViewWillBeginDragging:(UIScrollView *)scrollView
{
    [super scrollViewWillBeginDragging:scrollView];
    
    CGFloat offsetY = self.theTableview.contentOffset.y;
    if (offsetY < 0)
        return;
    
    if (offsetY > (self.theTableview.contentSize.height-self.view.frame.size.height))
        return;
    
    self.currentOffset = self.theTableview.contentOffset;
}



- (void)viewDidUnload
{
    [super viewDidUnload];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

@end
