//  LoginViewController.m
//  butterflyradio
//  Created by Denny Kwon on 6/2/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.


#import "LoginViewController.h"

@interface LoginViewController ()
@property (retain, nonatomic) UIImageView *bgImage;
@property (retain, nonatomic) UIView *mainView;
@end


@implementation LoginViewController
@synthesize theWebview;
@synthesize btnYahoo;
@synthesize btnGoogle;
@synthesize url;
@synthesize loginOnAppear;
@synthesize titleLabel;


- (id)init
{
    self = [super init];
    if (self) {
        [self configure];
    }
    return self;
}

- (id)initWithManager:(ButterflyManager *)mgr
{
    self = [super initWithManager:mgr];
    if (self) {
        [self configure];
    }
    return self;
}

- (void)configure
{
    self.loginOnAppear = FALSE;
    self.title = @"Account";
    self.tabBarItem.image = [UIImage imageNamed:@"tab_account.png"];
}


- (void)dealloc
{
    self.bgImage = nil;
    self.mainView = nil;
    self.btnGoogle = nil;
    self.btnYahoo = nil;
    self.url = nil;
    
    [titleLabel release];
    [super dealloc];
}

- (void)loadView
{
    UIView *view = [self baseViewWithColor:[UIColor colorWithPatternImage:[UIImage imageNamed:@"bg_login.png"]]];
    CGRect frame = view.frame;
    
    CGFloat y = 65.0f;

    titleLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, y, frame.size.width, 26.0f)];
    self.titleLabel.text = @"Login or Register";
    self.titleLabel.textAlignment = UITextAlignmentCenter;
    self.titleLabel.backgroundColor = [UIColor clearColor];
    self.titleLabel.font = [UIFont fontWithName:kFont size:20.0f];
    self.titleLabel.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    self.titleLabel.textColor = kBRRed;
    self.titleLabel.shadowColor = [UIColor lightGrayColor];
    self.titleLabel.shadowOffset = CGSizeMake(-0.5, 0.5f);
    [view addSubview:self.titleLabel];

    y += self.titleLabel.frame.size.height+16.0f;
    self.btnGoogle = [UIButton buttonWithType:UIButtonTypeCustom];
    UIImage *imgGoogle = [UIImage imageNamed:@"login_google.png"];
    self.btnGoogle.frame = CGRectMake(20, y, imgGoogle.size.width, imgGoogle.size.height);
    self.btnGoogle.center = CGPointMake(0.35f*frame.size.width, self.btnGoogle.center.y);
    [btnGoogle setBackgroundImage:imgGoogle forState:UIControlStateNormal];
    btnGoogle.showsTouchWhenHighlighted = YES;
    [btnGoogle addTarget:self action:@selector(btnPressed:) forControlEvents:UIControlEventTouchUpInside];
    [btnGoogle setTitle:@"google" forState:UIControlStateNormal];
    [btnGoogle setTitleColor:[UIColor clearColor] forState:UIControlStateNormal];
    [view addSubview:btnGoogle];

    self.btnYahoo = [UIButton buttonWithType:UIButtonTypeCustom];
    UIImage *imgYahoo = [UIImage imageNamed:@"login_yahoo.png"];
    self.btnYahoo.frame = CGRectMake(180, y, imgYahoo.size.width, imgYahoo.size.height);
    self.btnYahoo.center = CGPointMake(0.65f*frame.size.width, self.btnYahoo.center.y);
    [btnYahoo setBackgroundImage:imgYahoo forState:UIControlStateNormal];
    btnYahoo.showsTouchWhenHighlighted = YES;
    [btnYahoo addTarget:self action:@selector(btnPressed:) forControlEvents:UIControlEventTouchUpInside];
    [btnYahoo setTitle:@"yahoo" forState:UIControlStateNormal];
    [btnYahoo setTitleColor:[UIColor clearColor] forState:UIControlStateNormal];
    [view addSubview:btnYahoo];
    
    
    frame = view.frame;
    frame.origin.y = 0.0f;
    theWebview = [[UIWebView alloc] initWithFrame:frame];
    self.theWebview.autoresizingMask = UIViewAutoresizingFlexibleHeight;
    self.theWebview.delegate = self;
    self.theWebview.hidden = YES;
    [view addSubview:self.theWebview];
    
    self.view = view;
}


- (void)viewDidLoad
{
    [super viewDidLoad];
    
    [self setUpBackButton:self action:@selector(exit)];
    [self setupNowPlayingButton];
    [self setupLoadingIndicator:2 verticalOffset:360.0f];
}


- (void)viewWillAppear:(BOOL)animated
{
    if (!self.loginOnAppear)
        return;
    
    
    if (!self.url)
        return;
    
    self.loginOnAppear = FALSE;
    [self login:self.url];
}

- (void)viewWillDisappear:(BOOL)animated
{
    theWebview.hidden = YES;
}

- (void)exit
{
    [self dismissViewControllerAnimated:YES completion:NULL];
}

- (void)login:(NSString *)addr
{
    if (self.request){
        self.request.delegate = nil;
        [self.request cancel];
        self.request = nil;
    }

    self.request = [BRNetworkOp operationWithAddress:addr parameters:nil];
    [self.request setHttpMethod:@"GET"];
    self.request.delegate = self;
    [self.request sendRequest];

    [self.loading show];
}

- (void)btnPressed:(UIButton *)btn
{
    NSString *domain = btn.titleLabel.text;
    self.url = [NSString stringWithFormat:@"http://%@/api/login?host=%@", kUrl, domain];
    [self login:self.url];
}

- (void)requestData:(NSArray *)pkg //returns [address, data]
{
    if (!pkg)
        return;
    
    NSDictionary *json = [self.request parseJsonResponse];

    if (json==nil){
        NSLog(@"LOGIN VIEW CONTROLLER - JSON ERROR");
        if (self.loading.hidden)
            [self.loading hide];
        
        [self showAlert:@"Error" message:@"Butterfly Radio is currently experiencing technical diffifulties. Please try again later."];
        return;
    }
    
    NSDictionary *d = [json objectForKey:@"results"];
    NSLog(@"%@", [d description]);
    
    NSString *loggedIn = [d objectForKey:@"logged in"];
    if ([loggedIn isEqualToString:@"yes"]==TRUE){
        NSLog(@"LOGGED IN");
        theWebview.hidden = YES;
        [self.loading hide];
        NSString *found = [d objectForKey:@"found"];
        if ([found isEqualToString:@"yes"]){ //user had a profile - route to admin page
            NSDictionary *info = [d objectForKey:@"host"];
            NSString *email = [d objectForKey:@"email"];
            
            self.butterflyMgr.host.guest = ![email isEqualToString:self.butterflyMgr.appHost];
            if ([self.butterflyMgr.appHost isEqualToString:[NSString stringWithFormat:@"%@", kGeneralEmail]])
                self.butterflyMgr.host.guest = NO;
            
            [self.butterflyMgr.host populate:info];
            
            HostViewController *hostVC = [[HostViewController alloc] initWithManager:self.butterflyMgr];
            [self.navigationController pushViewController:hostVC animated:YES];
            [hostVC release];
            
        }
        else{ // user logged in but does not have profile - route to sign up page
            NSLog(@"NEW USER!");
            RegisterViewController *signup = [[RegisterViewController alloc] initWithManager:self.butterflyMgr];
            signup.delegate = self;
            signup.emailHost = [d objectForKey:@"emailhost"];
            [self.navigationController pushViewController:signup animated:YES];
            signup.emailField.text = [[d objectForKey:@"email"] lowercaseString];
            [signup release];
        }
        return;
    }
    
    NSLog(@"NOT LOGGED IN");
    NSString *redirect = [d objectForKey:@"login redirect"];
    [theWebview loadRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:redirect]]];
}

- (void)requestError:(NSError *)error
{
    [self.loading hide];
    [self showAlert:@"Error" message:@"There was an error. Please try again."];
}

- (void)registrationComplete
{
    self.loginOnAppear = TRUE;
}


#pragma mark - UIWebViewDelegate
- (BOOL)webView:(UIWebView *)webView shouldStartLoadWithRequest:(NSURLRequest *)request navigationType:(UIWebViewNavigationType)navigationType
{
    BOOL shouldContinue = TRUE;
    NSString *address = request.URL.absoluteString;
    NSLog(@"webView shouldStartLoadWithRequest: %@", address);
    if ([address isEqualToString:self.url]){
        shouldContinue = FALSE;
        [self login:self.url];
    }
    else {
        [self.loading show];
    }
    return shouldContinue;
}

- (void)webViewDidStartLoad:(UIWebView *)webView
{
    NSLog(@"webViewDidStartLoad:");
    if (self.loading.hidden==TRUE)
        [self.loading show];

}

- (void)webViewDidFinishLoad:(UIWebView *)webView
{
    NSLog(@"webViewDidFinishLoad:");
    self.theWebview.alpha = 0.0f;
    self.theWebview.hidden = NO;
    
    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDuration:0.4f];
    self.theWebview.alpha = 1.0f;
    [UIView commitAnimations];
    [self.loading hide];
}

- (void)webView:(UIWebView *)webView didFailLoadWithError:(NSError *)error
{
    NSLog(@"webView didFailLoadWithError:");
}

- (void)viewDidUnload
{
    [super viewDidUnload];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

@end
