//
//  StationDetailsViewController.m
//  butterflyradio
//
//  Created by Denny Kwon on 6/4/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "StationDetailsViewController.h"
#import <QuartzCore/QuartzCore.h>

@implementation UIImage (scale)
+ (UIImage*)imageWithImage:(UIImage*)img scaledToSize:(CGSize)newSize;
{
    UIGraphicsBeginImageContext( newSize );
    [img drawInRect:CGRectMake(0,0,newSize.width,newSize.height)];
    UIImage* newImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return newImage;
}
@end


@implementation UITextField (UITextFieldCategory)
+ (UITextField *)textFieldwithFrame:(CGRect)frame placeholder:(NSString *)p
{
    UITextField *textField = [[UITextField alloc] initWithFrame:frame];
    textField.contentVerticalAlignment = UIControlContentVerticalAlignmentCenter;
    textField.borderStyle = UITextBorderStyleNone;
    textField.placeholder = p;
    textField.font = [UIFont fontWithName:@"Heiti SC" size:14.0f];
    return [textField autorelease];
}
@end

@interface StationDetailsViewController ()
@property (retain, nonatomic) NSData *updatedImage;
@end

@implementation StationDetailsViewController
@synthesize station;
@synthesize nameField;
@synthesize tagsField;
@synthesize imgPicker;

- (CGRect)adjustFrame:(UIImage *)img;
{
    CGRect frame = image.frame;
    CGFloat width = img.size.width;
    CGFloat height = img.size.height;
    CGFloat max = (kMaxDimen-20);
    
    double scale;
    if (width>max){
        scale = max/width;
        width = max;
        height *= scale;
    }
    if (height>max){
        scale = max/height;
        height = max;
        width *= scale;
    }
    
    frame.size.width = width;
    frame.size.height = height;
    return frame;
}


- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        [self configure];
    }
    return self;
}

- (id)initWithManager:(ButterflyManager *)mgr
{
    self = [super initWithManager:mgr];
    if (self) {
        [self configure];
    }
    return self;
}

- (void)configure
{
    self.title = @"Details";
    self.hidesBottomBarWhenPushed = YES;
    categories = kCategories;
    queue = [[NSOperationQueue alloc] init];
    self.updatedImage = nil;
}

- (void)dealloc
{
    self.updatedImage = nil;
    station.delegate = nil;
    self.imgPicker = nil;
    self.nameField = nil;
    self.tagsField = nil;
    self.station = nil;
    
    [image release];
    [categoryLabel release];
    [categoriesPicker release];
    [categories release];
    [descriptionField release];
    [queue release];
    [super dealloc];
}

- (void)loadView
{
    UIView *view = [self baseViewWithColor:[UIColor colorWithRed:0.95f green:0.95f blue:0.95f alpha:1.0f]];
    CGRect frame = view.frame;
    
    descriptionField = [[UITextView alloc] initWithFrame:CGRectMake(10.0f, 115.0f, frame.size.width-20, 117.0f)];
    descriptionField.delegate = self;
    descriptionField.text = self.station.description;
    descriptionField.font = [UIFont fontWithName:@"Heiti SC" size:14.0f];
    descriptionField.backgroundColor = [UIColor yellowColor];
    descriptionField.delegate = self;
    [view addSubview:descriptionField];
    
    image = [[UIImageView alloc] initWithFrame:CGRectMake(10.0f, 240.0f, 100.0f, 100.0f)];
    image.userInteractionEnabled = YES;
    image.backgroundColor = [UIColor blackColor];
    image.layer.borderColor = [[UIColor blackColor] CGColor];
    image.layer.borderWidth = 1.0f;
    image.image = [UIImage imageNamed:@"Icon@2x.png"];
    [view addSubview:image];
    
    
    DetailBackground *nameBg = [DetailBackground backgroundWithFrame:CGRectMake(0, 0, 320, 35)];
    [nameBg.btnLabel setTitle:@"Name" forState:UIControlStateNormal];
    self.nameField = [UITextField textFieldwithFrame:CGRectMake(80, 5, 225, 25) placeholder:@"name"];
    self.nameField.borderStyle = UITextBorderStyleNone;
    self.nameField.delegate = self;
    self.nameField.text = self.station.name;
    [nameBg addSubview:self.nameField];
    [view addSubview:nameBg];
    
    DetailBackground *tagsBg = [DetailBackground backgroundWithFrame:CGRectMake(0, 35, 320, 35)];
    [tagsBg.btnLabel setTitle:@"Tags" forState:UIControlStateNormal];
    self.tagsField = [UITextField textFieldwithFrame:CGRectMake(80, 5, 225, 25) placeholder:@"tags"];
    self.tagsField.borderStyle = UITextBorderStyleNone;
    self.tagsField.delegate = self;
    self.tagsField.text = [self.station tagsString];
    [tagsBg addSubview:self.tagsField];
    [view addSubview:tagsBg];
    
    DetailBackground *categoryBg = [DetailBackground backgroundWithFrame:CGRectMake(0, 70, 320, 35)];
    [categoryBg.btnLabel setTitle:@"Category" forState:UIControlStateNormal];
    categoryLabel = [[UILabel alloc] initWithFrame:CGRectMake(80, 5, 225, 25)];
    categoryLabel.font = self.tagsField.font;
    categoryLabel.userInteractionEnabled = YES;
    categoryLabel.text = self.station.category;
    categoryLabel.backgroundColor = [UIColor clearColor];
    [categoryBg addSubview:categoryLabel];
    [view addSubview:categoryBg];
    
    UIImage *shadow = [UIImage imageNamed:@"dropShadow.png"];
    UIImageView *dropShadow = [[UIImageView alloc] initWithImage:shadow];
    dropShadow.frame = CGRectMake(0, categoryBg.frame.origin.y+categoryBg.frame.size.height, shadow.size.width, shadow.size.height);
    [view addSubview:dropShadow];
    [dropShadow release];
    
    CGFloat h = 45.0f;
    UIButton *btnUpdate = [UIButton buttonWithType:UIButtonTypeCustom];
    [btnUpdate addTarget:self action:@selector(update:) forControlEvents:UIControlEventTouchUpInside];
//    [btnUpdate setBackgroundImage:[UIImage imageNamed:@"button_perriwinkle.png"] forState:UIControlStateNormal];
    [btnUpdate setBackgroundImage:[UIImage imageNamed:@"button_aqua.png"] forState:UIControlStateNormal];
    [btnUpdate setTitle:@"Update" forState:UIControlStateNormal];
    [btnUpdate setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
    btnUpdate.titleLabel.font = [UIFont boldSystemFontOfSize:16.0f];
    btnUpdate.titleLabel.shadowColor = [UIColor blackColor];
    btnUpdate.titleLabel.shadowOffset = CGSizeMake(0.5f, 0.5f);
    btnUpdate.frame = CGRectMake(10.0f, frame.size.height-h-10.0f, frame.size.width-20.0f, h);
    btnUpdate.autoresizingMask = UIViewAutoresizingFlexibleTopMargin;
    [view addSubview:btnUpdate];
    
    categoriesPicker = [[UIPickerView alloc] initWithFrame:CGRectMake(0, frame.size.height, frame.size.width, 150)];
    categoriesPicker.showsSelectionIndicator = YES;
    categoriesPicker.delegate = self;
    categoriesPicker.dataSource = self;
    [view addSubview:categoriesPicker];

    
    self.view = view;
}




- (void)viewDidLoad
{
    [super viewDidLoad];
    [self setUpBackButton:self action:@selector(exit)];
    
    
//    UIBarButtonItem *btnUpdate = [[UIBarButtonItem alloc] initWithTitle:@"update" style:UIBarButtonItemStyleBordered target:self action:@selector(update:)];
//    btnUpdate.tintColor = [UIColor blueColor];
//    self.navigationItem.rightBarButtonItem = btnUpdate;
//    [btnUpdate release];
    
    [self setupLoadingIndicator];
    
    if (self.station.imgData==nil){
        self.station.delegate = self;
        [self.station fetchImage];
        [self.loading show];
        return;
    }
    
    [self imageReady:nil];
}

#pragma mark - UITextField
- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
    [textField resignFirstResponder];
    return TRUE;
}


- (void)exit
{
    [self.navigationController popViewControllerAnimated:YES];
}


- (void)slidePicker:(CGFloat)pos
{
    CGRect frame = categoriesPicker.frame;
    frame.origin.y = pos;
    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDuration:0.4];
    categoriesPicker.frame = frame;
    [UIView commitAnimations];
}

- (void)update:(UIBarButtonItem *)btn
{
    [self.nameField resignFirstResponder];
    [self.tagsField resignFirstResponder];
    [descriptionField resignFirstResponder];
    
    if (self.updatedImage==nil){
        NSString *url = [NSString stringWithFormat:@"http://%@/api/station", kUrl];
        ASIFormDataRequest *request = [[[ASIFormDataRequest alloc] initWithURL:[NSURL URLWithString:url]] autorelease];
        request.delegate = self;
        [request setPostValue:@"no" forKey:@"image"];
        [request setPostValue:self.station.unique_id forKey:@"station"];
        [request setPostValue:@"update" forKey:@"action"];
        [request setPostValue:self.nameField.text forKey:@"name"];
        [request setPostValue:self.tagsField.text forKey:@"tags"];
        [request setPostValue:descriptionField.text forKey:@"description"];
        [request setPostValue:categoryLabel.text forKey:@"category"];
        [request setPostValue:self.station.host forKey:@"email"];
        
        [request startAsynchronous];
    }
    else{
        //Get upload string first then POST image
        
        NSString *url = [NSString stringWithFormat:@"http://%@/api/upload?type=station", kUrl];
        self.request = [BRNetworkOp operationWithAddress:url parameters:nil];
        self.request.delegate = self;
        [self.request setHttpMethod:@"GET"];
        [self.request sendRequest];
    }
    self.loading.titleLabel.text = @"Updating Station...";
    [self.loading show];
}

- (void)photoBtnPressed
{
    UIActionSheet *actionsheet = [[UIActionSheet alloc] initWithTitle:@"Select Source" delegate:self cancelButtonTitle:@"cancel" destructiveButtonTitle:nil otherButtonTitles:@"photo library", @"take photo", nil];
    actionsheet.frame = CGRectMake(0, 150, 320, 100);
    actionsheet.actionSheetStyle = UIActionSheetStyleBlackOpaque;
    [actionsheet showInView:[UIApplication sharedApplication].keyWindow];
    [actionsheet release];
}

- (void)lauchCamera:(UIImagePickerControllerSourceType)sourceType
{
    NSLog(@"launchCamera");
    LauchImagePicker *camera = [[LauchImagePicker alloc] initWithTarget:self action:@selector(cameraReady:) sourceType:sourceType];
    [queue addOperation:camera];
    [camera release];
    
    self.loading.titleLabel.text = (sourceType==UIImagePickerControllerSourceTypeCamera) ? @"Launching Camera..." : @"Launching Photo Library...";
    [self.loading show];
}

- (void)cameraReady:(UIImagePickerController *)picker
{
    [self.loading hide];
    if (picker){
        self.imgPicker = picker;
        self.imgPicker.delegate = self;
        [self presentModalViewController:self.imgPicker animated:NO];
    }
}

- (void)requestData:(NSArray *)pkg //returns [address, data]
{
    if (!pkg)
        return;
    
    NSString *json = [[NSString alloc] initWithData:[pkg objectAtIndex:1] encoding:NSUTF8StringEncoding];
    NSDictionary *d = [json JSONValue];
    if (d==nil){
        [self.request sendRequest];
        return;
    }
    
    d = [d objectForKey:@"results"];
    NSLog(@"%@", [d description]);
    NSString *url = [d objectForKey:@"upload string"];
    
    ASIFormDataRequest *request = [[[ASIFormDataRequest alloc] initWithURL:[NSURL URLWithString:url]] autorelease];
    request.delegate = self;
    [request setPostValue:@"yes" forKey:@"image"];
    NSString *fileName = [self.station.unique_id stringByAppendingString:@".jpeg"];
    [request setData:self.updatedImage withFileName:fileName andContentType:@"image/jpeg" forKey:@"file"];
    [request setPostValue:self.station.unique_id forKey:@"station"];
    [request setPostValue:@"update" forKey:@"action"];
    [request setPostValue:self.nameField.text forKey:@"name"];
    [request setPostValue:self.tagsField.text forKey:@"tags"];
    [request setPostValue:descriptionField.text forKey:@"description"];
    [request setPostValue:categoryLabel.text forKey:@"category"];
    [request setPostValue:self.station.host forKey:@"email"];
    
    [request startAsynchronous];
    [json release];
    
    self.request.delegate = nil;
    self.request = nil;
}

#pragma mark -  UIActionSheetDelegate
- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex
{
    NSLog(@"actionSheet clickedButtonAtIndex: %d", buttonIndex);
    if (buttonIndex==2){
        
    }
    else{
        UIImagePickerControllerSourceType sourceType;
        if (buttonIndex==0){ sourceType = UIImagePickerControllerSourceTypePhotoLibrary; }
        if (buttonIndex==1){ sourceType = UIImagePickerControllerSourceTypeCamera; }
        [self lauchCamera:sourceType];
        
    }
}


#pragma mark -  StationDelegate
- (void)imageReady:(NSString *)addr
{
    [self.loading hide];
    self.station.thumbnail = nil;
    image.image = [UIImage imageWithData:self.station.imgData];;
    CGRect frame = [self adjustFrame:image.image];
    image.frame = frame;
    
    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDuration:0.6f];
    [UIView setAnimationTransition:UIViewAnimationTransitionFlipFromLeft forView:image cache:YES];
    image.frame = frame;
    [UIView commitAnimations];
}

- (void)stationInfoReady:(NSString *)stationId
{

}


#pragma mark -  UIPickerViewDataSource
- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView
{
    return 1;
}

- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component
{
    return [categories count];
}

- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component
{
    return [categories objectAtIndex:row];
}

- (void)pickerView:(UIPickerView *)pickerView didSelectRow:(NSInteger)row inComponent:(NSInteger)component
{
    NSString *category = [categories objectAtIndex:row];
    categoryLabel.text = category;
    [self slidePicker:self.view.frame.size.height];
}



#pragma mark - UITextViewDelegate
- (void)textViewDidBeginEditing:(UITextView *)textView
{
    [self.navigationController setNavigationBarHidden:TRUE animated:YES];

}

- (void)textViewDidEndEditing:(UITextView *)textView
{
    [self.navigationController setNavigationBarHidden:FALSE animated:YES];
}

- (BOOL)textView:(UITextView *)textView shouldChangeTextInRange:(NSRange)range replacementText:(NSString *)text
{
    BOOL confirmation = TRUE;
    if ([text isEqualToString:@"\n"]){
        confirmation = FALSE;
        [descriptionField resignFirstResponder];
    }
    return confirmation;
}


#pragma mark - ImagePickerDelegate
- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary *)info
{
    [picker dismissModalViewControllerAnimated:NO];
    NSLog(@"%@", [info description]);
    UIImage *img = [info objectForKey:@"UIImagePickerControllerEditedImage"];
    if (img==nil){ img = [info objectForKey:@"UIImagePickerControllerOriginalImage"]; }
    
    CGRect frame = [self adjustFrame:img];
    image.frame = frame;
    image.image = img;
    
    CGFloat w = img.size.width;
    CGFloat h = img.size.height;
    CGFloat max = 640.0f;
    double scale;
    if (w>max){
        scale = max/w;
        w = max;
        h *= scale;
    }
    if (h>max){
        scale = max/h;
        h = max;
        w *= scale;
    }
    
    img = [UIImage imageWithImage:img scaledToSize:CGSizeMake(w, h)];
    
    self.updatedImage = UIImageJPEGRepresentation(img, 0.45);
}

- (void)imagePickerControllerDidCancel:(UIImagePickerController *)picker
{
    [self.imgPicker dismissModalViewControllerAnimated:NO];
}



#pragma mark - ASIHTTPRequestDelegate
- (void)requestFinished:(ASIHTTPRequest *)request
{
    NSString *json = [request responseString];
    NSLog(@"STATION DETAILS VIEW CONTROLLER - requestFinished: %@", json);
    
    [self.loading hide];
    [self showAlert:@"Station Updated" message:@"Your station has been successfully updated."];
    
    NSDictionary *d = [json JSONValue];
    if (d==nil)
        return;
    
    
    d = [d objectForKey:@"results"];
    NSLog(@"%@", [d description]);
    
    NSDictionary *info = [d objectForKey:@"station"];
    [self.station populate:info];
    self.station.imgData = nil;
    [[NSNotificationCenter defaultCenter] postNotification:[NSNotification notificationWithName:kRefreshNotification object:nil]];
    
    self.station.delegate = self;
    [self.station fetchImage];
}



#pragma mark - UIResponder Touch Methods
- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
    NSLog(@"touchesBegan:");
    [super touchesBegan:touches withEvent:event];
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
    NSLog(@"touchesMoved:");
    [super touchesMoved:touches withEvent:event];
}

-(void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
    NSLog(@"touchesEnded:");
    [super touchesEnded:touches withEvent:event];
    
    UITouch *t = [touches anyObject];
    if ([t.view isEqual:image]){
        [self photoBtnPressed];
        return;
    }
    
    if ([t.view isEqual:categoryLabel]){
        (categoriesPicker.frame.origin.y > 254.0f) ? [self slidePicker:254.0f] : [self slidePicker:self.view.frame.size.height];
    }
}

- (void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event
{
    NSLog(@"touchesCancelled:");
}



- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

@end
