//
//  BRAudioPlayer.m
//  AllThingsComedy
//
//  Created by Denny Kwon on 3/4/13.
//  Copyright (c) 2013 TheGridMedia. All rights reserved.
//

#import "BRAudioPlayer.h"
#import <MediaPlayer/MediaPlayer.h>


@interface BRAudioPlayer ()
@property (nonatomic) double check;
@end

@implementation BRAudioPlayer
@synthesize tracks;
@synthesize avPlayer;
@synthesize currentTrack;
@synthesize playbackTimer;
@synthesize delegate;
@synthesize butterflyStation;

static BRAudioPlayer *player;

- (id)init
{
    self = [super init];
    if (self){
        self.butterflyStation = NO;
        self.tracks = [NSMutableArray array];
        
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(trackCompleted)
                                                     name:AVPlayerItemDidPlayToEndTimeNotification
                                                   object:nil];
        
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(pause)
                                                     name:kInterruptionNotification
                                                   object:nil];
        
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(reset)
                                                     name:kRecordingNotification
                                                   object:nil];
    }
    return self;
}


+ (BRAudioPlayer *)audioPlayer
{
    if (!player)
        player = [[BRAudioPlayer alloc] init];
    
    return player;
}


- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self
                                                    name:AVPlayerItemDidPlayToEndTimeNotification
                                                  object:nil];
    
    [[NSNotificationCenter defaultCenter] removeObserver:self
                                                    name:kInterruptionNotification
                                                  object:nil];
    
    [[NSNotificationCenter defaultCenter] removeObserver:self
                                                    name:kRecordingNotification
                                                  object:nil];

    
    [self.avPlayer removeObserver:self forKeyPath:@"status"];
    
    if (player)
        [player release];
    
    if (self.avPlayer)
        [self.avPlayer pause];
    
    [self killTimer];
    self.tracks = nil;
    self.avPlayer = nil;
    self.currentTrack = nil;
    [super dealloc];
}

- (void)activateTimer
{
    self.playbackTimer = [NSTimer scheduledTimerWithTimeInterval:1.0f/8.0f
                                                          target:self
                                                        selector:@selector(timerAction)
                                                        userInfo:nil
                                                         repeats:YES];
    
}

- (void)killTimer
{
    if (!self.playbackTimer)
        return;
    
    [self.playbackTimer invalidate];
    self.playbackTimer = nil;
}


- (void)start:(int)index
{
    self.check = 0;
    index = index % self.tracks.count;
    BRTrack *track = [self.tracks objectAtIndex:index];
    if (!track)
        return;
    
    self.currentTrack = track;
    [self.currentTrack.playerItem addObserver:self forKeyPath:@"status" options:0 context:nil];
    [self.currentTrack preparePlayerItem];
    
    if (self.avPlayer){
        [self.avPlayer pause];
        [self.avPlayer removeObserver:self forKeyPath:@"status"];
        self.avPlayer = nil;
    }
    
    self.avPlayer = [AVPlayer playerWithPlayerItem:track.playerItem];
    [self.avPlayer addObserver:self forKeyPath:@"status" options:0 context:nil];
    
    [self killTimer];
    [self activateTimer];
}

- (void)reset
{
    if (!self.avPlayer)
        return;
    
    [self killTimer];
    [self.avPlayer pause];
    [self.avPlayer removeObserver:self forKeyPath:@"status" context:nil];
    self.avPlayer = nil;
}

- (void)timerAction
{
    if (self.currentTrack.duration==0){
        CMTime t = self.currentTrack.playerItem.duration;
        Float64 duration = CMTimeGetSeconds(t);
        self.currentTrack.duration = duration;
    }
    
    //    NSLog(@"TIMER ACTION: current: %.2f / %.2f", CMTimeGetSeconds(self.avPlayer.currentTime), self.currentTrack.duration);
    double pct = CMTimeGetSeconds(self.avPlayer.currentTime) / self.currentTrack.duration;
    
    //this prevents 'false' progress updates. sometimes, the player will indicate progress without actually playing audio. this keeps that in check:
    if ((pct-self.check)==pct){
        self.check = pct;
        return;
    }
    
    if ((pct-self.check)==0)
        return;
    
    [self.delegate progressUpdate:pct];
    self.check = pct;
}

- (BOOL)playPause
{
    NSLog(@"BR AUDIO PLAYER - playPause");
    if(self.avPlayer.rate==0){ // pasued - resume playback
        [self activateTimer];
        [self.avPlayer play];
        return YES;
    }
    
    [self killTimer];
    [self.avPlayer pause];
    return NO;
}


- (void)pause
{
    NSLog(@"BR AUDIO PLAYER - pause");
    [self killTimer];
    [self.avPlayer pause];
}

- (void)playPreviousTrack
{
    if(!self.currentTrack){
        [self start:0];
        return;
    }
    
    int index = [self.tracks indexOfObject:self.currentTrack];
    index = (index==0) ? (self.tracks.count-1) : index-1;
    [self start:index];
}

- (void)playNextTrack
{
    if(!self.currentTrack){
        [self start:0];
        return;
    }
    
    int index = [self.tracks indexOfObject:self.currentTrack];
    index++;
    index = index % self.tracks.count;
    [self start:index];
}

- (void)seek:(double)pct
{
    [self killTimer];
    [self pause];
    if (self.currentTrack.duration==0){
        CMTime t = self.currentTrack.playerItem.duration;
        Float64 duration = CMTimeGetSeconds(t);
        self.currentTrack.duration = duration;
    }
    
    double s = pct*self.currentTrack.duration;
    Float64 duration = (Float64)s;
    
    self.check = 0;
    [self.avPlayer seekToTime:CMTimeMakeWithSeconds(duration, 1)
            completionHandler:^(BOOL finished){
                NSLog(@"SEEKING FINISHED!");
                [self.avPlayer play];
                [self activateTimer];
            }];
}

- (void)trackCompleted
{
    NSLog(@"BR AUDIO PLAYER: trackCompleted");
    [self playNextTrack];
}

#pragma mark - KVO:
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
    NSLog(@"observeValueForKeyPath: %@", keyPath);
    if ([object isEqual:self.currentTrack.playerItem]){
        NSLog(@"CURRENT TRACK STATUS:");
        return;
    }
    
    if (object != avPlayer)
        return;
    
    if (![keyPath isEqualToString:@"status"])
        return;
    
    if (self.avPlayer.status == AVPlayerStatusFailed){
        NSLog(@"ERROR: %@", [self.avPlayer.error localizedDescription]);
        return;
    }
    
    if (self.avPlayer.status == AVPlayerStatusReadyToPlay) {
        NSLog(@"BR AUDIO PLAYER: PLAYER READY");
        [[NSNotificationCenter defaultCenter] postNotification:[NSNotification notificationWithName:kAVPlayerReadyNotification object:nil]];
        [self.avPlayer play];
        [self updateLockScreen];
        return;
    }
    
}

- (void)updateLockScreen //this notification comes from the Player whenever a new file is loaded
{
    if (![MPNowPlayingInfoCenter class])
        return;

    /* we're on iOS 5, so set up the now playing center */
    NSString *title = [NSString stringWithFormat:@"%@", self.currentTrack.name];
    UIImage *image = [UIImage imageNamed:@"Default-568h@2x.png"];
    if (!image)
        image = [UIImage imageNamed:@"bg_register.png"];
    
    MPMediaItemArtwork *albumArt = [[MPMediaItemArtwork alloc] initWithImage:image];
    NSDictionary *currentlyPlayingTrackInfo = [NSDictionary dictionaryWithObjectsAndKeys:title, MPMediaItemPropertyTitle, albumArt, MPMediaItemPropertyArtwork, nil];
    [albumArt release];
    [MPNowPlayingInfoCenter defaultCenter].nowPlayingInfo = currentlyPlayingTrackInfo;
}


#pragma mark - RemoteControlNotifications;
- (void)receivedRemoteControlEvent:(UIEvent *)event
{
    if (event.type != UIEventTypeRemoteControl)
        return;
    
    if (event.subtype==UIEventSubtypeRemoteControlPlay){
        NSLog(@"BR AUDIO PLAYER - receivedRemoteControlEvent: PLAY");
        if (self.avPlayer)
            [self.avPlayer play];
    }
    if (event.subtype==UIEventSubtypeRemoteControlPause){
        NSLog(@"BR AUDIO PLAYER - receivedRemoteControlEvent: PAUSE");
        if (self.avPlayer)
            [self.avPlayer pause];
    }
    if (event.subtype==UIEventSubtypeRemoteControlStop){
        NSLog(@"BR AUDIO PLAYER - receivedRemoteControlEvent: STOP");
        if (self.avPlayer)
            [self.avPlayer pause];
    }
    if (event.subtype==UIEventSubtypeRemoteControlTogglePlayPause){
        NSLog(@"BR AUDIO PLAYER - receivedRemoteControlEvent: TOGGLE PLAY PAUSE");
        (self.avPlayer.rate > 0) ? [self.avPlayer pause] : [self.avPlayer play];
    }
    if (event.subtype==UIEventSubtypeRemoteControlNextTrack){
        NSLog(@"BR AUDIO PLAYER - SKIP");
        if (self.avPlayer)
            [self playNextTrack];
    }
    if (event.subtype==UIEventSubtypeRemoteControlPreviousTrack)
        NSLog(@"GO BACK");
    
    
    [[NSNotificationCenter defaultCenter] postNotification:[NSNotification notificationWithName:kPlaystateChangedNotification object:nil]];
}





@end
