//
//  BRTrack.h
//  ButterflySDK
//
//  Created by Denny Kwon on 11/22/12.
//  Copyright (c) 2012 ButterflyRadio. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <AudioToolbox/AudioToolbox.h>
#import <AVFoundation/AVFoundation.h>
#import "ASIFormDataRequest.h"
#import "BRNetworkOp.h"


typedef enum {
    FileFormatMP3 = 0,
    FileFormatM4A,
    FileFormatNeither,
} FileFormat;

@protocol BRTrackDelegate <NSObject>
- (void)startPlayback;
@end

@interface BRTrack : NSObject    

@property (assign) id delegate;
@property (nonatomic) int fileSize;
@property (nonatomic) int leftoff;
@property (nonatomic) int value;
@property (nonatomic) double progressIndex;
@property (nonatomic) BOOL ad;
@property (nonatomic) BOOL full;
@property (nonatomic) BOOL reply;
@property (nonatomic) double duration;
@property (nonatomic) FileFormat format;
@property (nonatomic) AudioStreamBasicDescription asbd;
@property (nonatomic) NSInteger dataOffset;

@property (copy, nonatomic) NSString *url;
@property (copy, nonatomic) NSString *author;
@property (copy, nonatomic) NSString *station;
@property (copy, nonatomic) NSString *name;
@property (copy, nonatomic) NSString *fileIndex; //position in the array
@property (copy, nonatomic) NSString *image;
@property (copy, nonatomic) NSString *link;
@property (copy, nonatomic) NSString *description;
@property (copy, nonatomic) NSString *thread;
@property (copy, nonatomic) NSString *date;
@property (retain, nonatomic) NSDate *nsdate;
@property (retain, nonatomic) NSArray *tags;
@property (retain, nonatomic) UIImage *imgData;

@property (retain, nonatomic) BRNetworkOp *connection;
@property (retain, nonatomic) NSMutableData *fileData;
@property (retain, nonatomic) NSOperationQueue *queue;


//these properties are for AVPlayer audio player. BRTrack's use either AVPlayer OR BR Streamer, not both
@property (retain, nonatomic) AVPlayerItem *playerItem;
@property (nonatomic) int mins;



+ (BRTrack *)track;
+ (BRTrack *)trackWithInfo:(NSDictionary *)info;
- (void)populate:(NSDictionary *)trackInfo;
- (void)populatePostParameters:(ASIFormDataRequest *)request;
- (void)reset;
- (BOOL)checkCache;
- (void)download; //download the entire file
- (void)download:(NSRange)range; //download segment
- (NSData *)getData;
- (double)incrementIndex:(double)updateFreq;
- (double)estimateDuration:(unsigned long)bytesPlayed;
- (double)calculatedBitRate:(UInt64)processedPacketsSizeTotal packetsCount:(UInt64)processedPacketsCount;
- (NSString *)threadUrl;
- (void)prepare;
- (int)minDataSize;
- (BOOL)saveToDb;
- (void)deleteFromDb;
- (void)preparePlayerItem; // only applies to tracks that use AVPlayer framework NOT BR Streamer
- (void)fetchImage:(int)dimen;
@end
