//
//  BRTrack.m
//  ButterflySDK
//
//  Created by Denny Kwon on 11/22/12.
//  Copyright (c) 2012 ButterflyRadio. All rights reserved.
//

#import "BRTrack.h"
#include <sys/xattr.h>
#import "Globals.h"
#import "Database.h"

#define kMaxByteRange 5*1024*1024 // 5 MB max download

@implementation BRTrack
@synthesize url;
@synthesize author;
@synthesize name;
@synthesize fileIndex; //position in the array
@synthesize image;
@synthesize station;
@synthesize link;
@synthesize date;
@synthesize nsdate;
@synthesize description;
@synthesize thread;
@synthesize tags;

@synthesize ad;
@synthesize duration;
@synthesize connection;
@synthesize fileData;
@synthesize format;
@synthesize leftoff;
@synthesize full;
@synthesize progressIndex;
@synthesize delegate;
@synthesize fileSize;
@synthesize asbd;
@synthesize dataOffset;
@synthesize value;
@synthesize reply;
@synthesize mins;
@synthesize playerItem;
@synthesize imgData;
@synthesize queue;

- (id)init
{
    self=[super init];
    if (self){
        self.ad = NO;
        self.full = NO;
        self.reply = NO;
        self.value = 0;
    }
    return self;
}

+ (BRTrack *)track
{
    BRTrack *track = [[BRTrack alloc] init];
    return [track autorelease];
}

+ (BRTrack *)trackWithInfo:(NSDictionary *)info
{
    BRTrack *track = [[BRTrack alloc] init];
    [track populate:info];
    return [track autorelease];
}


- (void)dealloc
{
    if (self.queue){
        [self.queue cancelAllOperations];
        self.queue = nil;
    }
    
    self.url = nil;
    self.author = nil;
    self.name = nil;
    self.fileIndex = nil;
    self.image = nil;
    self.link = nil;
    self.date = nil;
    self.description = nil;
    self.thread = nil;
    self.tags = nil;
    self.nsdate = nil;
    self.fileData = nil;
    self.playerItem = nil;
    self.imgData = nil;
    self.station = nil;
    
    [self.connection cancel];
    self.connection.delegate = nil;
    self.connection = nil;
    [super dealloc];
}

- (void)preparePlayerItem
{
    if (self.playerItem)
        self.playerItem = nil;
    
    AVURLAsset *asset = [AVURLAsset assetWithURL:[NSURL URLWithString:self.url]];
    self.playerItem = [AVPlayerItem playerItemWithAsset:asset];
}


- (void)populate:(NSDictionary *)trackInfo
{
    NSString *trackId = [trackInfo objectForKey:@"id"];
    NSString *thr = [trackInfo objectForKey:@"in_reply_to"];
    if (thr==nil){ self.thread = trackId; }
    else{
        self.thread = ([thr isEqualToString:@"0"]) ? trackId : thr;
        self.reply = ([trackId rangeOfString:self.thread].location == NSNotFound);
//        if ([trackId rangeOfString:self.thread].location == NSNotFound)
//            self.reply = YES;
    }
    
    
    self.name = [trackInfo objectForKey:@"title"];
    self.station = [trackInfo objectForKey:@"station"]; // id number of host station
    NSString *summary = [trackInfo objectForKey:@"description"];
    self.description = ([summary isEqualToString:@"none"]) ? @"no description" : summary;
    self.url = [NSString stringWithFormat:@"http://%@/stream/%@", kUrl, trackId];
    
    NSString *d = [trackInfo objectForKey:@"date"];
    NSArray *p = [d componentsSeparatedByString:@" "];
    self.date = ([p count]>3) ? [NSString stringWithFormat:@"%@ %@, %@", [p objectAtIndex:1], [p objectAtIndex:2], [p lastObject]] : d;
    NSDateFormatter *dateformatter = [[NSDateFormatter alloc] init];
    [dateformatter setDateFormat:@"EEE MMM dd HH:mm:ss zzz yyyy"]; //AUDIO FILE DATE: Fri Nov 16 06:20:08 UTC 2012 - this is the
    
    self.nsdate = [dateformatter dateFromString:d];
    [dateformatter release];
    self.author = [trackInfo objectForKey:@"author"];
    
    NSString *v = [trackInfo objectForKey:@"value"];
    if (v)
        self.value = [v intValue];
}

- (void)fetchImage:(int)dimen
{
    if (self.queue) // already fetching, return
        return;
    
    // http://butterflyradio.com/image?resource=station&id=24087728375669082&size=150&crop=yes
    NSString *addr = [NSString stringWithFormat:@"http://%@/image?resource=station&id=%@&size=%d&crop=yes", kUrl, self.station, dimen];
    GetImage *fetch = [[GetImage alloc] initWithTarget:self
                                               address:addr
                                                action:@selector(iconReady:)];
    
    if (!self.queue)
        self.queue = [[[NSOperationQueue alloc] init] autorelease];
    
    [self.queue addOperation:fetch];
    [fetch release];
}

- (void)iconReady:(NSArray *)pkg
{
    NSLog(@"%@ - iconReady", [self.class description]);
    if (!pkg)
        return;
    
    NSData *imageData = [pkg lastObject];
    self.imgData = [UIImage imageWithData:imageData];
    self.queue = nil;
    [[NSNotificationCenter defaultCenter] postNotification:[NSNotification notificationWithName:kTrackImageReadyNotification object:nil]];
    
}

- (BOOL)saveToDb
{
    NSLog(@"BR TRACK - SAVE TO DB!");
    if (!self.fileData)
        return NO;
    
    int t = (int)[self.nsdate timeIntervalSince1970];
    NSString *timestamp = [NSString stringWithFormat:@"%d", t];

    NSError *error = nil;
    [self.fileData writeToFile:[self createFilePath:timestamp] options:NSDataWritingAtomic error:&error];
    if (error){
        NSLog(@"BR TRACK - SAVE TO DB: ERROR - %@", [error localizedDescription]);
        return NO;
    }
    
    Database *db = [Database database];
    NSDictionary *params = @{@"uniqueId":timestamp, @"name":self.name, @"author":self.author, @"description":@"description", @"tags":@"tag", @"value":[NSString stringWithFormat:@"%d", self.value], @"duration":[NSString stringWithFormat:@"%d", (int)self.duration], @"timestamp":timestamp };

    [db insertRecordingDB:params];
    return YES;
}

- (void)deleteFromDb
{
    NSLog(@"BR TRACK: deleteFromDb");
    Database *db = [Database database];
    int t = (int)[self.nsdate timeIntervalSince1970];
    NSString *timestampString = [NSString stringWithFormat:@"%d", t];
    [db deleteRecordingFromDB:timestampString];
    
    NSString *filePath = [self createFilePath:timestampString];
    NSError *error = nil;
    [[NSFileManager defaultManager] removeItemAtPath:filePath error:&error];
    if (error)
        NSLog(@"BR TRACK: deleteFromDb - error removing file data!");
    
//    NSMutableData *data = [NSMutableData dataWithContentsOfFile:[self createFilePath:timestampString directory:NSCachesDirectory]];

}

- (NSString *)threadUrl
{
    if (self.thread.length<8)
        return self.url;
    
    NSString *threadSuffix = [self.thread substringFromIndex:self.thread.length-8];
    return [NSString stringWithFormat:@"http://%@/site/audioplayer/%@", kUrl, threadSuffix];
}

- (void)prepare
{
    NSLog(@"BR TRACK - Prepare!");
    if (self.connection) // currently downloading, return.
        return;
    
    BOOL cached = [self checkCache];
    if (!cached)
        [self download];
    
}

- (double)estimateDuration:(unsigned long)bytesPlayed
{
    double adjustedFileSize = (double)self.fileSize-self.dataOffset;
    double multiple = adjustedFileSize/(double)bytesPlayed;
    
    double estimate = multiple * self.progressIndex;
    NSLog(@"BR TRACK - estimateDuration: = %.2f, dataOffset==%d", estimate, self.dataOffset);
    return estimate;
}

- (double)incrementIndex:(double)updateFreq
{
    //gets called every updateFreq seconds during playback by the streamer. the file then updates the player with current location.
    self.progressIndex += updateFreq;
    NSLog(@"BR TRACK - incrementIndex: progressIndex = %.2f", self.progressIndex);
    
    if (self.duration<1)
        return 0.0f;
    
    double percentage = (double)self.progressIndex/self.duration;
    NSLog(@"BR TRACK - incrementIndex: percentage = %.2f, duration=%.2f", percentage, self.duration);
    
    //    if (percentage>1.05){ [delegate skip]; } //backup plan - just in case the normal 'file finished' doesn't get called.
    
    if (self.connection)
        return percentage;
    
    return percentage;
}

/*
 - kills url connection if still running
 - nils out fileData */
- (void)reset
{
    if (self.connection!=nil){ //this calls if the user skips over this file while it is still downloading;
        self.connection.delegate = nil;
        [self.connection cancel];
        self.connection = nil;
        
    }
    
    self.fileData = nil;
    self.ad = NO;
    self.full  = NO;
    self.leftoff = 0;
    self.progressIndex = 0;
}

- (double)calculatedBitRate:(UInt64)processedPacketsSizeTotal packetsCount:(UInt64)processedPacketsCount
{
    double sampleRate = self.asbd.mSampleRate;
	double packetDuration = self.asbd.mFramesPerPacket / sampleRate;
    
    int bitRateEstimationMinPackets = 50;
    
	if (packetDuration && processedPacketsCount > bitRateEstimationMinPackets){
		double averagePacketByteSize = processedPacketsSizeTotal / processedPacketsCount;
		return 8.0 * averagePacketByteSize / packetDuration;
	}
	
    // m4a and a few other formats refuse to parse the bitrate so
    // we need to set an "unparseable" condition here. If you know
    // the bitrate (parsed it another way) you can set it on the
    // class if needed.
    //
    //    double bitRate = ~0;
    //    return bitRate;
	
	return 0;
}


- (NSData *)getData
{
//    int offSet = 40000;
//	if (self.format == FileFormatMP3){ offSet = 170000; }

    int offSet = (self.format == FileFormatMP3) ? 170000 : 40000;

    int max = (self.leftoff+offSet < self.fileData.length) ? leftoff+offSet : self.fileData.length;
    NSLog(@"BR TRACK: getData - %d to %d", self.leftoff, max);
    
	NSRange range = NSMakeRange(self.leftoff, max-self.leftoff);
	self.leftoff = max;
    [self checkNextSegment];
	return [self.fileData subdataWithRange:range];
}

- (void)checkNextSegment //check if almost out of data:
{
    NSLog(@"checkNextSegment: fileDataLength==%d, fileSize==%d, leftOff==%d", self.fileData.length, self.fileSize, self.leftoff);
    if (self.connection) // in the process of downloading, don't interrrupt
        return;
    
    if (self.full) //entire file is already downloaded, no need to check
        return;
    
    double pct = (double)self.leftoff/(double)self.fileData.length;
    NSLog(@"BR TRACK: checkNextSegment - %.3f USED", pct);
    if (pct > 0.50f){ //file progressed past half way point of available data - fetch next segment
        NSLog(@"TIME TO DOWNLOAD MORE DATA!");
        int max = self.fileData.length+kMaxByteRange;
        if (max>self.fileSize)
            max = self.fileSize;
        
        [self download:NSMakeRange(self.fileData.length, kMaxByteRange)]; //download next 5 MB
    }
}

- (BOOL)checkCache
{
    NSLog(@"BR TRACK - check cache");
    NSString *filePath = [self createFilePath:self.url];
    NSData *data = [NSData dataWithContentsOfFile:filePath];
    if (data){
        NSLog(@"BR TRACK - check cache: USING CACHED FILE!");
        self.full = YES;
        self.fileData = [NSMutableData dataWithData:data];
        self.fileSize = self.fileData.length;
        [self calculateFileDuration:filePath];
        return YES;
    }
    
    NSLog(@"BR TRACK - check cache: NO CACHED FILE!");
    self.full = NO;
    return NO;
}

- (void)download:(NSRange)range;
{
    if (self.full) { //cached file, start playback right away:
        [self fileDataReady];
        return;
    }
    
    if (self.connection) //in the middle of download, abort:
        return;
    
    if (!self.fileData)
        self.fileData = [NSMutableData data];
    
    self.connection = [BRNetworkOp operationWithAddress:self.url parameters:nil];
    self.connection.delegate = self;
    self.connection.responseData = self.fileData;
    
    if (range.length>0)
        self.connection.requestHeaders = @{@"Range" : [NSString stringWithFormat:@"bytes=%d-%d", range.location, (range.location+range.length)]};
    
    [self.connection setHttpMethod:@"GET"];
    [self.connection sendRequest];
}

- (void)download
{
    [self download:NSMakeRange(0, kMaxByteRange)]; // 5 MB max
}

- (void)calculateFileDuration:(NSString *)filePath
{
    NSLog(@"calculateFileDuration:");
    AudioFileID fileID;
    NSURL *fileUrl = [NSURL fileURLWithPath:filePath];
    OSStatus result = AudioFileOpenURL((CFURLRef)fileUrl, kAudioFileReadPermission, 0, &fileID);
    
    if (result!=noErr){
        NSLog(@"calculateFileDuration - ERROR: %ld", result);
        return;
    }
    
    NSTimeInterval seconds;
    UInt32 propertySize = sizeof(seconds);
    
    result = AudioFileGetProperty (fileID, kAudioFilePropertyEstimatedDuration, &propertySize, &seconds);
    if (result!=noErr){
        NSLog(@"BR TRACK - ERROR: %ld", result);
        return;
    }
    
    double multiple = (double)self.fileSize/(double)self.fileData.length;
    seconds *= multiple;
    self.duration = seconds;
    NSLog(@"calculateFileDuration: RESULT == %.2f SECONDS", self.duration);
}

- (void)fileDataReady
{
    NSLog(@"fileDataReady: %d bytes", self.fileData.length);
    if (self.delegate==nil)
        return;
    
    [self.delegate startPlayback];
    self.delegate = nil;
}

- (void)populatePostParameters:(ASIFormDataRequest *)request
{
    [request setPostValue:self.author forKey:@"author"];
    [request setPostValue:self.name forKey:@"title"];
    [request setPostValue:self.description forKey:@"description"];
    [request setPostValue:[NSString stringWithFormat:@"%d", (int)self.duration] forKey:@"duration"];
    [request setPostValue:[NSString stringWithFormat:@"%d", self.value] forKey:@"value"];
    
    if ([self.tags containsObject:@"none"]){
        [request setPostValue:@"none" forKey:@"tags"];
        return;
    }
    
    NSString *tagString = @"";
    for (NSString *t in self.tags) {
        tagString = [tagString stringByAppendingString:[NSString stringWithFormat:@"%@,", t]];
    }
    tagString = [tagString substringToIndex:tagString.length-1]; //remove trailing comma
    [request setPostValue:tagString forKey:@"tags"];
}

#pragma mark - BRNetworkOpDelegate:
- (void)initialResponse:(NSDictionary *)headers
{
    
    /* Sometimes, this error msg comes back from Libsyn. If you just try again, it works the 2nd time:
     Connection = close;
     "Content-Encoding" = gzip;
     "Content-Length" = 26;
     "Content-Type" = "text/html";
     Date = "Wed, 26 Dec 2012 04:38:13 GMT";
     Server = Apache;
     Vary = "Accept-Encoding";
     "X-Libsyn-Host" = "traffic2.libsyn.com";
     */
    
    NSLog(@"INITIAL RESPONSE: %@", [headers description]);
    NSString *mimeType = [headers objectForKey:@"Content-Type"];
    if ([mimeType rangeOfString:@"audio"].location==NSNotFound){
        //error - not an audio file, handle error
        NSLog(@"BR TRACK - ERROR: NOT AN AUDIO FILE!");
        NSString *contentEncoding = [headers objectForKey:@"Content-Encoding"];
        if ([contentEncoding isEqualToString:@"gzip"]){
            self.connection = nil;
            [self download];
        }
        return;
    }
    
    if (self.fileSize > 0)
        return;
    
    NSString *contentRange = [headers objectForKey:@"Content-Range"];
    NSString *contentLength = [headers objectForKey:@"Content-Length"];
    if (contentRange){
        NSArray *parts = [contentRange componentsSeparatedByString:@"/"];
        self.fileSize = (parts.count>1) ? [[parts lastObject] intValue] : [contentLength intValue];
        NSLog(@"BR TRACK - EXPECTED FILE SIZE: %d", self.fileSize);
        return;
    }
    
    self.fileSize = [contentLength intValue];
    NSLog(@"BR TRACK - EXPECTED FILE SIZE: %d", self.fileSize);
}

- (void)downloadedData:(NSData *)data
{
    NSLog(@"BR TRACK - downloadedData: %d", self.fileData.length);
    if (!self.delegate)
        return;
    
    NSLog(@"TEST 1");
    int min = [self minDataSize];
    if (self.fileData.length > min){
        NSLog(@"TEST 2");
        [self fileDataReady];
    }
}

- (int)minDataSize
{
    int min = 0.1*self.fileSize; //min for wi-fi, download 10% first
    if (self.connection.networkStatus==ReachableViaWWAN) //need more data for 3G
        min = 0.2*self.fileSize; // download 20% first
    
    int ceiling = 200*1024;
    if (min > ceiling)
        min = ceiling;
    
    return min;
}

- (void)requestData:(NSArray *)pkg //returns [address, data]
{
    if (!pkg)
        return;
    
    self.connection.delegate = nil;
    self.connection = nil;
    
    NSLog(@"BR TRACK - requestData: %d bytes, EXPECTED SIZE: %d bytes", self.fileData.length, self.fileSize);
    self.full = (self.fileData.length >= self.fileSize);
    
    if (self.full){ //entire file downloaded, run duration calculation
        NSString *filePath = [self createFilePath:self.url];
        if (self.fileData.length < 2*kMaxByteRange){ //don't save any files larger than 10 MB - basically, podcasts
            [self.fileData writeToFile:filePath atomically:YES];
            [self addSkipBackupAttributeToItemAtURL:[NSURL URLWithString:filePath]]; //this prevents files from being backed up on itunes and iCloud
        }
        [self calculateFileDuration:filePath];
    }
}

#pragma mark - File Saving Stuff:
- (NSString *)createFilePath:(NSString *)fileName
{
	fileName = [fileName stringByReplacingOccurrencesOfString:@"/" withString:@"+"];
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES);
	NSString *filePath = [[paths objectAtIndex:0] stringByAppendingPathComponent:fileName];
	return filePath;
}

- (BOOL)addSkipBackupAttributeToItemAtURL:(NSURL *)URL
{
    const char* filePath = [[URL path] fileSystemRepresentation];
    const char* attrName = "com.apple.MobileBackup";
    u_int8_t attrValue = 1;
    
    int result = setxattr(filePath, attrName, &attrValue, sizeof(attrValue), 0, 0);
    return result == 0;
}


@end
