//
//  ButterflyManager.m
//  frenchkiss
//
//  Created by Denny Kwon on 8/19/12.
//  Copyright (c) 2012 Frenchkiss Records. All rights reserved.
//

#import "ButterflyManager.h"


static ButterflyManager *manager;

@implementation ButterflyManager
@synthesize currentStation;
@synthesize stations;
@synthesize host;
@synthesize delegate;
@synthesize appBundle;
@synthesize appName;
@synthesize appVersion;
@synthesize appHost;
@synthesize db;
@synthesize favorites;
@synthesize streamer;


- (id)initWithAppHost:(NSString *)hostStation
{
    self = [super init];
    if (self){
        self.appHost = (hostStation) ? [hostStation lowercaseString] : [NSString stringWithFormat:@"%@", kGeneralEmail]; //default app host
        self.stations = nil;
        
        queue = [[NSOperationQueue alloc] init];
        NSLog(@"BUTTERFLY MANAGER - INIT WITH APP HOST: %@", self.appHost);
        [self configure];

        BRHost *h = [[BRHost alloc] init];
        self.host = h;
        self.host.appHost = self.appHost;
        [h release];
        
        self.streamer = [BRStreamer streamer];
        [self checkDatabase];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(checkFavorites) name:kResetDatabase object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(interrupt) name:kInterruptionNotification object:nil];
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(newFile:) name:kBRStreamerStartedNotification object:nil];
//        [self checkPurchases];
    }
    return self;
}



+ (ButterflyManager *)managerWithAppHost:(NSString *)appHost
{
    if (manager==nil){
        manager = [[ButterflyManager alloc] initWithAppHost:appHost];
    }

    return manager;
}

- (void)dealloc
{
//    [[NSNotificationCenter defaultCenter] removeObserver:self name:@"new file" object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:kResetDatabase object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:kInterruptionNotification object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:kBRStreamerStartedNotification object:nil];

    self.currentStation = nil;
    self.host = nil;
    self.appBundle = nil;
    self.appName = nil;
    self.appVersion = nil;
    self.appHost = nil;
    self.db = nil;
    self.favorites = nil;
    self.streamer = nil;
    
    [self voidRequest];
//    if (req!=nil){
//        req.delegate = nil;
//        [req cancel];
//        [req release];
//    }
    
    if (manager)
        [manager release];

    [queue release];
    [super dealloc];
}

- (void)configure
{
    NSDictionary *appInfo = [[NSBundle mainBundle] infoDictionary];
    NSLog(@"%@", [appInfo description]);
    if (!appInfo)
        return;

    NSString *bundleId = [appInfo objectForKey:@"CFBundleIdentifier"];
    if (bundleId)
        self.appBundle = bundleId;
    
    NSString *aName = [appInfo objectForKey:@"CFBundleDisplayName"];
    if (aName)
        self.appName = aName;
    
    NSString *version = [appInfo objectForKey:@"CFBundleVersion"];
    if (version)
        self.appVersion = version;
}



- (NSString *)createFilePath:(NSString *)fileName
{
	fileName = [fileName stringByReplacingOccurrencesOfString:@"/" withString:@"+"];
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *filePath = [[paths objectAtIndex:0] stringByAppendingPathComponent:fileName];
	return filePath;
}

- (void)checkDatabase
{
    // First, test for existence.
    NSFileManager *fileManager = [NSFileManager defaultManager];
    
    NSString *writableDBPath = [self createFilePath:kDatabase];
    BOOL success = [fileManager fileExistsAtPath:writableDBPath];
    if (success==TRUE){
        NSLog(@"DATABASE FOUND!");
        self.db = [Database database];
        [self checkFavorites];

        return;
    }
    
    // The writable database does not exist, so copy the default to the appropriate location.    NSLog(@"CREATING NEW DATABASE");
    NSError *error = nil;
    NSString *defaultDBPath = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:kDatabase];
    success = [fileManager copyItemAtPath:defaultDBPath toPath:writableDBPath error:&error];
    if (!success) {
        NSAssert1(0, @"Failed to create writable database file with message '%@'.", [error localizedDescription]);
    }
    
    self.db = [Database database];
    [self checkFavorites];
}

- (void)checkFavorites
{
    NSLog(@"BUTTERFLY MANAGER - checkFavorites");
    self.favorites = [db fetchAll];
}

//load traack from tableview which is segmented into multiple sections
- (void)loadTrack:(NSIndexPath *)indexPath fromStation:(Station *)station
{
    Thread *thread = (Thread *)[station.threadArray objectAtIndex:indexPath.section];
    BRTrack *track = (BRTrack *)[thread.thread objectAtIndex:indexPath.row];
    
    self.streamer.tracks = station.tracks;
    self.currentStation = station;
    [self playTrack:track fromStation:self.currentStation];
    
}

- (void)playTrack:(BRTrack *)track fromStation:(Station *)station
{
    NSLog(@"BUTTERFLY MGR - playTrack: %@, index = %@", track.name, track.fileIndex);
    if ([track isEqual:self.streamer.source]) // selected track is already playing.
        return;
    
    
    if ([self.currentStation isEqual:station]==NO)
        self.currentStation = station;

    if ([self.currentStation.tracks containsObject:track])
        self.streamer.tracks = station.tracks;
    
    if ([self.currentStation.premiumTracks containsObject:track]) // might be coming from premium array
        self.streamer.tracks = station.premiumTracks;

    if ([self.currentStation.podcastTracks containsObject:track]) // might be coming from podcast array
        self.streamer.tracks = station.podcastTracks;

    [self.streamer reset:NO];
    self.streamer.index = [track.fileIndex intValue];
    [self.streamer start];
}

- (void)voidRequest
{
    if (!req)
        return;
    
    req.delegate = nil;
    [req cancel];
    [req release];
}

- (void)executeRequest:(NSString *)url
{
    NSLog(@"BUTTERFLY MANAGER - EXECUTE REQUEST: %@", url);
    [self voidRequest];
    
    req = [[BRNetworkOp alloc] initWithAddress:url parameters:nil];
    req.delegate = self;
    [req setHttpMethod:@"GET"];
    [req sendRequest];
}

- (void)fetchStations
{
//    [self checkCache:kAdminStationsReq];
//    if (req!=nil){
//        req.delegate = nil;
//        [req cancel];
//        [req release];
//    }
    
//    [self voidRequest];
    
    NSString *url = [NSString stringWithFormat:@"http://%@/api/station?admins=%@", kUrl, self.appHost];
    [self executeRequest:url];
//    req = [[BRNetworkOp alloc] initWithAddress:url parameters:nil];
//    req.delegate = self;
//    [req setHttpMethod:@"GET"];
//    [req sendRequest];
}

- (void)fetchTopTracks:(int)offset
{
//    [self voidRequest];
    NSString *url = [NSString stringWithFormat:@"http://%@/api/track?offset=%d", kUrl, offset];
    [self executeRequest:url];

//    req = [[BRNetworkOp alloc] initWithAddress:url parameters:nil];
//    req.delegate = self;
//    [req setHttpMethod:@"GET"];
//    [req sendRequest];
}


- (void)requestData:(NSArray *)pkg
{
    if (pkg==nil)
        return;
    
    NSData *returnData = [pkg objectAtIndex:1];
    NSError *error = nil;
    NSDictionary *d = (NSDictionary *)[NSJSONSerialization JSONObjectWithData:returnData
                                                                      options:NSJSONReadingMutableContainers
                                                                        error:&error];
    if (error){
        [req sendRequest];
        return;
    }
    
    d = [d objectForKey:@"results"];
//    NSLog(@"BUTTERFLY MANAGER - REQUEST DATA: %@", [d description]);
    
    NSString *confirmation = [d objectForKey:@"confirmation"];
    if (![confirmation isEqualToString:@"found"]){
        return;
    }

    // Stations request
    NSArray *s = [d objectForKey:@"stations"];
    if (s){
        if ([s containsObject:@"none"])
            return;
        
        if (self.stations==nil)
            self.stations = [NSMutableDictionary dictionary];
        
        [self.stations removeAllObjects];
        NSMutableArray *allStations = [NSMutableArray array];
        NSMutableArray *hostStations = [NSMutableArray array];
        NSMutableArray *admin = [NSMutableArray array];
        
        for (int i=0; i<s.count; i++){
            NSDictionary *info = [s objectAtIndex:i];
            Station *station = [Station stationWithInfo:info];
            [allStations addObject:station];
            ([station.host isEqualToString:self.appHost]) ? [hostStations addObject:station] : [admin addObject:station];
        }
        
        [self.stations setObject:allStations forKey:@"all"];
        [self.stations setObject:hostStations forKey:@"host"];
        [self.stations setObject:admin forKey:@"admin"];

        [[NSNotificationCenter defaultCenter] postNotification:[NSNotification notificationWithName:kStationsReadyNotification object:nil]];
    }
    
    
    // Tracks request - set up response handler
//    NSArray *t = [d objectForKey:@"tracks"];


}


- (void)showRadio
{
    NSLog(@"BUTTERFLY MANAGER - show radio");
}

- (void)start:(int)index
{
    NSLog(@"BUTTERFLY MANAGER - start: %d", index);
}



- (void)newFile:(NSNotification *)note //this notification comes from the Player whenever a new file is loaded
{
    NSString *title = [NSString stringWithFormat:@"%@: %@", self.currentStation.name, self.streamer.source.name];
    if ([MPNowPlayingInfoCenter class])  { /* we're on iOS 5, so set up the now playing center */
        UIImage *image = [UIImage imageNamed:@"Default-568h@2x.png"];
        if (!image)
            image = [UIImage imageNamed:@"bg_register.png"];

        
        MPMediaItemArtwork *albumArt = [[MPMediaItemArtwork alloc] initWithImage:image];
        NSDictionary *currentlyPlayingTrackInfo = [NSDictionary dictionaryWithObjectsAndKeys:title, MPMediaItemPropertyTitle, albumArt, MPMediaItemPropertyArtwork, nil];
        [albumArt release];
        [MPNowPlayingInfoCenter defaultCenter].nowPlayingInfo = currentlyPlayingTrackInfo;
    }
}

- (void)interrupt
{
    if (self.streamer)
        [self.streamer pause];
}

- (void)checkPurchases
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    
    // if this array exists, then a purchase was executed but the update on ButterflyRadio didn't go through:
    NSArray *storedPurchase = [defaults arrayForKey:kTicketsPurchaseKey];
    if (!storedPurchase)
        return;

    NSLog(@"STORED PURCHASE: %@", [storedPurchase description]);
//    NSString *email = [storedPurchase objectAtIndex:0];
//    NSString *value = [storedPurchase lastObject];
    
    
}

#pragma mark - StationSearches:
- (void)searchStations:(BRStationSearchFilter)filter
{
    NSString *url = nil;
    
    if (filter==StationSearchFilterTop)
        url = [NSString stringWithFormat:@"http://%@/api/station", kUrl];


    if (filter==StationSearchFilterEmail)
        url = [NSString stringWithFormat:@"http://%@/api/station?host=%@", kUrl, self.appHost];


    if (filter==StationSearchFilterAdmin)
        url = [NSString stringWithFormat:@"http://%@/api/station?admins=%@", kUrl, self.appHost];

    if (filter==StationSearchFilterHost){
        if ([self.host.email isEqualToString:@"none"]==NO)
            url = [NSString stringWithFormat:@"http://%@/api/station?admins=%@", kUrl, self.host.email];
    }

    
    if (!url)
        return;

    GetImage *adminStations = [[GetImage alloc] initWithTarget:self address:url action:@selector(searchResultsReturned:) filter:filter];
    [queue addOperation:adminStations];
    [adminStations release];
}

- (void)searchStationsByCategory:(NSString *)category
{
    NSArray *categories = @[@"all", @"advice", @"comedy", @"misc", @"politics", @"sports"];

    category = [category lowercaseString];
    if (![categories containsObject:category])
        return;
    
    NSString *url = [NSString stringWithFormat:@"http://%@/api/station", kUrl];
    if (![category isEqualToString:@"all"]) {
        url = [url stringByAppendingString:[NSString stringWithFormat:@"?category=%@", category]];
    }
    
    GetImage *categoryStations = [[GetImage alloc] initWithTarget:self address:url action:@selector(searchResultsReturned:)];
    [queue addOperation:categoryStations];
    [categoryStations release];
}

- (void)searchResultsReturned:(NSArray *)pkg
{
    NSLog(@"searchResultsReturned:");
    if (pkg==nil)
        return;
    
    NSData *returnData = [pkg lastObject];
    NSError *error = nil;
    id jsonObject = [NSJSONSerialization JSONObjectWithData:returnData options:NSJSONReadingMutableContainers error:&error];
    if (error)
        return;
    
    NSMutableArray *searchResults = [NSMutableArray array];
    NSDictionary *results = [jsonObject objectForKey:@"results"];
    NSString *confirmation = [results objectForKey:@"confirmation"];
    if (!confirmation){
        NSNotification *note = [NSNotification notificationWithName:kSearchResultsReadyNotification object:nil userInfo:[NSDictionary dictionaryWithObject:searchResults forKey:@"results"]];
        [[NSNotificationCenter defaultCenter] postNotification:note];
        return;
    }
    
    if (![confirmation isEqualToString:@"found"]){
        NSNotification *note = [NSNotification notificationWithName:kSearchResultsReadyNotification object:nil userInfo:[NSDictionary dictionaryWithObject:searchResults forKey:@"results"]];
        [[NSNotificationCenter defaultCenter] postNotification:note];
        return;
    }
    
    NSArray *s = [results objectForKey:@"stations"];
    for (int i=0; i<s.count; i++){
        NSDictionary *info = [s objectAtIndex:i];
        Station *station = [Station stationWithInfo:info];
        [station populate:info];
        [searchResults addObject:station];
    }
    
    NSNotification *note = [NSNotification notificationWithName:kSearchResultsReadyNotification object:nil userInfo:[NSDictionary dictionaryWithObject:searchResults forKey:@"results"]];
    [[NSNotificationCenter defaultCenter] postNotification:note];

}

#pragma mark - RemoteControlNotifications;
- (void)receivedRemoteControlEvent:(UIEvent *)event
{
    if (event.type == UIEventTypeRemoteControl) {
        if (event.subtype==UIEventSubtypeRemoteControlPlay){
            NSLog(@"BUTTERFLY MGR - receivedRemoteControlEvent: PLAY");
            if (self.streamer)
                [self.streamer resume];
        }
        if (event.subtype==UIEventSubtypeRemoteControlPause){
            NSLog(@"BUTTERFLY MGR - receivedRemoteControlEvent: PAUSE");
            if (self.streamer)
                [self.streamer pause];
        }
        if (event.subtype==UIEventSubtypeRemoteControlStop){
            NSLog(@"BUTTERFLY MGR - receivedRemoteControlEvent: STOP");
            if (self.streamer)
                [self.streamer pause];
        }
        if (event.subtype==UIEventSubtypeRemoteControlTogglePlayPause){
            NSLog(@"BUTTERFLY MGR - receivedRemoteControlEvent: TOGGLE PLAY PAUSE");
            (self.streamer.isRunning) ? [self.streamer pause] : [self.streamer resume];
        }
        if (event.subtype==UIEventSubtypeRemoteControlNextTrack){
            NSLog(@"SKIP");
            if (self.streamer)
                [self.streamer skip];
        }
        if (event.subtype==UIEventSubtypeRemoteControlPreviousTrack){
            NSLog(@"GO BACK");
        }
    }
}




@end
