//
//  ButterflyViewController.m
//  frenchkiss
//
//  Created by Denny Kwon on 8/20/12.
//  Copyright (c) 2012 Frenchkiss Records. All rights reserved.


#import "ButterflyViewController.h"
#import "RadioViewController.h"
#import "HostViewController.h"
#import "LoginViewController.h"
#import "BRPurchaseViewController.h"


@interface ButterflyViewController ()

@end

@implementation ButterflyViewController
@synthesize butterflyMgr;
@synthesize signal;
@synthesize loading;
@synthesize request;
@synthesize premiumTrack;
@synthesize ptStart;
@synthesize purhcaseManager;
@synthesize bottomToolbar;
@synthesize isOnTop;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        self.isOnTop = NO;
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(noSignalAlert) name:kNoSignalNotification object:nil];
        reloadable = TRUE;
        self.signal = [SignalCheck signalWithDelegate:self];
    }
    return self;
}

- (id)initWithManager:(ButterflyManager *)mgr
{
    self = [super init];
    if (self) {
        [self addNavigationBanner];
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(noSignalAlert) name:kNoSignalNotification object:nil];
        self.butterflyMgr = mgr;
        reloadable = TRUE;
        self.signal = [SignalCheck signalWithDelegate:self];
    }
    return self;
}


- (void)dealloc
{
    if (self.request){
        self.request.delegate = nil;
        [self.request cancel];
        self.request = nil;
    }
    self.bottomToolbar = nil;
    self.premiumTrack = nil;
    self.loading = nil;
    self.signal = nil;
    self.butterflyMgr = nil;
    self.purhcaseManager = nil;
    [super dealloc];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    self.isOnTop = YES;
    if (self.butterflyMgr.host.loggedIn){
        self.bottomToolbar.lblAccount.text = self.butterflyMgr.host.email;
        self.bottomToolbar.lblTickets.text = [NSString stringWithFormat:@"tickets: %d", self.butterflyMgr.host.tickets];
    }
    else{
        self.bottomToolbar.lblAccount.text = @"account: logged out";
        self.bottomToolbar.lblTickets.text = @"tickets: 0";
    }
}

- (void)viewDidDisappear:(BOOL)animated
{
    [super viewDidDisappear:animated];
    self.isOnTop = NO;
}

- (UIView *)baseViewWithColor:(UIColor *)color
{
    CGRect frame = [UIScreen mainScreen].applicationFrame;
    frame.origin.y = 0.0f;
    frame.origin.x = 0.0f;
    UIView *view = [[UIView alloc] initWithFrame:frame];
    view.autoresizingMask = UIViewAutoresizingFlexibleHeight;
    view.backgroundColor = color;
    return [view autorelease];
}

// Adds Butterfly nogo to navigation bar on top: 
- (void)addNavigationBanner
{
    CGRect appFrame = [UIScreen mainScreen].applicationFrame;
    UIImage *img_banner = [UIImage imageNamed:@"banner_home.png"];
    CGFloat w = img_banner.size.width;
    CGFloat h = img_banner.size.height;
    UIImageView *banner = [[UIImageView alloc] initWithFrame:CGRectMake(0.5*(appFrame.size.width-w), 10, w, h)];
    banner.image = img_banner;
    self.navigationItem.titleView = banner;
    [banner release];
}


- (void)setupLoadingIndicator
{
    loading = [[LoadingIndicator alloc] initWithFrame:self.view.frame];
    loading.hidden = YES;
    [self.view addSubview:loading];
}

- (void)setupLoadingIndicator:(int)color verticalOffset:(CGFloat)y
{
    self.loading = [[[LoadingIndicator alloc] initWithFrame:self.view.frame loadingIndicator:color verticalOffset:y] autorelease];
    self.loading.hidden = YES;
    [self.view addSubview:loading];
}

- (void)setupNowPlayingButton
{
    UIButton *nowPlaying = [UIButton buttonWithType:UIButtonTypeCustom];
    UIImage *img = [UIImage imageNamed:@"nowPlaying.png"];
    [nowPlaying setBackgroundImage:img forState:UIControlStateNormal];
    [nowPlaying addTarget:self action:@selector(showRadio) forControlEvents:UIControlEventTouchUpInside];
    nowPlaying.showsTouchWhenHighlighted = YES;
    nowPlaying.frame = CGRectMake(0, 0, img.size.width, img.size.height);
    UIBarButtonItem *showRadio = [[UIBarButtonItem alloc] initWithCustomView:nowPlaying];
    
    self.navigationItem.rightBarButtonItem = showRadio;
    [showRadio release];
}


- (void)setUpBackButton:(id)target action:(SEL)action
{
    UIImage *imgNavBack = [UIImage imageNamed:@"navBack.png"];
    UIButton *back = [UIButton buttonWithType:UIButtonTypeCustom];
    [back setImage:imgNavBack forState:UIControlStateNormal];
    back.frame = CGRectMake(0, 0, 0.65f*imgNavBack.size.width, 0.65f*imgNavBack.size.height);
    [back addTarget:target action:action forControlEvents:UIControlEventTouchUpInside];
    UIBarButtonItem *btnBack = [[UIBarButtonItem alloc] initWithCustomView:back];
    self.navigationItem.leftBarButtonItem = btnBack;
    [btnBack release];
}


- (void)showAlert:(NSString *)title message:(NSString *)msg
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:title message:msg delegate:nil cancelButtonTitle:@"ok" otherButtonTitles:nil];
    [alert show];
    [alert release];
}

- (void)noSignalAlert
{
    [self showAlert:@"No Connection" message:@"Please find an internet connection."];
}

- (void)showRadio
{
    if (!self.butterflyMgr.currentStation){
        [self showAlert:@"No Station" message:@"There is no station currently selected."];
        return;
    }
    
    RadioViewController *radio = [[RadioViewController alloc] init];
    radio.butterflyMgr = self.butterflyMgr;
    [self.navigationController pushViewController:radio animated:YES];
    [radio release];
}

- (void)showRadioWithLoader
{
    if (!self.butterflyMgr.currentStation){
        [self showAlert:@"No Station" message:@"There is no station currently selected."];
        return;
    }
    
    RadioViewController *radio = [[RadioViewController alloc] init];
    radio.butterflyMgr = self.butterflyMgr;
    [self.navigationController pushViewController:radio animated:YES];
    [radio.loading show];
    [radio release];
}


- (NSString *)createFilePath:(NSString *)fileName
{
	fileName = [fileName stringByReplacingOccurrencesOfString:@"/" withString:@"+"];
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES);
	NSString *docPath = [paths objectAtIndex:0];
	NSString *filePath = [docPath stringByAppendingPathComponent:fileName];
	NSLog(@"filepath = %@", filePath);
	return filePath;
}


- (void)requestError:(NSError *)error
{
    [self.loading hide];
    [self showAlert:@"Error" message:@"There was an error. Please try again."];
}

- (UIBarButtonItem *)barButtonWithImage:(UIImage *)image tag:(int)t
{
    UIButton *btn = [UIButton buttonWithType:UIButtonTypeCustom];
    [btn setBackgroundImage:image forState:UIControlStateNormal];
    btn.frame = CGRectMake(0, 0, image.size.width, image.size.height);
    btn.tag = t;
    [btn addTarget:self action:@selector(barButtonTapped:) forControlEvents:UIControlEventTouchUpInside];
    
    UIBarButtonItem *barButton = [[UIBarButtonItem alloc] initWithCustomView:btn];
    return [barButton autorelease];
}

- (UIBarButtonItem *)barButtonWithText:(NSString *)text tag:(int)t
{
    UIBarButtonItem *barButton = [[UIBarButtonItem alloc] initWithTitle:text style:UIBarButtonItemStyleBordered target:self action:@selector(barButtonTapped:)];
    barButton.tag = t;
    return [barButton autorelease];
}

- (void)barButtonTapped:(UIBarButtonItem *)btn
{
    //override in subclass
}

- (void)showAccount
{
    if (self.butterflyMgr.host.loggedIn){
        HostViewController *hostVC = [[HostViewController alloc] initWithManager:self.butterflyMgr];
        UINavigationController *acctNavCtr = [[UINavigationController alloc] initWithRootViewController:hostVC];
        [hostVC release];
        
        acctNavCtr.navigationBar.tintColor = [UIColor orangeColor];
        [self presentViewController:acctNavCtr animated:YES completion:NULL];
        [acctNavCtr release];
        return;
    }
    
    LoginViewController *login = [[LoginViewController alloc] initWithManager:self.butterflyMgr];
    UINavigationController *loginNavCtr = [[UINavigationController alloc] initWithRootViewController:login];
    loginNavCtr.navigationBar.tintColor = [UIColor orangeColor];
    [login release];
    
    [self presentViewController:loginNavCtr animated:YES completion:NULL];
    [loginNavCtr release];
}


#pragma mark - RefreshHeader
- (void)addPullToRefreshHeader:(UIScrollView *)container
{
    refreshHeaderView = [[UIView alloc] initWithFrame:CGRectMake(0, 0 - REFRESH_HEADER_HEIGHT, 320, REFRESH_HEADER_HEIGHT)];
    refreshHeaderView.backgroundColor = [UIColor clearColor];
    
    refreshLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, 0, 320, REFRESH_HEADER_HEIGHT)];
    refreshLabel.font = [UIFont boldSystemFontOfSize:15.0f];
    refreshLabel.shadowColor = [UIColor whiteColor];
    refreshLabel.shadowOffset = CGSizeMake(-0.5f, 0.5f);
    refreshLabel.backgroundColor = [UIColor clearColor];
    refreshLabel.textColor = [UIColor grayColor];
    refreshLabel.numberOfLines = 2;
    refreshLabel.textAlignment = UITextAlignmentCenter;
    
    refreshArrow = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"arrow.png"]];
    refreshArrow.frame = CGRectMake(floorf((REFRESH_HEADER_HEIGHT - 27) / 2),
                                    (floorf(REFRESH_HEADER_HEIGHT - 44) / 2),
                                    27, 44);
    
    refreshSpinner = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleGray];
    refreshSpinner.frame = CGRectMake(floorf(floorf(REFRESH_HEADER_HEIGHT - 20) / 2), floorf((REFRESH_HEADER_HEIGHT - 20) / 2), 20, 20);
    refreshSpinner.hidesWhenStopped = YES;
    
    [refreshHeaderView addSubview:refreshLabel];
    [refreshHeaderView addSubview:refreshArrow];
    [refreshHeaderView addSubview:refreshSpinner];
    [container addSubview:refreshHeaderView];
}

- (BRTrackAuthorization)trackAuthorizationStatus:(BRTrack *)track fromStation:(Station *)station
{
    if (!self.butterflyMgr.host.loggedIn){ //not logged in
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Not Logged In" message:@"Please log in to listen to this track or to purchase it. To log in, navigate to the Account section and select your email provider. Thank you." delegate:nil cancelButtonTitle:@"ok" otherButtonTitles:nil];
        [alert show];
        [alert release];
        return BRTrackAuthorizationNotLoggedIn;
    }
    
    if ([self.butterflyMgr.host.premiumThreads containsObject:track.thread]){ // has access, play track:
        [self.butterflyMgr playTrack:track fromStation:station];
        return BRTrackAuthorizationAuthorized;
    }
    
    if ([station.host isEqualToString:self.butterflyMgr.host.email]){ // current user is author of track. it's all good.
        [self.butterflyMgr playTrack:track fromStation:station];
        return BRTrackAuthorizationAuthorized;
    }
    
    if ([station.admins containsObject:self.butterflyMgr.host.email]){ // user is an admin. it's all good.
        [self.butterflyMgr playTrack:track fromStation:station];
        return BRTrackAuthorizationAuthorized;
    }
    
    UIAlertView *alert = nil;
    if (self.butterflyMgr.host.tickets >= track.value){ //user has enough tickets
        NSString *msg = [NSString stringWithFormat:@"This conversation requires %d tickets. You have %d tickets remaining. Once you enter a conversation, you do not have to use tickets again. Would you like to enter this conversation?", track.value, self.butterflyMgr.host.tickets];
        alert = [[UIAlertView alloc] initWithTitle:@"Premium Track" message:msg delegate:self cancelButtonTitle:@"Yes" otherButtonTitles:@"No", nil];
        [alert show];
        [alert release];

        return BRTrackAuthorizationEnoughTickets;
    }
    
    //not enough tickets, have to buy more
    NSString *msg = [NSString stringWithFormat:@"This conversation requires %d tickets. You have %d tickets remaining. Would you like to purchase more tickets?", track.value, self.butterflyMgr.host.tickets];
    alert = [[UIAlertView alloc] initWithTitle:@"Premium Track" message:msg delegate:self cancelButtonTitle:@"Yes" otherButtonTitles:@"No", nil];
    [alert show];
    [alert release];
    return BRTrackAuthorizationNeedMoreTickets;

}



#pragma mark - ScrollviewDelegate
- (void)scrollViewWillBeginDragging:(UIScrollView *)scrollView
{
//    NSLog(@"scrollViewWillBeginDragging:");
    if (!reloadable)
        return;
    
    if (isLoading) return;
    isDragging = YES;
}

- (void)scrollViewDidScroll:(UIScrollView *)scrollView
{
//    NSLog(@"scrollViewDidScroll:");
    if (!reloadable)
        return;
    
    if (isLoading) { // Update the content inset, good for section headers
        if (scrollView.contentOffset.y > 0)
            scrollView.contentInset = UIEdgeInsetsZero;
        else if (scrollView.contentOffset.y >= -REFRESH_HEADER_HEIGHT)
            scrollView.contentInset = UIEdgeInsetsMake(-scrollView.contentOffset.y, 0, 0, 0);
    } else if (isDragging && scrollView.contentOffset.y < 0) {
        // Update the arrow direction and label
        [UIView animateWithDuration:0.25 animations:^{
            if (scrollView.contentOffset.y < -REFRESH_HEADER_HEIGHT) {
                // User is scrolling above the header
                refreshLabel.text = @"Release to refresh...";
                [refreshArrow layer].transform = CATransform3DMakeRotation(M_PI, 0, 0, 1);
            } else {
                // User is scrolling somewhere within the header
                refreshLabel.text = @"Pull down to refresh...";
                [refreshArrow layer].transform = CATransform3DMakeRotation(M_PI * 2, 0, 0, 1);
            }
        }];
    }
}


- (void)scrollViewDidEndDragging:(UIScrollView *)scrollView willDecelerate:(BOOL)decelerate
{
//    NSLog(@"scrollViewDidEndDragging: willDecelerate:");
    if (!reloadable)
        return;
    
    if (isLoading) return;
    isDragging = NO;
    if (scrollView.contentOffset.y <= -REFRESH_HEADER_HEIGHT) {
        // Released above the header
        [self startLoading:scrollView];
    }
}


- (void)startLoading:(UIScrollView *)scrollView
{
//    NSLog(@"startLoading");

    if (!reloadable)
        return;

    isLoading = YES;
    
    // Show the header
    [UIView animateWithDuration:0.3 animations:^{
        scrollView.contentInset = UIEdgeInsetsMake(REFRESH_HEADER_HEIGHT, 0, 0, 0);
        refreshLabel.text = @"Loading...";
        refreshArrow.hidden = YES;
        [refreshSpinner startAnimating];
    }];
    
    
    if (![signal checkSignal]){
        [self showAlert:@"No Connection" message:@"Please find an internet connection."];
        [self stopLoading:scrollView];
        return;
    }

    // Refresh action!
    [self updateInfo];
}


- (void)updateInfo //subclass should always override this!
{
    NSLog(@"%@ - UPDATE INFO: Subclass should override this !", [[self class] description]);
}


- (void)stopLoading:(UIScrollView *)scrollView
{
    NSLog(@"stopLoading");
    if (!reloadable)
        return;

    isLoading = NO;
    
    // Hide the header
    [UIView animateWithDuration:0.3 animations:^{
        scrollView.contentInset = UIEdgeInsetsZero;
        [refreshArrow layer].transform = CATransform3DMakeRotation(M_PI * 2, 0, 0, 1);
    }
                     completion:^(BOOL finished) {
                         [self performSelector:@selector(stopLoadingComplete)];
                     }];
}

- (void)stopLoadingComplete
{
    NSLog(@"stopLoadingComplete");
    if (!reloadable)
        return;

    // Reset the header
    refreshLabel.text = @"Pull down to refresh...";
    refreshArrow.hidden = NO;
    [refreshSpinner stopAnimating];
}

- (UIBarButtonItem *)flex
{
    UIBarButtonItem *flex = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil];
    return [flex autorelease];
}

- (void)slide
{
    //method stub for subclasses to override.
}

#pragma mark - UIResponder Touch Methods
- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
    NSLog(@"touchesBegan:");
    UITouch *t = [touches anyObject];
    self.ptStart = [t locationInView:self.view];
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
    NSLog(@"touchesMoved:");
}

-(void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
    NSLog(@"touchesEnded:");
    UITouch *t = [touches anyObject];
    CGPoint ptEnd = [t locationInView:self.view];
    
    double diff = ptEnd.x-self.ptStart.x;
    diff = abs(diff);
    if (diff > 50)
        [self slide];

}

- (void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event
{
    NSLog(@"touchesCancelled:");
}

- (void)launchPurchaseSequence
{
    if (!self.butterflyMgr.host.loggedIn) // have to be logged in
        return;
    
    if (!self.purhcaseManager)
        self.purhcaseManager = [PurchaseManager manager];
    
    self.purhcaseManager.delegate = self;
    [self.purhcaseManager getProductList:@[k5TicketsProductId, k10TicketsProductId]];
    [self.loading show];
}

#pragma mark - PurchaseManagerDelegate
- (void)productListReady
{
    NSLog(@"BR PURCHASE VC - productListReady");
    [self.loading hide];
    
    // No products found:
    if (self.purhcaseManager.SKProductsArray.count==0){
        [self showAlert:@"Error" message:@"There was an error. Please try again."];
        [self dismissModalViewControllerAnimated:YES];
        return;
    }
    
    UIImage *img = [self.navigationController screenshot];
    BRPurchaseViewController *purchaseVc = [[BRPurchaseViewController alloc] initWithBackgroundImage:img];
    purchaseVc.purhcaseManager = self.purhcaseManager;
    purchaseVc.butterflyMgr = self.butterflyMgr;
    [self presentModalViewController:purchaseVc animated:NO];
    [purchaseVc release];
}

- (void)purchaseFailed:(NSError *)error
{
    NSLog(@"PURCHASE FAILED: %@", [error localizedDescription]);
    [self.loading hide];
}

- (void)purchaseComplete:(BOOL)confirmation productID:(NSString *)pId reciept:(NSDictionary *)d
{
    
}

#pragma mark - View Life Cycle
- (void)viewDidLoad
{
    [super viewDidLoad];
    [self.navigationController setNavigationBarHidden:NO animated:YES];

}

- (void)viewDidUnload
{
    [super viewDidUnload];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

@end
