//
//  Database.m
//  sql
//
//  Created by Denny Kwon on 5/23/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "Database.h"

static Database *_database;

@implementation Database
@synthesize monthsMap;

- (NSString *)createFilePath:(NSString *)fileName directory:(NSSearchPathDirectory)d
{
	fileName = [fileName stringByReplacingOccurrencesOfString:@"/" withString:@"+"];
    NSArray *paths = paths = NSSearchPathForDirectoriesInDomains(d, NSUserDomainMask, YES);
	NSString *filePath = [[paths objectAtIndex:0] stringByAppendingPathComponent:fileName];
	return filePath;
}

+ (Database *)database
{
    if (_database==nil){
        _database = [[Database alloc] init];
    }
    else{
        NSLog(@"SINGLETON INSTANCE!");
    }
    return _database;
}

- (id)init
{
    self = [super init];
    if (self){
        NSString *sqLiteDb = [self createFilePath:kDatabase directory:NSDocumentDirectory];
        
        if (sqlite3_open([sqLiteDb UTF8String], &_database) != SQLITE_OK) {
            NSLog(@"Failed to open database!");
        }
        else{
            NSLog(@"DATABSE OPENED!");
        }
        
        // KEY is the UNESCAPED string ('), VALUE is the ESCAPED string (\')
        escapeCodes = [[NSDictionary alloc] initWithObjectsAndKeys:@"^", @"'", @"*", @"\"", nil];
        decodedMap = [[NSDictionary alloc] initWithObjectsAndKeys:@"'",  @"^", @"\"", @"*", nil];
        self.monthsMap = @{@"01":@"Jan", @"02":@"Feb", @"03":@"March", @"04":@"April", @"05":@"May", @"06":@"June", @"07":@"July", @"08":@"Aug", @"09":@"Sep", @"10":@"Oct", @"11":@"Nov", @"12":@"Dec", };
    }
    return self;
}

- (void)dealloc
{
    sqlite3_close(_database);
    [escapeCodes release];
    self.monthsMap = nil;
    [super dealloc];
}


- (void)executeStatement:(NSString *)query notification:(NSNotification *)note
{
    sqlite3_stmt *statement;
    int status = sqlite3_prepare_v2(_database, [query UTF8String], -1, &statement, nil);
    if (status != SQLITE_OK){
        NSLog(@"SQL ERROR: %d", status);
        return;
    }
    
    status = sqlite3_step(statement);
    if (status != SQLITE_DONE){
        NSLog(@"SQL ERROR: STATEMENT FAILED");
        return;
    }
    
    if (note)
        [[NSNotificationCenter defaultCenter] postNotification:note];
    
    sqlite3_finalize(statement);
}

#pragma mark - BRStations
- (BOOL)checkSaved:(NSString *)uniqueId
{
    BOOL saved = FALSE;
    NSString *query = [NSString stringWithFormat:@"SELECT * FROM stations where uniqueId = %@", uniqueId];
    NSLog(@"QUERY: %@", query);
    sqlite3_stmt *statement;
    int status = sqlite3_prepare_v2(_database, [query UTF8String], -1, &statement, nil);
    if (status == SQLITE_OK){
        NSLog(@"fetchByID: SQL SUCCESS");
        while (sqlite3_step(statement) == SQLITE_ROW) {
            saved = TRUE;
            char *idNum = (char *)sqlite3_column_text(statement, 0);
            char *nameChars = (char *)sqlite3_column_text(statement, 1);
            printf("%s == %s\n", idNum, nameChars);
        }
    }
    else { NSLog(@"SQL ERROR"); }
    return saved;
}



- (NSString *)fetchByID:(NSString *)uniqueId
{
    NSString *name = nil;
    NSString *query = [NSString stringWithFormat:@"SELECT * FROM stations where uniqueId = %@", uniqueId];
    NSLog(@"QUERY: %@", query);
    
    sqlite3_stmt *statement;
    int status = sqlite3_prepare_v2(_database, [query UTF8String], -1, &statement, nil);
    if (status == SQLITE_OK){
        
        NSLog(@"fetchByID: SQL SUCCESS");
        while (sqlite3_step(statement) == SQLITE_ROW) {
            char *idNum = (char *)sqlite3_column_text(statement, 0);
            char *nameChars = (char *)sqlite3_column_text(statement, 1);
            printf("%s == %s\n", idNum, nameChars);
            name = [[NSString alloc] initWithUTF8String:nameChars];
        }

    }
    else {
        NSLog(@"SQL ERROR");
    }
    return [name autorelease];
}

- (NSMutableArray *)fetchAll
{
    //sqlite> create table stations(uniqueId text, name text, host text, category text, image text);

    NSMutableArray *resultSet = [NSMutableArray array];
//    NSString *query = @"SELECT * FROM stations where age > 30 ORDER BY age DESC";
    NSString *query = @"SELECT * FROM stations";
    sqlite3_stmt *statement;
    
    int status = sqlite3_prepare_v2(_database, [query UTF8String], -1, &statement, nil);
    if (status == SQLITE_OK){
        NSLog(@"SQL SUCCESS");
        while (sqlite3_step(statement) == SQLITE_ROW) {
            char *idNum = (char *)sqlite3_column_text(statement, 0);
            char *nameChars = (char *)sqlite3_column_text(statement, 1);
            char *hostChars = (char *)sqlite3_column_text(statement, 2);
            char *categoryChars = (char *)sqlite3_column_text(statement, 3);
            char *imageChars = (char *)sqlite3_column_text(statement, 4);
            printf("uniqueId == %s; name==%s; host==%s\n", idNum, nameChars, hostChars);
            
            
            Station *station = [[Station alloc] init];
            station.saved = TRUE;
            station.unique_id = [NSString stringWithUTF8String:idNum];
            station.category = [NSString stringWithUTF8String:categoryChars];
            station.image = [NSString stringWithUTF8String:imageChars];

            NSString *name = [[NSString alloc] initWithUTF8String:nameChars];
            station.name = [self decodedString:name];
            [name release];

            NSString *host = [[NSString alloc] initWithUTF8String:hostChars];
            station.host = host;
            [host release];

            [resultSet addObject:station];
            [station release];
            
        }
        sqlite3_finalize(statement);
    }
    else { NSLog(@"SQL ERROR"); }
    return resultSet;
}

- (void)insertToDB:(NSDictionary *)params
{
//    NSString *query = @"INSERT INTO PEOPLE VALUES('jane', 'female', 25)";
    NSLog(@"INSERT TO DB: %@", [params description]);
    NSString *idNum = [params objectForKey:@"idNum"];
    NSString *name = [self fetchByID:idNum];
    if (name) //already in database, return
        return;
    
    NSLog(@"NOT FOUND"); //This is what should happen:
    name = [params objectForKey:@"name"];
    name = [self esacpedString:name];
    
    NSString *host = [params objectForKey:@"host"];
    NSString *category = [params objectForKey:@"category"];
    NSString *image = [params objectForKey:@"image"];
    NSString *values = [NSString stringWithFormat:@"'%@', '%@', '%@', '%@', '%@'", idNum, name, host, category, image];
    NSString *query = [NSString stringWithFormat:@"INSERT INTO STATIONS VALUES(%@)", values];
    
    [self executeStatement:query notification:[NSNotification notificationWithName:kResetDatabase object:nil]];
}

- (void)deleteFromDB:(NSString *)uniqueId
{
//    NSString *query = @"DELETE FROM people where age = 30"; //sample query
    NSString *query = [NSString stringWithFormat:@"DELETE FROM stations where uniqueId = %@", uniqueId];
    [self executeStatement:query notification:[NSNotification notificationWithName:kResetDatabase object:nil]];
}

#pragma mark - BRTracks
- (void)insertRecordingDB:(NSDictionary *)params
{
    if (!params)
        return;
    
    NSLog(@"DATABASE - INSERT RECORDING TO DB: %@", [params description]);
    NSString *uniqueId = [params objectForKey:@"uniqueId"]; //string representation of timestamp
    NSString *name = [params objectForKey:@"name"];
    NSString *author = [params objectForKey:@"author"];
    NSString *description = [params objectForKey:@"description"];
    NSString *tags = [params objectForKey:@"tags"];
    
    NSString *value = [params objectForKey:@"value"];
    NSString *duration = [params objectForKey:@"duration"];
    NSString *timestamp = [params objectForKey:@"timestamp"];
    
    //sqlite> create table brtracks(uniqueId text, name text, author text, duration integer, value integer, tags text, decription text, timestamp integer);
    NSString *values = [NSString stringWithFormat:@"'%@', '%@', '%@', %@, %@, '%@', '%@', %@", uniqueId, name, author, duration, value, tags, description, timestamp];
    NSString *query = [NSString stringWithFormat:@"INSERT INTO BRTRACKS VALUES(%@)", values];
    [self executeStatement:query notification:nil];
}

- (void)deleteRecordingFromDB:(NSString *)uniqueId
{
    NSString *query = [NSString stringWithFormat:@"DELETE FROM brtracks where uniqueId = %@", uniqueId];
    [self executeStatement:query notification:nil];
}

- (NSMutableArray *)fetchAllRecordings
{
    //sqlite> create table brtracks(uniqueId text, name text, author text, duration integer, value integer, tags text, decription text, timestamp integer);
    
    NSMutableArray *resultSet = [NSMutableArray array];
    NSString *query = @"SELECT * FROM brtracks ORDER BY timestamp DESC";  // NSString *query = @"SELECT * FROM stations where age > 30 ORDER BY age DESC";
    sqlite3_stmt *statement;
    
    int status = sqlite3_prepare_v2(_database, [query UTF8String], -1, &statement, nil);
    if (status != SQLITE_OK){
        NSLog(@"DATABASE - fetchAllRecordings: ERROR");
        return resultSet;
    }
    
    NSLog(@"SQL SUCCESS");
    while (sqlite3_step(statement) == SQLITE_ROW) {
        char *uniqueId = (char *)sqlite3_column_text(statement, 0);
        char *nameChars = (char *)sqlite3_column_text(statement, 1);
        char *authorChars = (char *)sqlite3_column_text(statement, 2);
        
        int duration = (int)sqlite3_column_int(statement, 3);
        int value = (int)sqlite3_column_int(statement, 4);
        char *tagsChars = (char *)sqlite3_column_text(statement, 5);
        char *decriptionChars = (char *)sqlite3_column_text(statement, 6);
        int timestamp = (int)sqlite3_column_int(statement, 7);
        
        printf("uniqueId == %s; name==%s; author==%s; duration=%d\n", uniqueId, nameChars, authorChars, duration);
        
        NSString *timestampString = [NSString stringWithFormat:@"%d", timestamp];
        NSMutableData *data = [NSMutableData dataWithContentsOfFile:[self createFilePath:timestampString directory:NSCachesDirectory]];

        if (data){
            BRTrack *track = [[BRTrack alloc] init];
            track.fileData = data;
            track.full = YES;
            track.fileSize = data.length;
            track.format = FileFormatM4A;
            track.name = [NSString stringWithUTF8String:nameChars];
            track.author = [NSString stringWithUTF8String:authorChars];
            track.description = [NSString stringWithUTF8String:decriptionChars];
            track.tags = [[NSString stringWithUTF8String:tagsChars] componentsSeparatedByString:@","];
            track.duration = duration;
            track.value = value;
            track.nsdate = [NSDate dateWithTimeIntervalSince1970:timestamp];
            NSString *dateString = track.nsdate.description;
            dateString = [[dateString componentsSeparatedByString:@" "] objectAtIndex:0];
            NSArray *parts = [dateString componentsSeparatedByString:@"-"]; // 2013-01-20
            if (parts.count>2){
                track.date = [NSString stringWithFormat:@"%@ %@, %@", [self.monthsMap objectForKey:[parts objectAtIndex:1]], [parts objectAtIndex:2], [parts objectAtIndex:0]];
            }
            else{
                track.date = dateString;
            }
            
            track.fileIndex = [NSString stringWithFormat:@"%d", resultSet.count];
            [resultSet addObject:track];
            [track release];
        }
        
    }
    sqlite3_finalize(statement);
    return resultSet;
}




#pragma mark -

- (NSString *)esacpedString:(NSString *)unencodedString
{
    
    for (NSString *unescaped in [escapeCodes allKeys]) {
        unencodedString = [unencodedString stringByReplacingOccurrencesOfString:unescaped withString:[escapeCodes objectForKey:unescaped]];
    }
    
    return unencodedString;
    
    
    
}

- (NSString *)decodedString:(NSString *)escapedNSString
{
    for (NSString *escaped in [decodedMap allKeys]) {
        
        escapedNSString = [escapedNSString stringByReplacingOccurrencesOfString:escaped withString:[decodedMap objectForKey:escaped]];
    }
    
    return escapedNSString;
}

@end
