//
//  ArticleViewController.m
//  butterflyradio
//
//  Created by Denny Kwon on 5/9/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "ArticleViewController.h"

@interface ArticleViewController ()
@property (retain, nonatomic) UIToolbar *toolbar;
@end

@implementation ArticleViewController
@synthesize article;
@synthesize station;
@synthesize theWebview;

- (id)init
{
    self = [super init];
    if (self) {
        self.hidesBottomBarWhenPushed = YES;
    }
    return self;
}

- (id)initWithManager:(ButterflyManager *)mgr
{
    self = [super initWithManager:mgr];
    if (self) {
        self.hidesBottomBarWhenPushed = YES;
    }
    return self;
}

- (void)dealloc
{
    self.theWebview = nil;
    self.station = nil;
    self.article = nil;
    self.toolbar = nil;
    [super dealloc];
}

- (void)loadView
{
    CGRect frame = [UIScreen mainScreen].applicationFrame;
    frame.origin.y = 0.0f;
    frame.origin.x = 0.0f;
    UIView *view = [[UIView alloc] initWithFrame:frame];
    view.autoresizingMask = (UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleWidth);
    view.backgroundColor = [UIColor greenColor];
    
    theWebview = [[UIWebView alloc] initWithFrame:frame];
    self.theWebview.delegate = self;
    self.theWebview.autoresizingMask = view.autoresizingMask;
    [view addSubview:self.theWebview];
    
    CGFloat height = 44.0f;
    self.toolbar = [[UIToolbar alloc] initWithFrame:CGRectMake(0, view.frame.size.height-height+1, view.frame.size.width, height)];
    self.toolbar.barStyle = UIBarStyleBlack;
    self.toolbar.translucent = YES;
    self.toolbar.autoresizingMask = (UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleTopMargin);
    
    NSArray *barButtons = @[[self flex], [self barButtonWithImage:[UIImage imageNamed:@"toolbarBack.png"] tag:1000], [self flex], [self barButtonWithImage:[UIImage imageNamed:@"toolbarNext.png"] tag:1001], [self flex]];
    
    self.toolbar.items = barButtons;
    [view addSubview:self.toolbar];

    
    UIButton *btnHide = [UIButton buttonWithType:UIButtonTypeCustom];
    btnHide.showsTouchWhenHighlighted = YES;
    btnHide.backgroundColor = [UIColor blackColor];
    [btnHide setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
    [btnHide setTitle:@"+" forState:UIControlStateNormal];
    btnHide.layer.cornerRadius = 5.0f;
    btnHide.alpha = 0.6f;
    CGFloat width = 30.0f;
    btnHide.autoresizingMask = (UIViewAutoresizingFlexibleTopMargin | UIViewAutoresizingFlexibleLeftMargin);
    btnHide.frame = CGRectMake(view.frame.size.width-width, view.frame.size.height-width, width, width);
    [btnHide addTarget:self action:@selector(toggleNavBar) forControlEvents:UIControlEventTouchUpInside];
    [view addSubview:btnHide];

    if (article.link==TRUE){
        theWebview.scalesPageToFit = YES;
        [theWebview loadRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:article.content]]];
        self.view = view;
        [view release];
        return;

    }

    //send url request, parse contents from results and load html string:
    theWebview.scalesPageToFit = NO;
    if (article.content==nil){
        NSString *url = [NSString stringWithFormat:@"http://%@/api/article/%@", kUrl, article.unique_id];
        self.request = [BRNetworkOp operationWithAddress:url parameters:nil];
        [self.request setHttpMethod:@"GET"];
        self.request.delegate = self;
        [self.request sendRequest];
        [self.loading show];
    }
    else{ [self setupHtml]; }

    self.view = view;
    [view release];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    [self setUpBackButton:self action:@selector(exit)];

//    [self setupLoadingIndicator];
    [self setupLoadingIndicator:1 verticalOffset:160.0f];

    
    UIBarButtonItem *btnTweet = [[UIBarButtonItem alloc] initWithImage:[UIImage imageNamed:@"icon_twitter.png"] style:UIBarButtonItemStyleBordered target:self action:@selector(tweet:)];
    
    self.navigationItem.rightBarButtonItems = [NSArray arrayWithObjects:btnTweet, nil];
    [btnTweet release];
    [self.loading show];
}

- (void)viewWillAppear:(BOOL)animated
{
    [[NSNotificationCenter defaultCenter] postNotification:[NSNotification notificationWithName:kSupportLandscape object:nil]];
}


#pragma mark -
- (void)toggleNavBar
{
    BOOL hide = !self.navigationController.navigationBarHidden;
    [self.navigationController setNavigationBarHidden:hide animated:YES];
    
    CGRect frame = self.toolbar.frame;
    frame.origin.y = (hide) ? self.view.frame.size.height : self.view.frame.size.height-44.0f;
    self.toolbar.frame = frame;
}

- (void)exit
{
    if (self.request){
        self.request.delegate = nil;
        [self.request cancel];
    }
    [self.navigationController popViewControllerAnimated:YES];
}

- (void)setupHtml
{
    NSString *path = [[NSBundle mainBundle] pathForResource:@"template" ofType:@"txt"];
    NSError *error = nil;
    NSString *template = [[NSString alloc] initWithContentsOfFile:path encoding:NSUTF8StringEncoding error:&error];
    if (error){
        NSLog(@"TEMPLATE NOT FOUND");
        return;
    }
    
    NSString *formattedHtml = [template stringByReplacingOccurrencesOfString:@"{{content}}" withString:article.content];
    [template release];
    formattedHtml = [formattedHtml stringByReplacingOccurrencesOfString:@"{{author}}" withString:article.author];
    formattedHtml = [formattedHtml stringByReplacingOccurrencesOfString:@"{{date}}" withString:article.date];
    formattedHtml = [formattedHtml stringByReplacingOccurrencesOfString:@"{{title}}" withString:article.title];
    
    if ([article.imageUrl isEqualToString:@"none"]==TRUE){
        formattedHtml = [formattedHtml stringByReplacingOccurrencesOfString:@"{{image}}" withString:@""];
        [theWebview loadHTMLString:formattedHtml baseURL:nil];
        return;
    }
    
    NSString *image = [[article.imageUrl componentsSeparatedByString:@"=="] objectAtIndex:0];
    image = [NSString stringWithFormat:@"<img style='width:300px' src='%@' />", image];
    formattedHtml = [formattedHtml stringByReplacingOccurrencesOfString:@"{{image}}" withString:image];
    [theWebview loadHTMLString:formattedHtml baseURL:nil];
}

- (void)barButtonTapped:(UIBarButtonItem *)btn
{
    if (btn.tag==1000){
        [self.theWebview goBack];
    }
    if (btn.tag==1001){
        [self.theWebview goForward];
    }
}

- (void)requestData:(NSArray *)pkg //returns [address, data]
{
    if (!pkg)
        return;
    
    NSDictionary *d = [self.request parseJsonResponse];
    if (d==nil){
        [self.request sendRequest];
        return;
    }
    
    d = [d objectForKey:@"results"];
    NSLog(@"%@", [d description]);
    NSString *confirmation = [d objectForKey:@"confirmation"];
    
    if ([confirmation isEqualToString:@"found"]==NO){
        [self.loading hide];
        [self showAlert:@"Article Not Found" message:@"This article was removed from the stataion"];
        return;
    }
    
    NSDictionary *a = [d objectForKey:@"article"];
    NSString *content = [a objectForKey:@"content"];
    article.content = content;
    
    NSString *img = [a objectForKey:@"image"];
    article.imageUrl = img;
    [self setupHtml];
}

- (void)tweet:(UIBarButtonItem *)btn
{
    NSLog(@"TWEET: %@", article.title);
    Class tweeterClass = NSClassFromString(@"TWTweetComposeViewController");
    if(tweeterClass == nil) {   // check for Twitter integration
        // no Twitter integration; default to third-party Twitter framework
        return;
    }
    
    // check Twitter accessibility and at least one account is setup
    if([TWTweetComposeViewController canSendTweet]==NO) {
        NSLog(@"NO TWITTER ACCOUNT SET UP!");
        [self showAlert:@"No Twitter Account" message:@"Please link your Twitter account in the Settings section of your iPhone."];
        return;
    }

    TWTweetComposeViewController *tweetViewController = [[TWTweetComposeViewController alloc] init];
    [tweetViewController setInitialText:[NSString stringWithFormat:@"Found this on Butterfly Radio:\n%@", article.title]];
    (article.link==TRUE) ? [tweetViewController addURL:[NSURL URLWithString:article.content]] : [tweetViewController addURL:[NSURL URLWithString:[NSString stringWithFormat:@"http://%@/site/article/%@", kUrl, article.unique_id]]];
    
    [tweetViewController addURL:[NSURL URLWithString:kUrl]];
    tweetViewController.completionHandler = ^(TWTweetComposeViewControllerResult result) {
        if(result == TWTweetComposeViewControllerResultDone) { // the user finished composing a tweet
            
        }
        else if(result == TWTweetComposeViewControllerResultCancelled) { // the user cancelled composing a tweet
            
        }
        [self dismissViewControllerAnimated:YES completion:nil];
    };
    [self presentViewController:tweetViewController animated:YES completion:nil];
    [tweetViewController release];
}

- (void)viewComments:(UIBarButtonItem *)btn
{
    ReviewsViewController *reviews = [[ReviewsViewController alloc] initWithMode:ReviewModeArticle];
    reviews.station = station;
    reviews.uniqueID = article.unique_id;
    [self.navigationController pushViewController:reviews animated:YES];
    [reviews release];
}

#pragma mark - UIWebViewDelegate
- (BOOL)webView:(UIWebView *)webView shouldStartLoadWithRequest:(NSURLRequest *)request navigationType:(UIWebViewNavigationType)navigationType
{
    return TRUE;
}

- (void)webViewDidStartLoad:(UIWebView *)webView
{
    
}

- (void)webViewDidFinishLoad:(UIWebView *)webView
{
    [self.loading hide];
}

- (void)webView:(UIWebView *)webView didFailLoadWithError:(NSError *)error
{
    
}


#pragma mark - UIResponder
- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
    NSLog(@"touchesBegan:");
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
    NSLog(@"touchesMoved:");
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
    NSLog(@"touchesEnded");
}

- (void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event
{
    NSLog(@"touchesCancelled");
}



- (void)viewDidUnload
{
    [super viewDidUnload];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return YES;
}

@end
