//
//  SubmitArticleViewController.m
//  butterflyradio
//
//  Created by Denny Kwon on 4/9/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "SubmitArticleViewController.h"

static NSString *file = @"picture.jpg";

@implementation UIImage (scale)
+ (UIImage*)imageWithImage:(UIImage*)img scaledToSize:(CGSize)newSize;
{
    UIGraphicsBeginImageContext( newSize );
    [img drawInRect:CGRectMake(0,0,newSize.width,newSize.height)];
    UIImage* newImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return newImage;
}
@end

@interface SubmitArticleViewController ()
@property (retain, nonatomic) OriginalView *original;
@property (retain, nonatomic) LinkView *linkView;
@property (retain, nonatomic) NSOperationQueue *queue;
@property (retain, nonatomic) BRToolbar *bottomToolbar;
@property (nonatomic) int mode;
@end

@implementation SubmitArticleViewController
@synthesize station;
@synthesize imgPicker;
@synthesize ipAddress;
@synthesize admin;

- (id)initWithManager:(ButterflyManager *)mgr
{
    self = [super initWithManager:mgr];
    if (self) {
        
        self.queue = [[[NSOperationQueue alloc] init] autorelease];
        self.mode = 0;
        self.ipAddress = @"0";
        self.admin = NO;
        
        NSString *filePath = [self createFilePath:file];
        NSError *error = nil;
        [[NSFileManager defaultManager] removeItemAtPath:filePath error:&error]; //this removes files
    }
    return self;
}

- (void)dealloc
{
    self.original = nil;
    self.linkView = nil;
    self.station = nil;
    self.imgPicker = nil;
    self.ipAddress = nil;
    self.queue = nil;
    self.bottomToolbar = nil;
    [super dealloc];
}

- (void)loadView
{
    CGRect frame = [UIScreen mainScreen].applicationFrame;
    frame.origin.y = 0.0f;
    UIViewAutoresizing resize = UIViewAutoresizingFlexibleHeight;
    
    UIView *view = [[UIView alloc] initWithFrame:frame];
    view.autoresizingMask = resize;
    view.backgroundColor = [UIColor blackColor];
    
    ToolBar *toolbar = [[ToolBar alloc] initWithFrame:CGRectMake(0.0f, 0.0f, frame.size.width, 44.0f)];
    toolbar.delegate = self;
    
    UIImage *imgNavBack = [UIImage imageNamed:@"navBack.png"];
    UIButton *back = [UIButton buttonWithType:UIButtonTypeCustom];
    [back setImage:imgNavBack forState:UIControlStateNormal];
    back.frame = CGRectMake(0, 0, 0.65f*imgNavBack.size.width, 0.65f*imgNavBack.size.height);
    [back addTarget:self action:@selector(exit) forControlEvents:UIControlEventTouchUpInside];
    UIBarButtonItem *btnBack = [[UIBarButtonItem alloc] initWithCustomView:back];

    UIBarButtonItem *submit = [[UIBarButtonItem alloc] initWithTitle:@"Send" style:UIBarButtonItemStyleDone target:self action:@selector(submitArticle:)];
    toolbar.items = @[btnBack, [toolbar.items objectAtIndex:1], submit];
    [btnBack release];
    [submit release];
    [view addSubview:toolbar];
    [toolbar release];
    
    UISegmentedControl *typeSegment = [[UISegmentedControl alloc] initWithItems:[NSArray arrayWithObjects:@"Original", @"Link", nil]];
    typeSegment.frame = CGRectMake(80.0f, 8.0f, 160.0f, 30.0f);
    [typeSegment addTarget:self action:@selector(switchView:) forControlEvents:UIControlEventValueChanged];
    typeSegment.selectedSegmentIndex = 0;
    typeSegment.segmentedControlStyle = UISegmentedControlStyleBar;
    typeSegment.tintColor = [UIColor darkGrayColor];
    
    [toolbar addSubview:typeSegment];
    [typeSegment release];
    
    CGFloat y = toolbar.frame.size.height;
    self.original = [[[OriginalView alloc] initWithFrame:CGRectMake(0.0f, y, frame.size.width, frame.size.height-y)] autorelease];
    self.original.titleLabel.text = [NSString stringWithFormat:@"submit original article: %@", self.station.name];
    self.original.delegate = self;
    [view addSubview:self.original];

    self.linkView = [[[LinkView alloc] initWithFrame:self.original.frame] autorelease];
    self.linkView.titleLabel.text = [NSString stringWithFormat:@"submit link: %@", self.station.name];
    self.linkView.hidden = YES;
    [view addSubview:self.linkView];
    
    CGFloat height = 44.0f;
    self.bottomToolbar = [[[BRToolbar alloc] initWithFrame:CGRectMake(0, view.frame.size.height-height+1, view.frame.size.width, height)] autorelease];
    self.bottomToolbar.barStyle = UIBarStyleBlack;
    self.bottomToolbar.translucent = YES;
    self.bottomToolbar.autoresizingMask = (UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleTopMargin);
    
    NSArray *barButtons = nil;
    if (self.admin){
        barButtons = @[[self flex], [self barButtonWithImage:[UIImage imageNamed:@"toolbarPhoto.png"] tag:1001], [self flex]];
    }
    else{
        barButtons = @[[self flex], [self barButtonWithImage:[UIImage imageNamed:@"toolbarSettings.png"] tag:1000], [self flex], [self barButtonWithImage:[UIImage imageNamed:@"toolbarPhoto.png"] tag:1001], [self flex]];
    }
    
//    NSArray *barButtons = @[[self flex], [self barButtonWithImage:[UIImage imageNamed:@"toolbarSettings.png"] tag:1000], [self flex], [self barButtonWithImage:[UIImage imageNamed:@"toolbarPhoto.png"] tag:1001], [self flex]];
    
    self.bottomToolbar.items = barButtons;
    [view addSubview:self.bottomToolbar];


    self.view = view;
    [view release];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    [self setupLoadingIndicator];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    if (self.butterflyMgr.host.loggedIn){
        self.bottomToolbar.lblAccount.text = self.butterflyMgr.host.email;
        self.bottomToolbar.lblTickets.text = [NSString stringWithFormat:@"tickets: %d", self.butterflyMgr.host.tickets];
        return;
    }
    
    self.bottomToolbar.lblAccount.text = @"account: logged out";
    self.bottomToolbar.lblTickets.text = @"tickets: 0";
}


#pragma mark -
- (void)rotateViews:(UIView *)show hidden:(UIView *)hide
{
    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDuration:0.6f];
    [UIView setAnimationTransition:UIViewAnimationTransitionFlipFromLeft forView:hide cache:YES];
    hide.hidden = YES;
    [UIView commitAnimations];
    
    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDuration:0.6f];
    [UIView setAnimationTransition:UIViewAnimationTransitionFlipFromLeft forView:show cache:YES];
    show.hidden = NO;
    [UIView commitAnimations];
}

- (void)barButtonTapped:(UIBarButtonItem *)btn
{
    
    if (btn.tag==1000){ // login, account
        [self showAccount];
    }

    if (btn.tag==1001){
        if (self.mode==0){ //original
            UIActionSheet *actionsheet = [[UIActionSheet alloc] initWithTitle:@"Add Photo" delegate:self cancelButtonTitle:@"Cancel" destructiveButtonTitle:nil otherButtonTitles:@"Camera", @"Photo Library", nil];
            actionsheet.frame = CGRectMake(0, 150, self.view.frame.size.width, 100);
            actionsheet.actionSheetStyle = UIActionSheetStyleBlackOpaque;
            [actionsheet showInView:[UIApplication sharedApplication].keyWindow];
            [actionsheet release];
            return;
        }
        
        [self showAlert:@"No Image" message:@"You can only attach an image to original posts."];
    }
    
}


- (void)switchView:(UISegmentedControl *)sender
{
    NSLog(@"switchView");
    self.mode = sender.selectedSegmentIndex;
    (self.mode==0) ? [self rotateViews:self.original hidden:self.linkView] : [self rotateViews:self.linkView hidden:self.original];
}

- (void)upload:(NSString *)uploadString filePath:(NSString *)path
{
    ASIFormDataRequest *request = [[[ASIFormDataRequest alloc] initWithURL:[NSURL URLWithString:uploadString]] autorelease];
    request.delegate = self;
    [request setPostValue:[self.station adminsString] forKey:@"admins"];
    [request setPostValue:self.station.unique_id forKey:@"station"];
    
    
    if (self.mode==0){ 
        if (path==nil){ //no image included
            [request setPostValue:@"no" forKey:@"image"];
        }
        else{ //include image
            [request setPostValue:@"yes" forKey:@"image"];
            [request setFile:path forKey:@"file"];
        }
        [request setPostValue:self.original.titleField.text forKey:@"title"];
        [request setPostValue:@"tag1,tag2,tag3" forKey:@"tags"];
        [request setPostValue:self.original.contentField.text forKey:@"content"];
        [request setPostValue:@"no" forKey:@"link"];
        [request setPostValue:self.original.authorField.text forKey:@"author"];
    }
    else{ 
        [request setPostValue:@"no" forKey:@"image"];
        [request setPostValue:self.linkView.titleField.text forKey:@"title"];
        [request setPostValue:@"tag1,tag2,tag3" forKey:@"tags"];
        [request setPostValue:self.linkView.contentField.text forKey:@"content"];
        [request setPostValue:@"yes" forKey:@"link"];
        [request setPostValue:self.linkView.authorField.text forKey:@"author"];
    }

    [request startAsynchronous];
    self.loading.titleLabel.text = @"Sending Article...";
    [self.loading show];
}

- (void)showMissingValueAlert
{
    [self showAlert:@"Missing Value" message:@"Please complete all fields (image optional)."];
}

- (void)submitArticle:(UIBarButtonItem *)btn
{
    NSLog(@"submitArticle:");
    if (self.mode==0){ //original 
        if ([self.original.titleField.text length]==0 || [self.original.contentField.text length]==0 || [self.original.authorField.text length]==0){
            [self showMissingValueAlert];
            return;
        }
        NSString *filePath = [self createFilePath:file];
        NSData *imgData = [NSData dataWithContentsOfFile:filePath];
        if (imgData==nil){
            [self upload:@"http://www.butterflyradio.com/api/article" filePath:nil];
            return;
        }
        
        NSString *url = [NSString stringWithFormat:@"http://%@/api/upload?type=article", kUrl];
        self.request = [BRNetworkOp operationWithAddress:url parameters:nil];
        [self.request setHttpMethod:@"GET"];
        self.request.delegate = self;
        [self.request sendRequest];
        
        self.loading.titleLabel.text = @"Sending Article...";
        [self.loading show];
        return;
    }
    
    //link:
    if ([self.linkView.titleField.text length]==0 || [self.linkView.contentField.text length]==0 || [self.linkView.authorField.text length]==0){
        [self showMissingValueAlert];
        return;
    }
    
    [self upload:@"http://www.butterflyradio.com/api/article" filePath:nil];
}

- (void)requestData:(NSArray *)pkg //returns [address, data]
{
    if (!pkg)
        return;
    
    NSString *json = [[NSString alloc] initWithData:[pkg lastObject] encoding:NSUTF8StringEncoding];
    NSDictionary *d = [json JSONValue];
    [json release];
    if (d==nil)
        return;

    d = [d objectForKey:@"results"];
    NSLog(@"%@", [d description]);
    NSString *confirmation = [d objectForKey:@"confirmation"];
    if ([confirmation isEqualToString:@"success"]){
        NSString *url = [d objectForKey:@"upload string"];
        [self upload:url filePath:[self createFilePath:file]];
        self.ipAddress = [d objectForKey:@"IP"];
    }
}

- (void)cameraReady:(UIImagePickerController *)picker
{
    NSLog(@"cameraReady:");

    [self.loading hide];
    if (!picker)
        return;
    
    self.imgPicker = picker;
    imgPicker.delegate = self;
    [self presentModalViewController:self.imgPicker animated:NO];
}

#pragma mark -  UIActionSheetDelegate
- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex
{
    NSLog(@"actionSheet clickedButtonAtIndex: %d", buttonIndex);
    if (buttonIndex==2) //cancel button
        return;
    
    UIImagePickerControllerSourceType source;
    if (buttonIndex==0){ // camera
        source = UIImagePickerControllerSourceTypeCamera;
    }
    if (buttonIndex==1){
        source = UIImagePickerControllerSourceTypePhotoLibrary;
    }
    
    LauchImagePicker *camera = [[LauchImagePicker alloc] initWithTarget:self
                                                                 action:@selector(cameraReady:)
                                                             sourceType:source];
    
    [self.queue addOperation:camera];
    [camera release];
    self.loading.titleLabel.text = @"Launching Camera...";
    [self.loading show];
}

#pragma mark - ImagePickerDelegate
- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary *)info
{
    NSLog(@"picker didFinishPickingMediaWithInfo: %@", [info description]);
    UIImagePickerControllerSourceType source = picker.sourceType;
    [picker dismissModalViewControllerAnimated:NO];

    UIImage *img = (source==UIImagePickerControllerSourceTypeCamera) ? [info objectForKey:@"UIImagePickerControllerEditedImage"] : [info objectForKey:@"UIImagePickerControllerOriginalImage"];

    CGFloat width = img.size.width;
    CGFloat height = img.size.height;
    CGFloat max = self.original.image.frame.size.height;
    double scale;
    
    if (height > max){
        scale = max/height;
        height = max;
        width *= scale;
    }
    
    self.original.image.image = img;
    CGRect frame = self.original.image.frame;
    frame.size.width = width;
    frame.size.height = height;
    self.original.image.frame = frame;
    
    NSString *filePath = [self createFilePath:file];
    
    if (source==UIImagePickerControllerSourceTypeCamera){
        img = [UIImage imageWithImage:img scaledToSize:CGSizeMake(640.0f, 640.0f)];
        NSData *imgData = UIImageJPEGRepresentation(img, 0.45f);
        [imgData writeToFile:filePath atomically:YES];
        return;
    }
    
    width = img.size.width;
    height = img.size.height;
    max = 640.0f;
    if (height > max){
        scale = max/height;
        height = max;
        width *= scale;
    }
    if (width > max){
        scale = max/width;
        width = max;
        height *= scale;
    }
    
    img = [UIImage imageWithImage:img scaledToSize:CGSizeMake(width, height)];
    NSData *imgData = UIImageJPEGRepresentation(img, 1.0f);
    [imgData writeToFile:filePath atomically:YES];
    
    NSLog(@"WIDTH = %.2f, HEIGHT  = %.2f, FILE SIZE = %d", img.size.width, img.size.height, imgData.length);
}

- (void)imagePickerControllerDidCancel:(UIImagePickerController *)picker
{
    [self.imgPicker dismissModalViewControllerAnimated:NO];
}



#pragma mark - ToolBar
- (void)exit
{
    [self dismissModalViewControllerAnimated:YES];
}

- (void)showRadio
{
    
}


#pragma mark - ASIHTTPRequestDelegate
- (void)requestFinished:(ASIHTTPRequest *)request
{
    NSString *json = [request responseString];
    NSLog(@"RECORD VIEW CONTROLLER - requestFinished: %@", json);
    
    [self.loading hide];
    NSDictionary *d = [json JSONValue];
    if (d==nil){
        [self showAlert:@"Error" message:@"There was an error. Please try again."];
        return;
    }
    
    d = [d objectForKey:@"results"];
    NSLog(@"%@", [d description]);
    NSString *confirmation = [d objectForKey:@"confirmation"];
    if ([confirmation isEqualToString:@"success"]==NO){
        [self showAlert:@"Error" message:@"There was an error. Please try again."];
        return;
    }
    [self showAlert:@"Article Submitted" message:@"Your article was submitted. An admin will review it shortly. Thank you."];
    [self dismissModalViewControllerAnimated:YES];
}




- (void)viewDidUnload
{
    [super viewDidUnload];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

- (NSUInteger)supportedInterfaceOrientations
{
    return (UIInterfaceOrientationMaskPortrait | UIInterfaceOrientationMaskPortraitUpsideDown);
}



@end
