//
//  BRPurchaseViewController.m
//  butterflyradio
//
//  Created by Denny Kwon on 12/2/12.
//  Copyright (c) 2012 com.thegridmedia. All rights reserved.
//

#import "BRPurchaseViewController.h"
#import "Globals.h"

@interface BRPurchaseViewController ()
@property (retain, nonatomic) UIImageView *bgImage;
@property (retain, nonatomic) UIView *mainView;
@property (retain, nonatomic) BRGradientBar *gradientBar;
@property (retain, nonatomic) UIView *btnView;
@property (copy, nonatomic) NSString *purchasing;  // product ID of the item currently being purchased
@end

#define kOriginY 160.0f

@implementation BRPurchaseViewController
@synthesize selected;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        self.signal = [SignalCheck signalWithDelegate:self];
    }
    return self;
}

- (id)initWithBackgroundImage:(UIImage *)img
{
    self = [super init];
    if (self) {
        self.signal = [SignalCheck signalWithDelegate:self];
        self.bgImage = [[[UIImageView alloc] initWithImage:img] autorelease];
    }
    return self;
}

- (void)dealloc
{
    self.bgImage = nil;
    self.mainView = nil;
    self.gradientBar = nil;
    self.btnView = nil;
    self.purchasing = nil;
    [super dealloc];
}

- (void)loadView
{
    UIView *view = [self baseViewWithColor:[UIColor blackColor]];
    CGRect frame = view.frame;
    
    CGRect statusBarFrame = [UIApplication sharedApplication].statusBarFrame;
    if (statusBarFrame.size.width != 0){ // status bar not hidden
        CGRect bgImageFrame = self.bgImage.frame;
        bgImageFrame.origin.y = -20.0f; // adjust for status bar.
        self.bgImage.frame = bgImageFrame;
    }

    
    [view addSubview:self.bgImage];
    
    frame.origin.y = frame.size.height;
    self.mainView = [[[UIView alloc] initWithFrame:frame] autorelease];
    self.mainView.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageNamed:@"bg_blue.png"]];
    
    self.gradientBar = [BRGradientBar barWithFrame:CGRectMake(0, kOriginY, frame.size.width, 0)];
    self.gradientBar.delegate = self;
    self.gradientBar.lblTitle.text = @"Tickets";
    
    CGFloat padding = 10.0f;
    CGFloat y = self.gradientBar.frame.origin.y+self.gradientBar.frame.size.height;
    UIScrollView *theScrollview = [[UIScrollView alloc] initWithFrame:CGRectMake(0, y, frame.size.width, frame.size.height-y)];
    theScrollview.backgroundColor = [UIColor lightGrayColor];
    theScrollview.alpha = 0.6f;
    theScrollview.contentSize = CGSizeMake(0, 500);
    
    UILabel *lblDescTitle = [[UILabel alloc] initWithFrame:CGRectMake(0.0f, 0.0f, frame.size.width, 30.0f)];
    lblDescTitle.backgroundColor = [UIColor darkGrayColor];
    lblDescTitle.textAlignment = NSTextAlignmentCenter;
    lblDescTitle.text = @"Premium Conversations";
    [theScrollview addSubview:lblDescTitle];
    [lblDescTitle release];
    
    NSString *path = [[NSBundle mainBundle] pathForResource:@"purchase" ofType:@"txt"];
    NSError *error = nil;
    NSString *text = [[NSString alloc] initWithContentsOfFile:path encoding:NSUTF8StringEncoding error:&error];
    
    UILabel *lblDescription = [[UILabel alloc] initWithFrame:CGRectZero];
    lblDescription.font = [UIFont fontWithName:kFont size:14.0f];
    lblDescription.backgroundColor = [UIColor clearColor];
    lblDescription.textColor = [UIColor whiteColor];
    lblDescription.lineBreakMode = NSLineBreakByWordWrapping;
    lblDescription.numberOfLines = 0;
    lblDescription.text = text;
    [text release];
    CGSize size = [text sizeWithFont:lblDescription.font constrainedToSize:CGSizeMake(frame.size.width-2*padding, 500.0f) lineBreakMode:lblDescription.lineBreakMode];
    lblDescription.frame = CGRectMake(padding, 40.0f, frame.size.width-2*padding, size.height);
    [theScrollview addSubview:lblDescription];
    [lblDescription release];
    theScrollview.contentSize = CGSizeMake(0, lblDescription.frame.size.height+84.0f);

    
    [self.mainView addSubview:theScrollview];
    [theScrollview release];
    
    [self.mainView addSubview:self.gradientBar];
    
    self.btnView = [[UIView alloc] initWithFrame:CGRectMake(0, y, frame.size.width, frame.size.height-y)];
    self.btnView.backgroundColor = kBRYellow;
    
    CGRect f = CGRectMake(15.0f, 20.0f, frame.size.width-30.0f, 0.0f);
    UILabel *btnViewLabel = [[UILabel alloc] initWithFrame:CGRectZero];
    btnViewLabel.backgroundColor = [UIColor clearColor];
    btnViewLabel.numberOfLines = 0;
    btnViewLabel.textColor = [UIColor darkGrayColor];
    btnViewLabel.lineBreakMode = NSLineBreakByWordWrapping;
    btnViewLabel.textAlignment = NSTextAlignmentCenter;
    btnViewLabel.font = [UIFont fontWithName:@"Heiti SC" size:14.0f];
    btnViewLabel.text = @"Purchase tickets to enter premium conversations. Once you enter, you do not need tickets to enter again. Each subseqent post in the conversation is included with the purchase.";
    size = [btnViewLabel.text sizeWithFont:btnViewLabel.font constrainedToSize:CGSizeMake(f.size.width, 150.0f) lineBreakMode:btnViewLabel.lineBreakMode];
    f.size.height = size.height;
    btnViewLabel.frame = f;
    
    UIImage *img = [UIImage imageNamed:@"bg_trackinfo.png"];
    UIImageView *bg = [[UIImageView alloc] initWithImage:img];
    bg.frame = CGRectMake(10, 10, frame.size.width-20, btnViewLabel.frame.size.height+20);
    [self.btnView addSubview:bg];
    [bg release];

    [self.btnView addSubview:btnViewLabel];
    [btnViewLabel release];
    
    UIImage *imgBtn = [UIImage imageNamed:@"btn_black.png"];
    CGFloat w = imgBtn.size.width;
    CGFloat h = imgBtn.size.height;
    padding = (frame.size.width-2*imgBtn.size.width)/3.0f;
    
    NSArray *ticketPacks = @[@"5 Tickets\n$4.99", @"10 Tickets\n$8.99"];
    for (int i=0; i<ticketPacks.count; i++){
        NSString *btnTitle = [ticketPacks objectAtIndex:i];
        UIButton *btnTix = [UIButton buttonWithType:UIButtonTypeCustom];
        btnTix.tag = 1000+i;
        btnTix.titleLabel.numberOfLines = 2;
        btnTix.titleLabel.lineBreakMode = NSLineBreakByWordWrapping;
        btnTix.titleLabel.textAlignment = NSTextAlignmentCenter;
        btnTix.frame = CGRectMake(padding+i*(padding+w), self.btnView.frame.size.height-h-85.0, w, h);
        btnTix.titleLabel.font = [UIFont fontWithName:@"Heiti SC" size:14.0f];
        [btnTix setBackgroundImage:imgBtn forState:UIControlStateNormal];
        [btnTix setTitle:btnTitle forState:UIControlStateNormal];
        [btnTix addTarget:self action:@selector(ticketsBtnAction:) forControlEvents:UIControlEventTouchUpInside];
        [self.btnView addSubview:btnTix];
    }

    imgBtn = [UIImage imageNamed:@"btn_blue.png"];
    w = imgBtn.size.width;
    h = imgBtn.size.height;

    UIButton *btnCancel = [UIButton buttonWithType:UIButtonTypeCustom];
    [btnCancel addTarget:self action:@selector(exit) forControlEvents:UIControlEventTouchUpInside];
    btnCancel.showsTouchWhenHighlighted = YES;
    btnCancel.frame = CGRectMake(padding, self.btnView.frame.size.height-70.0, frame.size.width-2*padding, h);
    btnCancel.autoresizingMask = UIViewAutoresizingFlexibleTopMargin;
    [btnCancel setBackgroundImage:imgBtn forState:UIControlStateNormal];
    btnCancel.titleLabel.font = [UIFont fontWithName:@"Heiti SC" size:14.0f];
    [btnCancel setTitle:@"Back" forState:UIControlStateNormal];
    [self.btnView addSubview:btnCancel];
    
    UIImage *shadow = [UIImage imageNamed:@"dropShadow.png"];
    UIImageView *imgShadow = [[UIImageView alloc] initWithImage:shadow];
    imgShadow.frame = CGRectMake(0, 0, shadow.size.width, shadow.size.height);
    [self.btnView addSubview:imgShadow];
    [imgShadow release];
    
    [self.mainView addSubview:self.btnView];

    [view addSubview:self.mainView];

    self.view = view;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.purhcaseManager.delegate = self;
    [self setupLoadingIndicator:1 verticalOffset:160.0f];

    
    CGRect frame = self.mainView.frame;
    frame.origin.y = 0.0f;
    [UIView animateWithDuration:0.5f
                          delay:0.0f
                        options:UIViewAnimationOptionCurveLinear
                     animations:^{
                         self.mainView.frame = frame;
                         self.bgImage.transform = CGAffineTransformMakeScale(0.9f, 0.9f);
                         
                     }
                     completion:^(BOOL finisehd){
                         UIBarButtonItem *btnBack = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemCancel target:self action:@selector(exit)];
                         btnBack.tintColor = [UIColor redColor];
                         self.navigationItem.leftBarButtonItem = btnBack;
                         [btnBack release];
                     }];
}

- (void)exit
{
    CGRect frame = self.mainView.frame;
    frame.origin.y = self.view.frame.size.height;
    [UIView animateWithDuration:0.5f
                          delay:0.0f
                        options:UIViewAnimationOptionCurveLinear
                     animations:^{
                         self.mainView.frame = frame;
                         self.bgImage.transform = CGAffineTransformIdentity;
                         
                     }
                     completion:^(BOOL finished){
                         [self dismissModalViewControllerAnimated:NO];
                     }];

}

- (void)ticketsBtnAction:(UIButton *)btn
{
    //check signal
    if (![self.signal checkSignal]){
        [self showAlert:@"No Connection" message:@"Please find an internet connection."];
        return;
    }
    
    int tag = btn.tag;
    NSLog(@"ticketsBtnAction: %d", tag); // 1000=5 tix, 1001=10 tix
    self.purchasing = (tag==1000) ? k5TicketsProductId : k10TicketsProductId;
    
    SKProduct *skProduct = nil;
    for (SKProduct *p in self.purhcaseManager.SKProductsArray){
        if (tag==1000){
//            NSLog(@"5 TICKETS");
            if ([p.productIdentifier isEqualToString:k5TicketsProductId]){
                skProduct = p;
                break;
            }
        }
        if (tag==1001){
//            NSLog(@"10 TICKETS");
            if ([p.productIdentifier isEqualToString:k10TicketsProductId]){
                skProduct = p;
                break;
            }
        }
    }
    
    if (!skProduct)
        return;
    
    [self.purhcaseManager purchase:skProduct];
    [self.loading show];
}


- (void)shiftUp:(NSTimeInterval)duration
{
    [self.gradientBar rotateButton:CGAffineTransformIdentity];
    [UIView animateWithDuration:duration
                          delay:0.0f
                        options:UIViewAnimationOptionCurveLinear
                     animations:^{
                         CGRect frame = self.gradientBar.frame;
                         frame.origin.y = kOriginY;
                         self.gradientBar.frame = frame;
                         
                         frame = self.btnView.frame;
                         frame.origin.y = self.gradientBar.frame.origin.y+self.gradientBar.frame.size.height;
                         self.btnView.frame = frame;
                     }
                     completion:^(BOOL finished){
                         [UIView animateWithDuration:0.07f
                                               delay:0.0f
                                             options:UIViewAnimationOptionCurveLinear
                                          animations:^{
                                              CGRect frame = self.gradientBar.frame;
                                              frame.origin.y += 10.0;
                                              self.gradientBar.frame = frame;
                                              
                                              frame = self.btnView.frame;
                                              frame.origin.y += 10.0f;
                                              self.btnView.frame = frame;
                                          }
                                          completion:^(BOOL finished){
                                              [UIView animateWithDuration:0.07f
                                                                    delay:0.0f
                                                                  options:UIViewAnimationOptionCurveLinear
                                                               animations:^{
                                                                   CGRect frame = self.gradientBar.frame;
                                                                   frame.origin.y = kOriginY;
                                                                   self.gradientBar.frame = frame;
                                                                   
                                                                   frame = self.btnView.frame;
                                                                   frame.origin.y = self.gradientBar.frame.origin.y+self.gradientBar.frame.size.height;
                                                                   self.btnView.frame = frame;
                                                               }
                                                               completion:NULL];
                                          }];
                     }];
    
}

- (void)shiftDown:(NSTimeInterval)duration
{
    [self.gradientBar rotateButton:CGAffineTransformMakeRotation(M_PI)];
    [UIView animateWithDuration:duration
                          delay:0.0f
                        options:UIViewAnimationOptionCurveEaseInOut
                     animations:^{
                         CGRect frame = self.btnView.frame;
                         frame.origin.y = self.view.frame.size.height;
                         self.btnView.frame = frame;
                         
                         frame = self.gradientBar.frame;
                         frame.origin.y = self.view.frame.size.height-frame.size.height;
                         self.gradientBar.frame = frame;
                     }
                     completion:^(BOOL finished){
                         [UIView animateWithDuration:0.07
                                               delay:0.0f
                                             options:UIViewAnimationOptionCurveLinear
                                          animations:^{

                                              CGRect frame = self.btnView.frame;
                                              frame.origin.y -= 10;
                                              self.btnView.frame = frame;
                                              
                                              frame = self.gradientBar.frame;
                                              frame.origin.y -= 10;
                                              self.gradientBar.frame = frame;

                                          }
                                          completion:^(BOOL finished){
                                              [UIView animateWithDuration:0.07
                                                                    delay:0.0f
                                                                  options:UIViewAnimationOptionCurveLinear
                                                               animations:^{
                                                                   CGRect frame = self.btnView.frame;
                                                                   frame.origin.y = self.view.frame.size.height;
                                                                   self.btnView.frame = frame;
                                                                   
                                                                   frame = self.gradientBar.frame;
                                                                   frame.origin.y = self.view.frame.size.height-frame.size.height;
                                                                   self.gradientBar.frame = frame;
                                                               }
                                                               completion:NULL];
                                          }];
                     }];
}


#pragma mark - PurchaseManagerDelegate
- (void)productListReady
{
    
}

- (void)purchaseFailed:(NSError *)error
{
    NSLog(@"PURCHASE FAILED: %@", [error localizedDescription]);
    [self.loading hide];
    
    [self showAlert:@"Error" message:@"There was an error. Your account was not charged. Please try again. "];
}

- (void)purchaseComplete:(BOOL)confirmation productID:(NSString *)pId reciept:(NSDictionary *)d
{
    NSLog(@"PURCHASE COMPLETE: %@, %@", pId, [d description]);
    NSString *value = ([self.purchasing isEqualToString:k5TicketsProductId]) ? @"5" : @"10";

    // update the local version right away just incase the request fails:
    self.butterflyMgr.host.tickets += [value intValue];
    
    
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    [defaults setObject:@[self.butterflyMgr.host.email, value] forKey:kTicketsPurchaseKey];
    [defaults synchronize];

    
    NSString *url = [NSString stringWithFormat:@"http://%@/api/host?email=%@", kUrl, self.butterflyMgr.host.email];
    NSDictionary *params = @{@"action" : @"tickets", @"value" : value};
    self.request = [BRNetworkOp operationWithAddress:url parameters:params];
    [self.request setHttpMethod:@"POST"];
    self.request.delegate = self;
    [self.request sendRequest];
    
    
    
    /*
     
     PURCHASE COMPLETE: com.butterflyradio.5ticket, {
     bid = "com.butterflyradio.ios";
     bvrs = "1.5";
     "item_id" = 608076321;
     "original_purchase_date" = "2013-02-26 00:57:07 Etc/GMT";
     "original_purchase_date_ms" = 1361840227967;
     "original_purchase_date_pst" = "2013-02-25 16:57:07 America/Los_Angeles";
     "original_transaction_id" = 1000000066167096;
     "product_id" = "com.butterflyradio.5ticket";
     "purchase_date" = "2013-02-26 00:57:07 Etc/GMT";
     "purchase_date_ms" = 1361840227967;
     "purchase_date_pst" = "2013-02-25 16:57:07 America/Los_Angeles";
     quantity = 1;
     "transaction_id" = 1000000066167096;
     "unique_identifier" = 78185590cfdfdc8a0a876c839634d17a80a56df5;
     "unique_vendor_identifier" = "76C44DCB-5631-4B1F-97F5-5122F5DC6983";
     }
     
     */
    
}

#pragma mark - BRNetworkOpDelegate
- (void)requestData:(NSArray *)pkg //returns [address, data]
{
    [self.loading hide];
    
    if (!pkg)
        return;
    
    NSDictionary *results = [self.request parseJsonResponse];
    results = [results objectForKey:@"results"];
    NSLog(@"%@", [results description]);
    NSString *confirmation = [results objectForKey:@"confirmation"];
    if ([confirmation isEqualToString:@"success"]==NO){
        
        // show alert
        return;
    }
    
    
    NSDictionary *h = [results objectForKey:@"host"];
    [self.butterflyMgr.host populate:h]; // repopulate host object
    
    self.request.delegate = nil;
    self.request = nil;
    
    [self dismissModalViewControllerAnimated:YES];
}

#pragma mark - GradientBarDelegate
- (void)btnGradientTapped:(UIButton *)btn
{
    NSLog(@"btnGradientTapped:");
    (self.gradientBar.center.y < 280) ? [self shiftDown:0.2f] : [self shiftUp:0.2f];
}

- (void)gradientBarMoved:(double)diff
{
    NSLog(@"gradientBarMoved:");
    CGPoint ctr = self.btnView.center;
    ctr.y += diff;
    self.btnView.center = ctr;
}

- (void)gradientBarStopped
{
    NSLog(@"gradientBarStopped: %.2f", self.gradientBar.center.y);
    (self.gradientBar.center.y > 280) ? [self shiftDown:0.2f] : [self shiftUp:0.2f];
}

- (void)swipe:(double)direction
{
    NSLog(@"SWIPE: %.2f", direction); // if dirction>0, sliding down:
    (direction > 0) ? [self shiftDown:0.15f] : [self shiftUp:0.15f];
}


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

@end
