//
//  PurchaseManager.m
//  HeatMoment
//
//  Created by Denny Kwon on 11/15/12.
//  Copyright (c) 2012 Teju Prasad. All rights reserved.
//

#import "PurchaseManager.h"

@implementation PurchaseManager
@synthesize SKProductsArray;
@synthesize delegate;
@synthesize productsRequest;

static PurchaseManager *manager;

- (id)init
{
    self = [super init];
    if (self){
        purchasing = FALSE;
        [[SKPaymentQueue defaultQueue] addTransactionObserver:self];
    }
    return self;
}

+ (PurchaseManager *)manager
{
    if (!manager){
        manager = [[[PurchaseManager alloc] init] autorelease];
    }
    
    return manager;
}

- (void)dealloc
{
    self.delegate = nil;
    [[SKPaymentQueue defaultQueue] removeTransactionObserver:self];
    [self.productsRequest cancel];
    self.productsRequest.delegate = nil;
    self.productsRequest = nil;
    self.SKProductsArray = nil;
    if (manager){
        [manager release];
        manager = nil;
    }
    [super dealloc];
}

- (void)getProductList:(NSArray *)list
{
    if (list.count==0)
        return;
    
    // declare a new set, then put objects into the set
    NSMutableSet *set = [NSMutableSet set];
    
    for (NSString *productID in list) {
        [set addObject:productID];
    }
    
    productsRequest = [[SKProductsRequest alloc] initWithProductIdentifiers:set];
    productsRequest.delegate = self;
    [productsRequest start];
}


- (void)purchase:(SKProduct *)prod
{
    NSLog(@"PURCHASE MANAGER - purchase:");
    if(purchasing == YES)
        return;

    
    if ([SKPaymentQueue canMakePayments]==FALSE){
        NSLog(@"Cannot make purchases");
        return;
    }
    
    if (prod==nil){
        //            [self showAlert:@"Error" message:@"There was an error. Please try again later."];
        return;
    }
    
    SKPayment *payment = [SKPayment paymentWithProduct:prod];
    [[SKPaymentQueue defaultQueue] addPayment:payment];
    purchasing = YES;
}



-(void)errorWithTransaction:(SKPaymentTransaction *)transaction
{
    NSLog(@"PURCHASE MANAGER - errorWithTransaction:");
    [[SKPaymentQueue defaultQueue] finishTransaction:transaction];
    purchasing = FALSE;
    
    [delegate purchaseFailed:transaction.error];
}

-(void)finishedTransaction:(SKPaymentTransaction *)transaction
{
    NSLog(@"PURCHASE MANAGER - finishedTransaction:");
    [[SKPaymentQueue defaultQueue] finishTransaction:transaction];
    
    
    // save receipt
    //    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    //    NSString *receipt = [NSString stringWithFormat:@"RECEIPT:%@", self.selected.productId];
    //    [defaults setObject:transaction.transactionIdentifier forKey:receipt];
    //    [defaults synchronize];
    
    // check receipt
    [self checkReceipt:transaction.transactionReceipt];
    purchasing = FALSE;
}


-(void)checkReceipt:(NSData *)receipt
{
    NSLog(@"PURCHASE MANAGER - checkReceipt:");
    
    ReceiptCheck *checker = [[ReceiptCheck alloc] init];
    checker.delegate = self;
    checker.receiptData = receipt;
    //    [checker checkReceipt];
    [checker checkReceipt:NO];
}



//- (void)receiptReceived:(BOOL)status prodID:(NSString *)pID
- (void)receiptReceived:(BOOL)status prodID:(NSString *)pID reciept:(NSDictionary *)d
{
    if (status==FALSE){
        NSLog(@"PURCHASE MANAGER - receiptReceived: ERROR");
        //        - (void)purchaseComplete:(BOOL)confirmation productID:(NSString *)pId;
        [self.delegate purchaseComplete:status productID:pID reciept:d];
        return;
    }
    
    
//    NSLog(@"PURCHASE MANAGER - receiptReceived: SUCCESS! %@", [d description]);
    [self.delegate purchaseComplete:status productID:pID reciept:d];
}




#pragma mark - SKProductsRequestDelegate
-(void)productsRequest:(SKProductsRequest *)request didReceiveResponse:(SKProductsResponse *)response
{
    NSLog(@"PURCHASE MANAGER - productsRequest: didReceiveResponse:");

    for(SKProduct *product in response.products) {
        self.SKProductsArray = response.products;
        
        NSLog(@"PURCHASE MANAGER - PRODUCT: %@ -- %.2f", product.productIdentifier, [product.price doubleValue]);
        
    }
    
    //Just in case one or more of them didn't work:
    for (NSString *invalidProductId in response.invalidProductIdentifiers) {
        NSLog(@"PURCHASE MANAGER - Invalid product id: %@" , invalidProductId);
    }
    
    if ([delegate respondsToSelector:@selector(productListReady)])
        [delegate productListReady];
    
}



#pragma mark - SKPaymentTransactionObserver
-(void)paymentQueue:(SKPaymentQueue *)queue updatedTransactions:(NSArray *)transactions
{
    for(SKPaymentTransaction *transaction in transactions) {
        switch (transaction.transactionState) {
            case SKPaymentTransactionStateFailed:
                NSLog(@"updatedTransactions: FAILED: %@", [transaction.error localizedDescription]);
                [self errorWithTransaction:transaction];
                break;
                
            case SKPaymentTransactionStatePurchasing:
                NSLog(@"PURCHASE MANAGER - updatedTransactions: PURCHASING");
                break;
                
            case SKPaymentTransactionStatePurchased:
                NSLog(@"updatedTransactions: PURCHASED");
                [self finishedTransaction:transaction];
                break;
                
            case SKPaymentTransactionStateRestored:
                NSLog(@"updatedTransactions: RESTORED");
                [self finishedTransaction:transaction];
                break;
                
            default:
                break;
        }
    }
}

-(void)paymentQueueRestoreCompletedTransactionsFinished:(SKPaymentQueue *)queue
{
    NSLog(@"paymentQueueRestoreCompletedTransactionsFinished");
}


@end