//
//  ReceiptCheck.m
//  Newsstand
//
//  Created by Carlo Vigiani on 29/Oct/11.
//  Copyright (c) 2011 viggiosoft. All rights reserved.
//

#import "ReceiptCheck.h"
#import "NSString+Base64.h"
#import "Globals.h"


@implementation ReceiptCheck
@synthesize delegate;

@synthesize receiptData,completionBlock;

+(ReceiptCheck *)validateReceiptWithData:(NSData *)_receiptData completionHandler:(void(^)(BOOL,NSString *))handler
{
    ReceiptCheck *checker = [[ReceiptCheck alloc] init];
    checker.receiptData = _receiptData;
    checker.completionBlock = handler;
    [checker checkReceipt:YES];
    return checker;
    
}

-(void)checkReceipt:(BOOL)live
{
    // verifies receipt with Apple
    NSError *jsonError = nil;
    NSString *receiptBase64 = [NSString base64StringFromData:receiptData length:[receiptData length]];
    
    //NSString *jsonRequest=[NSString stringWithFormat:@"{\"receipt-data\":\"%@\"}",receiptBase64];
    //NSLog(@"Sending this JSON: %@",jsonRequest);
    NSData *jsonData = [NSJSONSerialization dataWithJSONObject:[NSDictionary dictionaryWithObjectsAndKeys:
                                                                receiptBase64,@"receipt-data",
                                                                kSHARED_SECRET,@"password",
                                                                nil]
                                                       options:NSJSONWritingPrettyPrinted
                                                         error:&jsonError
                        ];
    
//    NSLog(@"JSON: %@",[[[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding] autorelease]);
    
    // URL for sandbox receipt validation; replace "sandbox" with "buy" in production or you will receive error codes 21006 or 21007
    
    NSString *url = (live) ? [NSString stringWithFormat:@"%@", kItunesUrl] : [NSString stringWithFormat:@"%@", kSandboxItunesUrl];
    
//    NSString *url = nil;
//    if (live){ // live
//        url = [NSString stringWithFormat:@"%@", kItunesUrl];
//    }
//    else{ // sandbox
//        url = [NSString stringWithFormat:@"%@", kSandboxItunesUrl];
//    }
    
    NSMutableURLRequest *req = [[NSMutableURLRequest alloc] initWithURL:[NSURL URLWithString:url]];
    [req setHTTPMethod:@"POST"];
    [req setHTTPBody:jsonData];
    NSURLConnection *conn = [[NSURLConnection alloc] initWithRequest:req delegate:self];
    if(conn) {
        receivedData = [[NSMutableData alloc] init];
    }
    else {
        NSLog(@"Cannot create connection");
    }
}


#pragma mark - NSURLConnectionDelegate:
-(void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
    NSLog(@"connection didFailWithError: %@", [error localizedDescription]);
}

-(void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
    [receivedData setLength:0];
}

-(void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
    [receivedData appendData:data];
}

-(void)connectionDidFinishLoading:(NSURLConnection *)connection
{
    NSError *error = nil;
    id jsonObject = [NSJSONSerialization JSONObjectWithData:receivedData options:NSJSONReadingMutableContainers error:&error];
    if (error != nil) { //! Handle error
        NSLog(@"Error with json %@", [error localizedDescription]);
        return;
    }

    
    NSDictionary *d = (NSDictionary *)jsonObject;
    if (!d)
        return;
    
    
//    NSLog(@"RECEIPT CHECK - iTunes response: %@",[d description]);
    int status = [[d objectForKey:@"status"] intValue];
    NSLog(@"STATUS = %d", status);
    
    if (status==21007){
        [self checkReceipt:NO]; // try sandbox
        return;
    }
    
    if (status!=0){
        [self.delegate receiptReceived:FALSE prodID:@"none" reciept:d];
        return;
    }
    
    d = [d objectForKey:@"receipt"];
    NSString *prodId = [d objectForKey:@"product_id"];
    [self.delegate receiptReceived:TRUE prodID:prodId reciept:d];
    //if everything checks out okay, clear for download

    
}


@end
