//
//  RadioViewController.m
//  butterflyradio
//
//  Created by Denny Kwon on 5/15/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "RadioViewController.h"
#import "BRTrackCell.h"
#import "BRRedditViewController.h"

@interface RadioViewController ()
@property (retain, nonatomic) UIImage *img_pause;
@property (retain, nonatomic) UIImage *img_play;
@property (nonatomic) CGPoint loc;
@end

@implementation RadioViewController
@synthesize playPause;
@synthesize image;
@synthesize detailsView;
@synthesize trackTitleLabel;
@synthesize reflectionView;
@synthesize show;
@synthesize theTableview;
@synthesize mainView;
@synthesize backView;

static const CGFloat kDefaultReflectionFraction = 0.65;
static const CGFloat kDefaultReflectionOpacity = 0.60;



- (id)init
{
    self = [super init];
    if (self) {
        [self configure];
    }
    return self;
}

- (id)initWithManager:(ButterflyManager *)mgr
{
    self = [super initWithManager:mgr];
    if (self) {
        [self configure];
    }
    return self;
}

- (void)configure
{
    self.hidesBottomBarWhenPushed = YES;
    self.title = @"Radio";
    show = FALSE;
    self.img_pause = [UIImage imageNamed:@"radio_pause.png"];
    self.img_play = [UIImage imageNamed:@"radio_play.png"];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(fillInDetails)
                                                 name:kBRStreamerStartedNotification
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(fillInDetails)
                                                 name:kBRStreamerPlaybackNotification
                                               object:nil];

}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self name:kBRStreamerStartedNotification object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:kBRStreamerPlaybackNotification object:nil];
    
    self.butterflyMgr.streamer.delegate = nil;
    self.img_pause = nil;
    self.img_play = nil;
    
    [image release];
    [playPause release];
    [reflectionView release];
    [detailsView release];
    [trackTitleLabel release];
    [theTableview release];
    [mainView release];
    [backView release];
    [super dealloc];
}

- (void)loadView
{
    reloadable = NO;
    UIView *view = [self baseViewWithColor:[UIColor blackColor]];
    CGRect frame = view.frame;
    
    mainView = [[UIView alloc] initWithFrame:frame];
    mainView.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageNamed:@"Default.png"]];
    mainView.autoresizingMask = (UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleWidth);
    
    image = [[UIImageView alloc] initWithFrame:CGRectMake(0, 0, view.frame.size.width, view.frame.size.width)];
    image.backgroundColor = [UIColor clearColor];
    [mainView addSubview:image];
    
    reflectionView = [[UIImageView alloc] initWithFrame:CGRectMake(0, image.frame.size.height, image.frame.size.width, image.frame.size.height)];
    [mainView addSubview:reflectionView];
    
    CGFloat height = 75.0f;
    
    frame = view.frame;
    frame.size.height -= (height-1);
    
    detailsView = [[TrackDetailView alloc] initWithFrame:CGRectMake(0, 0, frame.size.width, frame.size.height)];
    detailsView.delegate = self;
    
    detailsView.alpha = 0;
    detailsView.slider.value = 0;
    [self fillInDetails];
    [mainView addSubview:detailsView];
    
    [view addSubview:mainView];
    
    backView = [[UIView alloc] initWithFrame:mainView.frame];
    backView.autoresizingMask = (UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleWidth);
    backView.backgroundColor = [UIColor colorWithRed:254.0f/255.0f green:253.0f/255.0f blue:231.0f/255.0f alpha:1.0f]; //yellow
    backView.alpha = 0.0f;
    
    CGFloat x = 15.0f;
    theTableview = [[UITableView alloc] initWithFrame:CGRectMake(x, 0, frame.size.width-2*x, view.frame.size.height) style:UITableViewStylePlain];
    theTableview.showsVerticalScrollIndicator = NO;
    theTableview.autoresizingMask = UIViewAutoresizingFlexibleHeight;
    self.theTableview.dataSource = self;
    self.theTableview.delegate = self;
    
    UIView *paddingBottom = [[UIView alloc] initWithFrame:CGRectMake(0, 0, frame.size.width, height)];
    theTableview.tableFooterView = paddingBottom;
    [paddingBottom release];
    
    [backView addSubview:theTableview];
    
    [view addSubview:backView];

    
    UIToolbar *toolbar = [[UIToolbar alloc] initWithFrame:CGRectMake(0, view.frame.size.height-height+1, 320, height)];
    toolbar.barStyle = UIBarStyleBlack;
    toolbar.translucent = YES;
    toolbar.autoresizingMask = (UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleTopMargin);
    
    trackTitleLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, 0, toolbar.frame.size.width, 20)];
    trackTitleLabel.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    trackTitleLabel.textAlignment = UITextAlignmentCenter;
    trackTitleLabel.textColor = [UIColor whiteColor];
    trackTitleLabel.font = [UIFont systemFontOfSize:12.0f];
    trackTitleLabel.backgroundColor = [UIColor clearColor];
    [toolbar addSubview:trackTitleLabel];
    
    self.playPause = [UIButton buttonWithType:UIButtonTypeCustom];
    playPause.showsTouchWhenHighlighted = YES;
    [playPause addTarget:self action:@selector(btnPlayPausePressed:) forControlEvents:UIControlEventTouchUpInside];
    [playPause setBackgroundImage:[UIImage imageNamed:@"radio_pause.png"] forState:UIControlStateNormal];
    playPause.frame = CGRectMake(0, 0, 42, 42);
    UIBarButtonItem *btnPlayPause = [[UIBarButtonItem alloc] initWithCustomView:playPause];
    
    UIButton *skip = [UIButton buttonWithType:UIButtonTypeCustom];
    [skip setBackgroundImage:[UIImage imageNamed:@"radio_skip.png"] forState:UIControlStateNormal];
    [skip addTarget:self action:@selector(btnSkipPressed:) forControlEvents:UIControlEventTouchUpInside];
    skip.showsTouchWhenHighlighted = YES;
    skip.frame = playPause.frame;
    UIBarButtonItem *btnSkip = [[UIBarButtonItem alloc] initWithCustomView:skip];
    
    UIBarButtonItem *btnReply = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemReply target:self action:@selector(reply:)];
    UIBarButtonItem *btnTweet = [[UIBarButtonItem alloc] initWithImage:[UIImage imageNamed:@"icon_twitter.png"] style:UIBarButtonItemStylePlain target:self action:@selector(btnTweetPressed:)];
    UIBarButtonItem *btnShare = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAction target:self action:@selector(btnSharePressed:)];
    
    toolbar.items = [NSArray arrayWithObjects:[self flex], btnPlayPause, [self flex], btnSkip, [self flex], btnReply, [self flex], btnShare, [self flex], nil];
    [btnShare release];
    [btnTweet release];
    [btnPlayPause release];
    [btnSkip release];
    [btnReply release];
    
    [view addSubview:toolbar];
    [toolbar release];
    
    self.view = view;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.butterflyMgr.streamer.delegate = self;
    [self setUpBackButton:self action:@selector(exit)];

    
    UIBarButtonItem *btnTrackList = [[UIBarButtonItem alloc] initWithImage:[UIImage imageNamed:@"icon_more.png"] style:UIBarButtonItemStyleBordered target:self action:@selector(rotate)];
    
    self.navigationItem.rightBarButtonItems = [NSArray arrayWithObjects:btnTrackList, nil];
    [btnTrackList release];
    [self setupLoadingIndicator:1 verticalOffset:160.0f];
}

- (void)viewWillAppear:(BOOL)animated
{

    if (!self.butterflyMgr.currentStation)
        return;
    
    (self.butterflyMgr.streamer.isRunning) ? [self.playPause setBackgroundImage:self.img_pause forState:UIControlStateNormal] : [self.playPause setBackgroundImage:self.img_play forState:UIControlStateNormal];
    
    double percentage = (double)self.butterflyMgr.streamer.source.progressIndex/self.butterflyMgr.streamer.source.duration;
    [self.detailsView.slider setValue:percentage animated:NO];
    
    Station *station = self.butterflyMgr.currentStation;
    if (station.imgData==nil){
        station.delegate = self;
        [station fetchImage];
        return;
    }
    
    UIImage *img = [UIImage imageWithData:station.imgData];
    [self populateImage:img];
}


#pragma mark -
- (void)exit
{
    [self.navigationController popViewControllerAnimated:YES];
}

- (void)rotate
{
    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDuration:0.6f];
    [UIView setAnimationTransition:UIViewAnimationTransitionFlipFromLeft forView:self.mainView cache:YES];
    mainView.alpha = !mainView.alpha;
    [UIView commitAnimations];
    
    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDuration:0.6f];
    [UIView setAnimationTransition:UIViewAnimationTransitionFlipFromLeft forView:self.backView cache:YES];
    backView.alpha = !backView.alpha;
    [UIView commitAnimations];
}

- (void)reply:(UIBarItem *)btn
{
    BRRecordViewController *submitTrack = [[BRRecordViewController alloc] initWithManager:self.butterflyMgr];
    submitTrack.butterflyMgr = self.butterflyMgr;
    NSString *thread = self.butterflyMgr.streamer.source.thread;
    
    if (thread != nil){
        NSLog(@"REPLY TO: %@", thread);
        submitTrack.thread = thread;
    }
    
    submitTrack.station = self.butterflyMgr.currentStation;
    UINavigationController *nav = [[UINavigationController alloc] initWithRootViewController:submitTrack];
    nav.navigationBar.barStyle = UIBarStyleBlack;
    [submitTrack release];

    nav.modalTransitionStyle = UIModalTransitionStyleFlipHorizontal;
    
    [self presentViewController:nav animated:YES completion:^{
        [self.playPause setBackgroundImage:self.img_play forState:UIControlStateNormal];
    }];
    
    [nav release];
}

- (void)fillInDetails
{
    NSLog(@"Radio VC - fillInDetails");

    BRTrack *currentTrack = self.butterflyMgr.streamer.source;
    if (!currentTrack)
        return;
    
    trackTitleLabel.text = currentTrack.name;
    detailsView.nameLabel.text = [NSString stringWithFormat:@"%@: %@", self.butterflyMgr.currentStation.name, currentTrack.name];
    detailsView.authorLabel.text = [NSString stringWithFormat:@"by %@", currentTrack.author];
    detailsView.dateLabel.text = currentTrack.date;
}

- (void)populateImage:(UIImage *)img
{
    image.alpha = 0.0f;
    
    self.mainView.backgroundColor = nil;
    CGFloat w = img.size.width;
    CGFloat h = img.size.height;
    CGFloat max = self.view.frame.size.width;
    double scale;
    if (w>max){
        scale = max/w;
        w = max;
        h *= scale;
    }
    
    image.image = img;
    CGRect frame = image.frame;
    frame = CGRectMake(0.5*(self.view.frame.size.width-w), 0, w, h);
    image.frame = frame;
    
    frame.origin.y = frame.size.height;
    reflectionView.frame = frame;
    
    NSUInteger reflectionHeight = image.bounds.size.height * kDefaultReflectionFraction;
    reflectionView.image = [self reflectedImage:image withHeight:reflectionHeight];
	reflectionView.alpha = kDefaultReflectionOpacity;
    
    [UIView animateWithDuration:0.3f
                          delay:0.0f
                        options:UIViewAnimationOptionCurveLinear
                     animations:^{
                         image.alpha = 1.0f;
                     }
                     completion:NULL];

}

- (void)imageReady:(NSString *)addr
{
    UIImage *img = [UIImage imageWithData:self.butterflyMgr.currentStation.imgData];
    [self populateImage:img];
}

- (void)btnPlayPausePressed:(UIButton *)btn
{
//    NSLog(@"RADIO VIEW CONTROLLER - btnPlayPausePressed:");
    if (self.butterflyMgr.streamer.isRunning){
        [self.butterflyMgr.streamer pause];
        [self.playPause setBackgroundImage:self.img_play forState:UIControlStateNormal];
        return;
    }
    
    [self.butterflyMgr.streamer resume];
    [self.playPause setBackgroundImage:self.img_pause forState:UIControlStateNormal];
}

- (void)btnSkipPressed:(UIButton *)btn
{
    [self.butterflyMgr.streamer skip];
    [self.playPause setBackgroundImage:self.img_pause forState:UIControlStateNormal];
    [self.loading show];
    
}

- (void)viewComments:(UIButton *)btn
{
    if (self.butterflyMgr.streamer.source==nil)
        return;
    
    NSString *url = self.butterflyMgr.streamer.source.url;
    if ([url rangeOfString:@"stream"].location != NSNotFound){
        
        NSArray *parts = [url componentsSeparatedByString:@"/"];
        url = [parts lastObject];
//        NSLog(@"viewComments: %@", url);
        
        ReviewsViewController *reviews = [[ReviewsViewController alloc] initWithMode:ReviewModeTrack];
        reviews.uniqueID = url;
        reviews.station = self.butterflyMgr.currentStation;
        [self.navigationController pushViewController:reviews animated:YES];
        [reviews release];
    }
}

- (void)btnSharePressed:(UIBarButtonItem *)btn
{
    UIActionSheet *actionsheet = [[UIActionSheet alloc] initWithTitle:@"Share" delegate:self cancelButtonTitle:@"cancel" destructiveButtonTitle:nil otherButtonTitles:@"Email", @"Twitter", @"Reddit", nil];
    actionsheet.frame = CGRectMake(0, 150, self.view.frame.size.width, 100);
    actionsheet.actionSheetStyle = UIActionSheetStyleBlackOpaque;
    [actionsheet showInView:[UIApplication sharedApplication].keyWindow];
    [actionsheet release];
    return;
}

#pragma mark -  UIActionSheetDelegate
- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex
{
    NSLog(@"actionSheet clickedButtonAtIndex: %d", buttonIndex);
    if (buttonIndex==0){ // Email
        if ([MFMailComposeViewController canSendMail]==NO){
            [self showAlert:@"Error" message:@"Please configure the email account on your device."];
            return;
        }
        
        MFMailComposeViewController *picker = [[MFMailComposeViewController alloc] init];
        picker.navigationBar.barStyle = UIBarStyleBlack;
        picker.mailComposeDelegate = self;
        
        BRTrack *track = self.butterflyMgr.streamer.source;
        NSString *thread = track.thread;
        if (thread.length>100)
            thread = [thread substringToIndex:100];
        
        NSString *hyperLink = [NSString stringWithFormat:@"<a style='text-decoration:none' href='%@'>play</a>", [track threadUrl]];
        
        NSString *emailBody = [NSString stringWithFormat:@"<html><body><a href='http://www.butterflyradio.com'><img style='width:250px' src='http://www.butterflyradio.com/site/img/logo.png' /></a><br /><br />Check out what I heard on Butterfly Radio: %@</body></html>", hyperLink];
        [picker setMessageBody:emailBody isHTML:YES];
        
        if (track.full){
            NSString *fileName = track.name;
            NSString *mimeType = nil;
            if (track.format==FileFormatM4A){
                mimeType = @"audio/x-m4a";
                fileName = [fileName stringByAppendingString:@".m4a"];
            }
            else{
                mimeType = @"audio/mpeg";
                fileName = [fileName stringByAppendingString:@".mp3"];
            }
            
            [picker addAttachmentData:track.fileData mimeType:mimeType fileName:fileName];
        }
        
        [self presentModalViewController:picker animated:YES];
        [picker release];
        return;
    }
    
    if (buttonIndex==1){ // Twitter
        if (!self.butterflyMgr.currentStation)
            return;
        
        
        Class tweeterClass = NSClassFromString(@"TWTweetComposeViewController");
        if(tweeterClass == nil) {   // No Twitter integration; default to third-party Twitter sdk
            return;
        }
        
        if(![TWTweetComposeViewController canSendTweet]) { // Twitter is not accessible or the user has not setup an account
            NSLog(@"NO TWITTER ACCOUNT SET UP!");
            [self showAlert:@"No Twitter Account" message:@"Please set up your Twitter account in the Settings section of your iPhone."];
            return;
        }
        
        TWTweetComposeViewController *tweetViewController = [[TWTweetComposeViewController alloc] init];
        [tweetViewController setInitialText:[NSString stringWithFormat:@"%@: %@", self.butterflyMgr.currentStation.name, self.butterflyMgr.streamer.source.name]];
        BRTrack *track = self.butterflyMgr.streamer.source;
        NSString *thread = track.thread;
        if (thread.length>100)
            thread = [thread substringToIndex:100];
        
        NSString *url = ([track threadUrl]==nil) ? [NSString stringWithFormat:@"http://www.butterflyradio.com/site/audioplayer?station=%@&thread=%@", self.butterflyMgr.currentStation.unique_id, thread] : [track threadUrl];

        [tweetViewController addURL:[NSURL URLWithString:url]];
        
        tweetViewController.completionHandler = ^(TWTweetComposeViewControllerResult result) {
            if(result == TWTweetComposeViewControllerResultDone) { // the user finished composing a tweet
                NSLog(@"TWTweetComposeViewControllerResultDone");
            }
            if(result == TWTweetComposeViewControllerResultCancelled) { // the user cancelled composing a tweet
                NSLog(@"TWTweetComposeViewControllerResultCancelled");
            }
            [self dismissViewControllerAnimated:YES completion:NULL];
        };
        
        [self presentViewController:tweetViewController animated:YES completion:NULL];
        [tweetViewController release];
        return;
    }
    
    if (buttonIndex==2){ // Reddit
        BRRedditViewController *reddit = [[BRRedditViewController alloc] init];
        reddit.track = self.butterflyMgr.streamer.source;
        [self presentViewController:reddit animated:YES completion:NULL];
        [reddit release];
    }
}


- (void)mailComposeController:(MFMailComposeViewController *)controller didFinishWithResult:(MFMailComposeResult)result error:(NSError *)error
{
    NSLog(@"controller didFinishWithResult:");
    controller.delegate = nil;
    [controller dismissViewControllerAnimated:YES completion:NULL];
}


#pragma mark - UITableViewStuff
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return self.butterflyMgr.streamer.tracks.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *cellID = @"ID";
    BRTrackCell *cell = [theTableview dequeueReusableCellWithIdentifier:cellID];
    if (cell==nil) {
        cell = [[[BRTrackCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:cellID] autorelease];
        [cell setup];
    }
    
    if ([self.butterflyMgr.streamer.tracks containsObject:@"none"]==TRUE || self.butterflyMgr.streamer.tracks.count==0){
        cell.imageView.image = [UIImage imageNamed:@"icon_microphone.png"];
        cell.textLabel.text = @"None";
        cell.detailTextLabel.text = @"";
        cell.btnComments.hidden = YES;
        return cell;
    }

    BRTrack *track = (BRTrack *)[self.butterflyMgr.streamer.tracks objectAtIndex:indexPath.row];
    cell.imageView.image = (track.reply) ? [UIImage imageNamed:@"icon_reply.png"] : [UIImage imageNamed:@"icon_microphone.png"];
    cell.btnComments.hidden = NO;
    cell.textLabel.text = track.name;
    cell.detailTextLabel.text = [NSString stringWithFormat:@"By %@\n%@", track.author, track.date];
    
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return kCellHeight;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    BRTrack *track = (BRTrack *)[self.butterflyMgr.streamer.tracks objectAtIndex:indexPath.row];
    if (!track)
        return;

    
    [self.butterflyMgr playTrack:track fromStation:self.butterflyMgr.currentStation];
    [self.detailsView.slider setValue:0.0f animated:NO];
    [self fillInDetails];
    [self rotate];
    [self.loading show];
    [theTableview deselectRowAtIndexPath:[theTableview indexPathForSelectedRow] animated:YES];
}


#pragma mark - Image Reflection
CGImageRef CreateGradientImage(int pixelsWide, int pixelsHigh)
{
	CGImageRef theCGImage = NULL;
    
	// gradient is always black-white and the mask must be in the gray colorspace
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceGray();
	
	// create the bitmap context
	CGContextRef gradientBitmapContext = CGBitmapContextCreate(NULL, pixelsWide, pixelsHigh, 8, 0, colorSpace, kCGImageAlphaNone);
	
	// define the start and end grayscale values (with the alpha, even though
	// our bitmap context doesn't support alpha the gradient requires it)
	CGFloat colors[] = {0.0, 1.0, 1.0, 1.0};
	
	// create the CGGradient and then release the gray color space
	CGGradientRef grayScaleGradient = CGGradientCreateWithColorComponents(colorSpace, colors, NULL, 2);
	CGColorSpaceRelease(colorSpace);
	
	// create the start and end points for the gradient vector (straight down)
	CGPoint gradientStartPoint = CGPointZero;
	CGPoint gradientEndPoint = CGPointMake(0, pixelsHigh);
	
	// draw the gradient into the gray bitmap context
	CGContextDrawLinearGradient(gradientBitmapContext, grayScaleGradient, gradientStartPoint,
								gradientEndPoint, kCGGradientDrawsAfterEndLocation);
	CGGradientRelease(grayScaleGradient);
	
	// convert the context into a CGImageRef and release the context
	theCGImage = CGBitmapContextCreateImage(gradientBitmapContext);
	CGContextRelease(gradientBitmapContext);
	
	// return the imageref containing the gradient
    return theCGImage;
}

CGContextRef MyCreateBitmapContext(int pixelsWide, int pixelsHigh)
{
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
	
	// create the bitmap context
	CGContextRef bitmapContext = CGBitmapContextCreate (NULL, pixelsWide, pixelsHigh, 8,
														0, colorSpace,
														// this will give us an optimal BGRA format for the device:
														(kCGBitmapByteOrder32Little | kCGImageAlphaPremultipliedFirst));
	CGColorSpaceRelease(colorSpace);
    return bitmapContext;
}

- (UIImage *)reflectedImage:(UIImageView *)fromImage withHeight:(NSUInteger)height
{
    if(height == 0)
		return nil;
    
	CGContextRef mainViewContentContext = MyCreateBitmapContext(fromImage.bounds.size.width, height); // create a bitmap graphics context the size of the image
	
	// create a 2 bit CGImage containing a gradient that will be used for masking the 
	// main view content to create the 'fade' of the reflection.  The CGImageCreateWithMask
	// function will stretch the bitmap image as required, so we can create a 1 pixel wide gradient
	CGImageRef gradientMaskImage = CreateGradientImage(1, height);
	
	// create an image by masking the bitmap of the mainView content with the gradient view
	// then release the  pre-masked content bitmap and the gradient bitmap
	CGContextClipToMask(mainViewContentContext, CGRectMake(0.0, 0.0, fromImage.bounds.size.width, height), gradientMaskImage);
	CGImageRelease(gradientMaskImage);
	
	// In order to grab the part of the image that we want to render, we move the context origin to the
	// height of the image that we want to capture, then we flip the context so that the image draws upside down.
	CGContextTranslateCTM(mainViewContentContext, 0.0, height);
	CGContextScaleCTM(mainViewContentContext, 1.0, -1.0);
	
	// draw the image into the bitmap context
	CGContextDrawImage(mainViewContentContext, fromImage.bounds, fromImage.image.CGImage);
	
	// create CGImageRef of the main view bitmap content, and then release that bitmap context
	CGImageRef reflectionImage = CGBitmapContextCreateImage(mainViewContentContext);
	CGContextRelease(mainViewContentContext);
	
	// convert the finished reflection image to a UIImage 
	UIImage *theImage = [UIImage imageWithCGImage:reflectionImage];
	
	// image is retained by the property setting above, so we can release the original
	CGImageRelease(reflectionImage);
	
	return theImage;
}

#pragma mark - TrackDetailsViewDelegate
- (void)sliding
{
    NSLog(@"RADIO VC - SLIDING");
    if (!self.butterflyMgr.streamer.source)
        return;
    
    if (!self.butterflyMgr.streamer.isRunning)
        return;
    
    [self.butterflyMgr.streamer pause];
}

- (void)seekTo:(float)x
{
    //this gets called on -touchUpInside of slider:
    
    NSLog(@"RADIO VC - SEEK TO: %.2f", x);
    [self.butterflyMgr.streamer seekto:x];
}

#pragma mark - StreamerDelegate
- (void)playbackStarted
{
    [self fillInDetails];
}

- (void)updateProgress:(double)pct
{
    NSLog(@"updateProgress: %.2f", pct);
    [self.detailsView.slider setValue:pct animated:YES];
    self.detailsView.slider.userInteractionEnabled = !(pct==0); // disable slider until a progress value is calcuated. sometimes, the value is not valid in which case slider shouldn't work.
    
    if (!self.butterflyMgr.streamer.isRunning)
        return;
    
    [self.playPause setBackgroundImage:self.img_pause forState:UIControlStateNormal];
    if (!self.loading.isHidden)
        [self.loading hide];

}

- (void)fileFinished
{
    if (self.loading.alpha == 0)
        [self.loading show];
}

#pragma mark - UIResponder
- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
//    NSLog(@"touchesBegan:");
    show = TRUE;
    UITouch *touch = [touches anyObject];
    self.loc = [touch locationInView:self.view];
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
//    NSLog(@"touchesMoved:");
    show = FALSE;
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
//    NSLog(@"touchesEnded");
    
    UITouch *touch = [touches anyObject];
    CGPoint newLoc = [touch locationInView:self.view];
    CGFloat delta = newLoc.x-self.loc.x;
    
    if (abs(delta)>50)
        [self rotate];
    
    if (show==NO)
        return;
    
    CGFloat alpha = abs(detailsView.alpha-1);
    [UIView animateWithDuration:0.25f
                          delay:0.0f
                        options:UIViewAnimationOptionCurveLinear
                     animations:^{
                         detailsView.alpha = alpha;
                     }
                     completion:NULL];
}

- (void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event
{
//    NSLog(@"touchesCancelled");
    show = FALSE;
}




- (void)viewDidUnload
{
    [super viewDidUnload];
    self.backView = nil;
    self.mainView = nil;
    self.theTableview = nil;
    self.image = nil;
    self.reflectionView = nil;
    self.detailsView = nil;
    self.trackTitleLabel = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

@end
