//
//  BRRecordViewController.m
//  ButterflySDK
//
//  Copyright (c) 2013 ButterflyRadio. All rights reserved.


#import "BRRecordViewController.h"
#import "BRRecordedTracksViewController.h"
#import "BRTextCell.h"
#import "BRTextBox.h"
#import <QuartzCore/QuartzCore.h>


@interface BRRecordViewController ()
@property (retain, nonatomic) BRTrack *currentTrack;
@property (retain, nonatomic) UITableView *theTableview;
@property (retain, nonatomic) UIButton *btnMain; // record, play, stop
@property (retain, nonatomic) UIButton *btnClear;
@property (retain, nonatomic) UIButton *btnUpload;
@property (retain, nonatomic) UIButton *btnPause;
@property (retain, nonatomic) UIButton *btnHistory;
@property (retain, nonatomic) UIButton *btnSave;
@property (retain, nonatomic) UIImageView *imgStation;
@property (retain, nonatomic) UILabel *timeLabel;
@property (retain, nonatomic) UIView *pauseView;
@property (retain, nonatomic) UIPickerView *valuePicker;

//Track Info fields:
@property (retain, nonatomic) UITextField *titleField;
@property (retain, nonatomic) UITextField *authorField;
@property (retain, nonatomic) UITextView *commentField;
@end

static NSString *record = @"record";
static NSString *fileName = @"track.m4a";

#define kMaxTime @"time - 5:00"

@implementation BRRecordViewController
@synthesize upload;
@synthesize station;
@synthesize thread;
@synthesize timer;
@synthesize timeLeft;
@synthesize state;
@synthesize recordSettings;
@synthesize premium;

- (id)initWithManager:(ButterflyManager *)mgr
{
    self = [super initWithManager:mgr];
    if (self) {
        self.currentTrack = [BRTrack track];
        self.premium = NO;
        self.state = StateInactive;
        self.title = @"Record";
        self.tabBarItem.image = [UIImage imageNamed:@"tab_record.png"];
        self.timeLeft = kTimeLimit;
        
        //remove any old files that were left in the cache:
        NSString *filepath = [self createFilePath:fileName];
        NSError *error = nil;
        [[NSFileManager defaultManager] removeItemAtPath:filepath error:&error]; //this removes files
        if (error){
            
        }
        
        self.recordSettings = [NSMutableDictionary dictionary];
        [self.recordSettings setObject:[NSNumber numberWithInt:kAudioFormatMPEG4AAC] forKey: AVFormatIDKey];
//        [recordSettings setObject:[NSNumber numberWithInt:kAudioFormatMPEGLayer3] forKey: AVFormatIDKey]; //this is unsupported for some reason meaning iPhone CANNOT record to MP3
        
//        [recordSettings setObject:[NSNumber numberWithFloat:441000.0] forKey: AVSampleRateKey];
        [self.recordSettings setObject:[NSNumber numberWithFloat:16000.0] forKey: AVSampleRateKey];
        [self.recordSettings setObject:[NSNumber numberWithInt:1] forKey:AVNumberOfChannelsKey];
        
//        [recordSettings setObject:[NSNumber numberWithInt:12800] forKey:AVEncoderBitRateKey];
        [self.recordSettings setObject:[NSNumber numberWithInt:6400] forKey:AVEncoderBitRateKey];
        
//        [recordSettings setObject:[NSNumber numberWithInt:16] forKey:AVLinearPCMBitDepthKey];
        [self.recordSettings setObject:[NSNumber numberWithInt:32] forKey:AVLinearPCMBitDepthKey];
        
        [self.recordSettings setObject:[NSNumber numberWithInt: AVAudioQualityHigh] forKey: AVEncoderAudioQualityKey];
    }
    return self;
}

- (void)dealloc
{
    self.thread = nil;
    self.recordSettings = nil;
    self.timeLabel = nil;
    self.currentTrack = nil;
    self.commentField = nil;
    self.theTableview = nil;
    self.btnMain = nil;
    self.btnClear = nil;
    self.btnUpload = nil;
    self.btnPause = nil;
    self.imgStation = nil;
    self.titleField = nil;
    self.authorField = nil;
    self.pauseView = nil;
    self.btnHistory = nil;
    self.station = nil;
    self.valuePicker = nil;
    
    [recordSettings release];
    if (record)
        [recorder release];
    
    if (audioPlayer)
        [audioPlayer release];
    
    if (upload)
        [upload release];
    
    if (self.timer){
        [self.timer invalidate];
        self.timer = nil;
    }
    [super dealloc];
}

- (void)loadView
{
    UIView *view = [self baseViewWithColor:[UIColor colorWithRed:0.95f green:0.95f blue:0.95f alpha:1.0f]];
    CGRect appFrame = view.frame;
    
    UIImage *img = (self.station.thumbnail) ? self.station.thumbnail : [UIImage imageNamed:@"logo_red@2x.png"];
    self.imgStation = [[[UIImageView alloc] initWithImage:img] autorelease];
    self.imgStation.frame = CGRectMake(0.5*(view.frame.size.width-60.0f), 10.0f, 60.0f, 60.0f);
    self.imgStation.layer.borderWidth = 4.0f;
    self.imgStation.layer.borderColor = [[UIColor whiteColor] CGColor];
    [view addSubview:self.imgStation];
    
    
    CGFloat y = 70.0f;
    UILabel *titleLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, y, appFrame.size.width, 25)];
    titleLabel.backgroundColor = [UIColor clearColor];
    titleLabel.text = station.name;
    titleLabel.textAlignment = UITextAlignmentCenter;
    titleLabel.textColor = [UIColor darkGrayColor];
    titleLabel.font = [UIFont fontWithName:kFont size:16.0f];
    [view addSubview:titleLabel];
    [titleLabel release];
    
    appFrame.origin.y = titleLabel.frame.size.height;
    appFrame.size.height -= appFrame.origin.y;
    
    
    y += titleLabel.frame.size.height;
    self.theTableview = [[[UITableView alloc] initWithFrame:CGRectMake(0.0f, y, appFrame.size.width, 220.0f) style:UITableViewStyleGrouped] autorelease];
    self.theTableview.dataSource = self;
    self.theTableview.delegate = self;
    self.theTableview.scrollEnabled = NO;
    self.theTableview.backgroundView = nil;
    [view addSubview:self.theTableview];
    
    CGFloat w = 60.0f;
    y = view.frame.size.height-w-60.0f;
    self.btnMain = [UIButton buttonWithType:UIButtonTypeCustom];
    [self.btnMain setBackgroundImage:[UIImage imageNamed:@"btRecord.png"] forState:UIControlStateNormal];
    [self.btnMain addTarget:self action:@selector(mainbtnPressed) forControlEvents:UIControlEventTouchUpInside];
    self.btnMain.autoresizingMask = UIViewAutoresizingFlexibleTopMargin;
    self.btnMain.frame = CGRectMake(0.5*(view.frame.size.width-w), y+14.0f, w, w);
    [view addSubview:self.btnMain];

    w *= 0.8f;
    self.btnClear = [UIButton buttonWithType:UIButtonTypeCustom];
    [self.btnClear setBackgroundImage:[UIImage imageNamed:@"btClear.png"] forState:UIControlStateNormal];
    [self.btnClear addTarget:self action:@selector(clearFile) forControlEvents:UIControlEventTouchUpInside];
    self.btnClear.autoresizingMask = UIViewAutoresizingFlexibleTopMargin;
    self.btnClear.frame = CGRectMake(50.0f, y+20.0f, w, w);
    [view addSubview:self.btnClear];

    self.btnUpload = [UIButton buttonWithType:UIButtonTypeCustom];
    [self.btnUpload setBackgroundImage:[UIImage imageNamed:@"btUpload.png"] forState:UIControlStateNormal];
    [self.btnUpload addTarget:self action:@selector(submitFile) forControlEvents:UIControlEventTouchUpInside];
    self.btnUpload.autoresizingMask = UIViewAutoresizingFlexibleTopMargin;
    self.btnUpload.frame = CGRectMake(view.frame.size.width-w-50.0f, y+20.0f, w, w);
    [view addSubview:self.btnUpload];
    
    //this button is for pausing recording, not playback:
    self.btnPause = [UIButton buttonWithType:UIButtonTypeCustom];
    [self.btnPause setBackgroundImage:[UIImage imageNamed:@"btPause@2x.png"] forState:UIControlStateNormal];
    [self.btnPause addTarget:self action:@selector(pauseResumeRecording) forControlEvents:UIControlEventTouchUpInside];
    self.btnPause.autoresizingMask = UIViewAutoresizingFlexibleTopMargin;
    self.btnPause.hidden = YES;
    self.btnPause.frame = self.btnUpload.frame;
    [view addSubview:self.btnPause];
    

    y = self.btnMain.frame.origin.y+self.btnMain.frame.size.height+15.0f;
    self.timeLabel = [[[UILabel alloc] initWithFrame:CGRectMake(10.0f, y, view.frame.size.width-20.0f, 15.0f)] autorelease];
    self.timeLabel.autoresizingMask = UIViewAutoresizingFlexibleTopMargin;
    self.timeLabel.text = kMaxTime;
    self.timeLabel.backgroundColor = [UIColor clearColor];
    self.timeLabel.textAlignment = NSTextAlignmentCenter;
    [view addSubview:self.timeLabel];
    
    w = 40.0f;
    y = view.frame.size.height-w;
    self.btnHistory = [UIButton buttonWithType:UIButtonTypeCustom];
    [self.btnHistory addTarget:self action:@selector(gotoSavedTracks) forControlEvents:UIControlEventTouchUpInside];
    self.btnHistory.frame = CGRectMake(60.0f, y, w, w);
    [self.btnHistory setBackgroundImage:[UIImage imageNamed:@"btHistory.png"] forState:UIControlStateNormal];
    self.btnHistory.autoresizingMask = UIViewAutoresizingFlexibleTopMargin;
    [view addSubview:self.btnHistory];

    self.btnSave = [UIButton buttonWithType:UIButtonTypeCustom];
    [self.btnSave addTarget:self action:@selector(saveTrack) forControlEvents:UIControlEventTouchUpInside];
    self.btnSave.frame = CGRectMake(appFrame.size.width-w-60.0f, y, w, w);
    [self.btnSave setBackgroundImage:[UIImage imageNamed:@"btSave.png"] forState:UIControlStateNormal];
    self.btnSave.autoresizingMask = UIViewAutoresizingFlexibleTopMargin;
    [view addSubview:self.btnSave];
    
    if (self.premium){
        self.valuePicker = [[[UIPickerView alloc] initWithFrame:CGRectZero] autorelease];
        self.valuePicker.showsSelectionIndicator = YES;
        CGFloat h = self.valuePicker.frame.size.height;
        self.valuePicker.frame = CGRectMake(0.0f, view.frame.size.height, view.frame.size.width, h);
        self.valuePicker.autoresizingMask = UIViewAutoresizingFlexibleTopMargin;
        self.valuePicker.dataSource = self;
        self.valuePicker.delegate = self;
        [view addSubview:self.valuePicker];
    }
    
    CGFloat h = 0.68f*appFrame.size.height;
    self.pauseView = [[[UIView alloc] initWithFrame:CGRectMake(0.0f, -h, appFrame.size.width, h)] autorelease];
    self.pauseView.backgroundColor = [UIColor clearColor];
    
    UIView *blackScreen = [[UIView alloc] initWithFrame:CGRectMake(0.0f, 0.0, self.pauseView.frame.size.width, self.pauseView.frame.size.height-10.0f)];
    blackScreen.backgroundColor = [UIColor blackColor];
    blackScreen.alpha = 0.8f;
    [self.pauseView addSubview:blackScreen];
    [blackScreen release];
    
    UIImage *imgShadow = [UIImage imageNamed:@"dropShadow.png"];
    UIImageView *shadow = [[UIImageView alloc] initWithImage:imgShadow];
    shadow.frame = CGRectMake(0.0f, blackScreen.frame.origin.y+blackScreen.frame.size.height, imgShadow.size.width, imgShadow.size.height);
    [self.pauseView addSubview:shadow];
    [shadow release];
    
    BRTextBox *descriptionBox = [[BRTextBox alloc] initWithFrame:CGRectMake(20.0f, 100.0f, blackScreen.frame.size.width-40.0f, 10.0f)];
    descriptionBox.lblTitle.text = @"Currently Paused";
    descriptionBox.lblText.text = @"To resume recording, tap the red button below.";
    [self.pauseView addSubview:descriptionBox];
    [descriptionBox release];
    
    [view addSubview:self.pauseView];
    
    self.view = view;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    [self setUpBackButton:self action:@selector(exit)];

    [self setupLoadingIndicator];
    self.loading.titleLabel.text = @"Uploading File...";
    
    [[NSNotificationCenter defaultCenter] postNotification:[NSNotification notificationWithName:kInterruptionNotification object:nil]];
}

- (void)viewWillAppear:(BOOL)animated
{
    NSLog(@"RECORD VIEW CONTROLLER - viewWillAppear: %@==%.1f seconds", self.currentTrack.name, self.currentTrack.duration);
    [super viewWillAppear:animated];
    if (!self.currentTrack)
        return;
    
    self.timeLeft = kTimeLimit-(int)self.currentTrack.duration;
    [self updateTimeLabel];
}


#pragma mark -
- (BOOL)saveTrack
{
    if (self.state == StatePlaying){
        [self showAlert:@"Currently Playing" message:@"Please stop the track first."];
        return NO;
    }
    
    if (self.state == StateRecording){
        [self showAlert:@"Currently Recording" message:@"Please stop recording first."];
        return NO;
    }

    NSString *filePath = [self createFilePath:fileName];
    NSData *fileData = [NSData dataWithContentsOfFile:filePath];
    if (!fileData){
        [self showAlert:@"No File" message:@"Please record an audio file to submit."];
        return NO;
    }

    self.currentTrack.fileData = [NSMutableData dataWithData:fileData];
    [self updateTrackInfo:nil];
    self.currentTrack.nsdate = [NSDate date];
    [self.currentTrack saveToDb];
    return YES;
}

- (void)exit
{
    if (self.state == StateInactive){
        [self dismissModalViewControllerAnimated:YES];
        return;
    }
    
    if (self.state==StatePlaying)
        [self showAlert:@"Currently Playing" message:@"Please stop playing the file first."];
    else
        [self showAlert:@"Currently Recording" message:@"Please stop the recording first."];
}

- (BOOL)checkFile
{
    BOOL file = FALSE;
    NSString *filePath = [self createFilePath:fileName];
    NSData *d = [NSData dataWithContentsOfFile:filePath];
    if (d)
        file = TRUE;
    
    return file;
}

- (void)stopTimer
{
    if (self.timer!=nil){
        [self.timer invalidate];
        self.timer = nil;
    }
    
    self.view.backgroundColor = [UIColor colorWithRed:0.95f green:0.95f blue:0.95f alpha:1.0f];
}

- (void)reset //this stops timers and changes colors back to normal. it does NOT erase a track or clear out text fields.
{
    NSLog(@"Record VC - RESET");
    [self stopTimer];
    self.view.backgroundColor = [UIColor colorWithRed:0.95f green:0.95f blue:0.95f alpha:1.0f];
    if (recorder){
        if (recorder.recording==TRUE){
            NSLog(@"RESET: STOP RECORDING");
            [recorder stop];
            
            recorder.delegate = nil;
            [recorder release];
            recorder = nil;
        }
    }
    
    if (audioPlayer != nil){
        NSLog(@"RESET: STOP PLAYING");
        [audioPlayer stop];
        audioPlayer.delegate = nil;
        [audioPlayer release];
        audioPlayer = nil;
    }
    
    self.btnUpload.hidden = NO;
    self.btnClear.hidden = NO;
    self.btnPause.hidden = YES;
    [self.btnPause setBackgroundImage:[UIImage imageNamed:@"btPause.png"] forState:UIControlStateNormal];
    self.btnHistory.hidden = NO;
    self.btnSave.hidden = NO;
    self.state = StateInactive;
    
    if (self.pauseView.frame.origin.y==0) //pause view on screen
        [self hidePauseScreen];
}

- (void)startOver //this is a total reset - it removes any recorded track
{
    [self reset];
    
    self.timeLabel.text = [NSString stringWithFormat:@"%@", kMaxTime];
    self.commentField.textColor = [UIColor lightGrayColor];
    self.titleField.text = nil;
    self.authorField.text = nil;

    
    self.timeLeft = kTimeLimit;
    self.currentTrack = [BRTrack track];
    NSString *filepath = [self createFilePath:fileName];
    NSError *error = nil;
    [[NSFileManager defaultManager] removeItemAtPath:filepath error:&error]; //this removes files
    if (error){
        
    }
}



- (NSString *)createFilePath:(NSString *)fileName
{
	fileName = [fileName stringByReplacingOccurrencesOfString:@"/" withString:@"+"];
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES);
	NSString *docPath = [paths objectAtIndex:0];
	NSString *filePath = [docPath stringByAppendingPathComponent:fileName];
	NSLog(@"filepath = %@", filePath);
	return filePath;
}

- (void)timerMethod
{
    self.timeLeft--;
    if (self.timeLeft < 0){  //out of time
        [self reset];
        return;
    }
    
    [self updateTimeLabel];
}

- (void)updateTimeLabel
{
    if (self.timeLeft <= 30)
        self.view.backgroundColor = [UIColor redColor];
    
    float m = self.timeLeft/60;
    int mins = (int)m;
    int s = self.timeLeft % 60;
    
    NSString *seconds = (s < 10) ? [NSString stringWithFormat:@"0%d", s] : [NSString stringWithFormat:@"%d", s];
    self.timeLabel.text = [NSString stringWithFormat:@"time - %d:%@", mins, seconds];
}

- (void)uploadTrack
{
    NSLog(@"RECORD VIEW CONTROLLER - UPLOAD STRING: %@", self.upload);
    ASIFormDataRequest *request = [[[ASIFormDataRequest alloc] initWithURL:[NSURL URLWithString:self.upload]] autorelease];
    request.delegate = self;
    
    //artificially setting the MIME type here. this TRICKS the Android into thinking it's streaming an MP3 when it's really streaming an AAC - it works.
    [request setFile:[self createFilePath:fileName] withFileName:@"track.mp4" andContentType:@"audio/mpeg" forKey:@"file"];
    [request setUploadProgressDelegate:self.loading.progress];
    
    [self.currentTrack populatePostParameters:request]; //TRACK PROPERTIES - these properties only pertain to the track. no matter what station is posted to, these properties remain same:
    
    
    // - - - - - STATION PROPERTIES: these properties are dependent on the station - - - - -
    NSString *authorized = (self.premium) ? @"yes" : @"no";
    [request setPostValue:authorized forKey:@"authorized"]; //premium tracks can only be posted by station manager
    [request setPostValue:station.name forKey:@"stationName"];
    [request setPostValue:station.unique_id forKey:@"station"];
    if (self.thread!=nil)
        [request setPostValue:self.thread forKey:@"replyTo"];

    NSString *admins = @"";
    for (NSString *admin in station.admins){
        if ([admin length]>1){
            NSString *entry = [NSString stringWithFormat:@"%@,", admin];
            admins = [admins stringByAppendingString:entry];
        }
    }
    admins = [admins substringToIndex:admins.length-1]; //remove last comma
    [request setPostValue:admins forKey:@"admins"];
    [request setPostValue:self.butterflyMgr.host.email forKey:@"sender"];
    
    
    
    [request startAsynchronous];
    self.loading.progress.hidden = NO;
    [self.loading show];
}

- (void)updateButton
{
    
}

- (void)clearFile
{
    if (self.state != StateInactive)
        return;
    
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Are You Sure?" message:@"This will permanently erase any recorded track." delegate:self cancelButtonTitle:@"yes" otherButtonTitles:@"no", nil];
    [alert show];
    [alert release];
}

- (void)submitFile
{
    NSString *filePath = [self createFilePath:fileName];
    NSData *fileData = [NSData dataWithContentsOfFile:filePath];
    if (!fileData){
        [self showAlert:@"No File" message:@"Please record an audio file to submit."];
        return;
    }
    
    if (self.titleField.text.length==0 || self.authorField.text.length==0){
        [self showAlert:@"Missing Information" message:@"Please fill in the Title and Author fields."];
        return;
    }
    
    [self saveTrack]; // save file to local db. in case it crashed during upload, user can use saved version
    
    if (self.request){
        self.request.delegate = nil;
        [self.request cancel];
        self.request = nil;
    }
    
    //grap the upload url string first:
    NSString *url = [NSString stringWithFormat:@"http://%@/api/upload?type=track", kUrl];
    self.request = [BRNetworkOp operationWithAddress:url parameters:nil];
    self.request.delegate = self;
    [self.request setHttpMethod:@"GET"];
    [self.request sendRequest];
    [self.loading show];
}

- (void)gotoSavedTracks
{
    if (self.state == StatePlaying){
        [self showAlert:@"Currently Playing" message:@"Please stop the track first."];
        return;
    }

    if (self.state == StateRecording){
        [self showAlert:@"Currently Recording" message:@"Please stop recording first."];
        return;
    }

    BRRecordedTracksViewController *trackHistory = [[BRRecordedTracksViewController alloc] initWithManager:self.butterflyMgr];
    trackHistory.delegate = self;
    [self.navigationController pushViewController:trackHistory animated:YES];
    [trackHistory release];
}

- (void)startRecording
{
    NSLog(@"START RECORDER! ");
    
    //this should be conditional - it is only necessary for iPhone 4 and lower. iPhone 4S and better doesn't need this:
    //    [self.butterflyMgr.streamer reset:NO];
    [self.butterflyMgr.streamer suspend];
    
    AVAudioSession *audioSession = [AVAudioSession sharedInstance];
    [audioSession setCategory:AVAudioSessionCategoryRecord error:nil];
    [audioSession setActive:YES error:nil];
    
    if (recorder==nil){
        NSLog(@"START RECORDER: INSTANTIATE RECORDER");
        NSString *filePath = [self createFilePath:fileName];
        NSError *error = nil;
        recorder = [[AVAudioRecorder alloc] initWithURL:[NSURL fileURLWithPath:filePath] settings:self.recordSettings error:&error];
        if (error){
            NSLog(@"ERROR: %@", [error localizedDescription]);
            return;
        }
        recorder.delegate = self;
        if ([recorder prepareToRecord] == NO){
            int errorCode = CFSwapInt32HostToBig ([error code]);
            NSLog(@"Recording Error: %@ [%4.4s])" , [error localizedDescription], (char*)&errorCode);
            return;
        }
        
        [self stopTimer];
        [[NSNotificationCenter defaultCenter] postNotification:[NSNotification notificationWithName:kRecordingNotification object:nil]];
        [recorder record];
        NSLog(@"recording");
        self.view.backgroundColor = [UIColor blueColor];
        
        self.timer = [NSTimer scheduledTimerWithTimeInterval:1.0 target:self selector:@selector(timerMethod) userInfo:nil repeats:YES];
    }
    
    [self.btnMain setBackgroundImage:[UIImage imageNamed:@"btStop@2x.png"] forState:UIControlStateNormal];
    [[NSNotificationCenter defaultCenter] postNotification:[NSNotification notificationWithName:kInterruptionNotification object:nil]];
    
    self.btnClear.hidden = YES;
    self.btnUpload.hidden = YES;
    self.btnPause.hidden = NO;
    [self.btnPause setBackgroundImage:[UIImage imageNamed:@"btPause.png"] forState:UIControlStateNormal];
    self.btnHistory.hidden = YES;
    self.btnSave.hidden = YES;
    self.state = StateRecording;
}

- (void)playFile
{
    NSString *filePath = [self createFilePath:fileName];
    NSData *dataSize = [NSData dataWithContentsOfURL:[NSURL fileURLWithPath:filePath]];
    NSLog(@"playing: %d", dataSize.length);
    if (dataSize.length==0){
        [self showAlert:@"No File" message:@"There is no file recorded."];
        return;
    }
    
    AVAudioSession *audioSession = [AVAudioSession sharedInstance];
    [audioSession setCategory:AVAudioSessionCategoryPlayback error:nil];
    
    if (audioPlayer==nil){
        NSError *error = nil;
        audioPlayer = [[AVAudioPlayer alloc] initWithContentsOfURL:[NSURL fileURLWithPath:filePath] error:&error];
        audioPlayer.delegate = self;
        audioPlayer.numberOfLoops = 0;
    }
    [audioPlayer play];
}

- (void)hidePauseScreen
{
    [UIView animateWithDuration:0.20f
                          delay:0.0f
                        options:UIViewAnimationOptionCurveLinear
                     animations:^{
                         CGRect frame = self.pauseView.frame;
                         frame.origin.y = -self.pauseView.frame.size.height;
                         self.pauseView.frame = frame;
                     }
                     completion:NULL];
}

- (void)pauseResumeRecording
{
    NSLog(@"pauseResumeRecording");
    if (!recorder)
        return;
    
    //pause recording
    if (recorder.isRecording){
        NSLog(@"PAUSE RECORDING");
        [recorder pause];
        [self.btnPause setBackgroundImage:[UIImage imageNamed:@"btResume.png"] forState:UIControlStateNormal];
        if (self.timer){
            [self.timer invalidate];
            self.timer = nil;
        }
        
        [UIView animateWithDuration:0.25f
                              delay:0.0f
                            options:UIViewAnimationOptionCurveLinear
                         animations:^{
                             CGRect frame = self.pauseView.frame;
                             frame.origin.y = 0.0f;
                             self.pauseView.frame = frame;
                         }
                         completion:^(BOOL finished){
                             [UIView animateWithDuration:0.07f
                                                   delay:0.0f
                                                 options:UIViewAnimationOptionCurveLinear
                                              animations:^{
                                                  CGRect frame = self.pauseView.frame;
                                                  frame.origin.y = -7.0f;
                                                  self.pauseView.frame = frame;
                                              }
                                              completion:^(BOOL finished){
                                                  [UIView animateWithDuration:0.07f
                                                                        delay:0.0f
                                                                      options:UIViewAnimationOptionCurveLinear
                                                                   animations:^{
                                                                       CGRect frame = self.pauseView.frame;
                                                                       frame.origin.y = 0.0f;
                                                                       self.pauseView.frame = frame;
                                                                   }
                                                                   completion:NULL];
                                                  
                                              }];
                             
                         }];
        
        return;
    }
    
    // resume recording
    NSLog(@"RESUME RECORDING");
    [recorder record];
    [self.btnPause setBackgroundImage:[UIImage imageNamed:@"btPause.png"] forState:UIControlStateNormal];
    self.timer = [NSTimer scheduledTimerWithTimeInterval:1.0 target:self selector:@selector(timerMethod) userInfo:nil repeats:YES];
    
    [self hidePauseScreen];
}


#pragma mark - Tableview Stuff
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 2;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    NSInteger rows;
    if (self.premium)
        rows = (section==0) ? 3 : 1;
    else
        rows = (section==0) ? 2 : 1;
    
    return rows;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    NSString *sections[] = {@"title", @"author", [NSString stringWithFormat:@"%d tickets", self.currentTrack.value]};
    static NSString *cellID = @"ID";
    
    BRTextCell *cell = [self.theTableview dequeueReusableCellWithIdentifier:cellID];
    if (cell==nil){
        cell = [[[BRTextCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:cellID] autorelease];
        cell.textMode = indexPath.section;
        [cell setup];
        cell.textField.delegate = self;
        cell.textView.delegate = self;
        
        if (indexPath.section==0){
            if (indexPath.row==0){
                self.titleField = cell.textField;
            }
            if (indexPath.row==1){
                self.authorField = cell.textField;
            }
            if (indexPath.row==2){
                cell.textField.userInteractionEnabled = NO;
            }
        }
        else{
            self.commentField = cell.textView;
        }
    }
    
    [(UIImageView *)cell.backgroundView setImage:nil];
    cell.backgroundView.backgroundColor = nil;
    cell.textField.tag = cell.textView.tag = 1000*(indexPath.section+1)+indexPath.row;
    cell.textField.placeholder = sections[indexPath.row];
    
    if (indexPath.section==0){
        if (indexPath.row==0) // top cell:
            [(UIImageView *)cell.backgroundView setImage:[UIImage imageNamed:@"bg_cell_top.png"]];
        else {
            if (self.premium){
                if(indexPath.row==1){
                    [(UIImageView *)cell.backgroundView setImage:[UIImage imageNamed:@"bg_cell_middle.png"]];
                }
                else{
                    [(UIImageView *)cell.backgroundView setImage:[UIImage imageNamed:@"bg_cell_bottom.png"]];
                }
            }
            else{ // bottom cell - only two cells
                [(UIImageView *)cell.backgroundView setImage:[UIImage imageNamed:@"bg_cell_bottom.png"]];
            }
        }
        
        return cell;
    }
    
    [(UIImageView *)cell.backgroundView setImage:[UIImage imageNamed:@"bg_cell_description.png"]];
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return (indexPath.section==0) ? [BRTextCell cellHeight] : 70.0f;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    NSLog(@"tableView didSelectRowAtIndexPath:");
    if (indexPath.row==2){
        NSLog(@"Show Values Picker!");
        
        [UIView animateWithDuration:0.3f
                              delay:0.0f
                            options:UIViewAnimationOptionCurveLinear
                         animations:^{
                             CGRect frame = self.valuePicker.frame;
                             frame.origin.y = self.view.frame.size.height-frame.size.height;
                             self.valuePicker.frame = frame;
                         }
                         completion:NULL];
        
    }
}

#pragma mark - UIPickerView
- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView
{
    return 1;
}

- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component
{
    return 20;
}

- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component;
{
    return [NSString stringWithFormat:@"%d tickets", row];
    
}

- (void)pickerView:(UIPickerView *)pickerView didSelectRow:(NSInteger)row inComponent:(NSInteger)component
{
    NSLog(@"pickerView didSelectRow:");
    self.currentTrack.value = row;
    [self.theTableview reloadData];
    [self hidePicker];
    
}

- (void)hidePicker
{
    [UIView animateWithDuration:0.3f
                          delay:0.0f
                        options:UIViewAnimationOptionCurveLinear
                     animations:^{
                         CGRect frame = self.valuePicker.frame;
                         frame.origin.y = self.view.frame.size.height;
                         self.valuePicker.frame = frame;
                     }
                     completion:NULL];
}

#pragma mark - UIAlertViewDelegate
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    NSLog(@"alertView clickedButtonAtIndex: %d", buttonIndex);
    if (buttonIndex==0){
        [self startOver];
        [self.btnMain setBackgroundImage:[UIImage imageNamed:@"btRecord@2x.png"] forState:UIControlStateNormal];
    }
}


#pragma mark - RecordViewDelegate
- (void)mainbtnPressed
{
    if (self.state==StateInactive){
        NSLog(@"mainbtnPressed: INACTIVE");
        if ([self checkFile]==NO){ // start the recorder
            NSLog(@"TEST 1");
            [self startRecording];
            return;
        }
        
        NSLog(@"mainbtnPressed: PLAY BACK");
        (audioPlayer == nil) ? [self playFile] : [audioPlayer play];
        [self.btnMain setBackgroundImage:[UIImage imageNamed:@"btPause@2x.png"] forState:UIControlStateNormal];
        self.state = StatePlaying;
        return;
    }
    
    [self.btnMain setBackgroundImage:[UIImage imageNamed:@"btPlay@2x.png"] forState:UIControlStateNormal];
    if (self.state==StateRecording){
        NSLog(@"mainbtnPressed: RECORDING - STOP");
        [self reset];
        return;
    }
    
    if (self.state==StatePlaying){
        NSLog(@"mainbtnPressed: PLAYING - PAUSE");
        [audioPlayer pause];
        self.state = StateInactive;
    }
}

- (void)updateTrackInfo:(id)sender
{
    self.currentTrack.author = (self.authorField.text.length > 0) ? self.authorField.text : @"none";
    self.currentTrack.name = (self.titleField.text.length > 0) ? self.titleField.text : @"none";
    NSLog(@"updateTrackInfo: %@", self.currentTrack.name);
    
    self.currentTrack.tags = @[@"none"];
    self.currentTrack.duration = (kTimeLimit-self.timeLeft);
    
    NSString *commentString = self.commentField.text;
    if (commentString.length==0){
        self.currentTrack.description = @"none";
        return;
    }
    self.currentTrack.description = ([commentString isEqualToString:@"description (optional)"]) ? @"none" : commentString;
}


#pragma mark - BRRecordedTracksViewControllerDelegate
- (void)savedTrackSelected:(BRTrack *)track
{
    NSLog(@"trackSelected: %@", track.name);
    if (!track)
        return;
    
    if (!track.fileData) //no file data, return early
        return;
    
    self.currentTrack = track;
    self.titleField.text = track.name;
    self.authorField.text = track.author;
    self.commentField.text = @"description";
    
    
    NSError *error = nil;
    [track.fileData writeToFile:[self createFilePath:fileName] options:NSDataWritingAtomic error:&error];
    if (error)
        return;
    
    [self.btnMain setBackgroundImage:[UIImage imageNamed:@"btPlay@2x.png"] forState:UIControlStateNormal];
}

#pragma mark - BRNetworkOpCallback:
- (void)requestData:(NSArray *)pkg //returns [address, data]
{
    if (!pkg)
        return;
    
    id json = [self.request parseJsonResponse];
    if (!json){
        [self showAlert:@"Error" message:@"There was an error. Please try again."];
        [self.loading hide];
        return;
    }
    
    NSDictionary *d = [json objectForKey:@"results"];
    if (d==nil){
        [self.request sendRequest];
        return;
    }
    NSLog(@"%@", [d description]);
    
    NSString *confirmation = [d objectForKey:@"confirmation"];
    if ([confirmation isEqualToString:@"success"]==NO){
        [self.loading hide];
        [self showAlert:@"Error" message:@"There was an error. Please try again."];
        return;
    }
    
    self.upload = [d objectForKey:@"upload string"];
    [self uploadTrack];
}

#pragma mark - AVAudioRecorderDelegate
- (void)audioRecorderDidFinishRecording:(AVAudioRecorder *)recorder successfully:(BOOL)flag
{
    NSLog(@"audioRecorderDidFinishRecording:");
    self.view.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageNamed:@"bg_radio.png"]];
}

- (void)audioRecorderEncodeErrorDidOccur:(AVAudioRecorder *)recorder error:(NSError *)error
{
    NSLog(@"audioRecorderEncodeErrorDidOccur: %@", [error localizedDescription]);
    self.view.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageNamed:@"bg_radio.png"]];
}

- (void)audioRecorderBeginInterruption:(AVAudioRecorder *)recorder
{
    NSLog(@"audioRecorderBeginInterruption:");
    self.view.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageNamed:@"bg_radio.png"]];
    
}

- (void)audioRecorderEndInterruption:(AVAudioRecorder *)recorder withFlags:(NSUInteger)flags
{
    NSLog(@"audioRecorderEndInterruption:");
}


#pragma mark -  AVAudioPlayerDelegate
- (void)audioPlayerDidFinishPlaying:(AVAudioPlayer *)player successfully:(BOOL)flag
{
    NSLog(@"audioPlayerDidFinishPlaying:");
    [self stopTimer];
    [audioPlayer stop];
    
    [self reset];
    [self.btnMain setBackgroundImage:[UIImage imageNamed:@"btPlay@2x.png"] forState:UIControlStateNormal];
}

- (void)audioPlayerDecodeErrorDidOccur:(AVAudioPlayer *)player error:(NSError *)error
{
    NSLog(@"audioPlayerDecodeErrorDidOccur: %@", [error localizedDescription]);
}

- (void)audioPlayerBeginInterruption:(AVAudioPlayer *)player
{
    NSLog(@"audioRecorderEndInterruption:");
    [self stopTimer];
    [audioPlayer pause];
    
}

- (void)audioPlayerEndInterruption:(AVAudioPlayer *)player withFlags:(NSUInteger)flags
{
    NSLog(@"audioPlayerEndInterruption:");
    [audioPlayer play];
    
}


#pragma mark - ASIHTTPRequestDelegate
- (void)requestFinished:(ASIHTTPRequest *)request
{
    NSString *json = [request responseString];
//    NSLog(@"RECORD VIEW CONTROLLER - requestFinished: %@", json);
    
    [self.loading hide];
    NSDictionary *d = [json JSONValue];
    if (d==nil){
        [self showAlert:@"Error" message:@"There was an error. Please try again."];
        return;
    }
    
    d = [d objectForKey:@"results"];
    NSLog(@"%@", [d description]);
    NSString *confirmation = [d objectForKey:@"confirmation"];
    if ([confirmation isEqualToString:@"success"]==NO){
        [self showAlert:@"Error" message:@"There was an error with the submisssion. Please try again."];
        return;
    }
    
    
    if (self.premium){ // approval is automatic - update station info right away so tableview reflects updated data immediately.
        NSDictionary *stationInfo = [d objectForKey:@"station"];
        [self.station populate:stationInfo];
        [self showAlert:@"Track Published" message:@"Your audio track has been published. Thank you."];
    }
    else{
        [self showAlert:@"Track Submitted" message:@"Your audio track was submitted. An admin will review it shortly. Thank you."];
    }
    
    [self dismissModalViewControllerAnimated:YES];
}

- (void)requestFailed:(ASIHTTPRequest *)request
{
    NSError *error = [request error];
    NSLog(@"RECORD VIEW CONTROLLER - requestFailed: %d == %@",  error.code, [error localizedDescription]);
    [self.loading hide];
    NSString *errorMsg = [error localizedDescription];
    if ([errorMsg rangeOfString:@"The request timed out"].location != NSNotFound){ //whenever this happens, it's actually okay - the post went through
        [self showAlert:@"Announcement Submitted" message:@"Your announcement was submitted. An admin will review it shortly. Thank you."];
        [self dismissModalViewControllerAnimated:YES];
        return;
    }
    
    [self showAlert:@"Error" message:@"There was an error. Please try again."];
}

#pragma mark - UITextField
- (BOOL)textFieldShouldBeginEditing:(UITextField *)textField
{
    NSLog(@"textFieldShouldBeginEditing: %d", textField.tag);
    [self hidePicker];
    return YES;
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
    [textField resignFirstResponder];
    return YES;
}

- (void)textFieldDidEndEditing:(UITextField *)textField
{
    NSLog(@"textFieldDidEndEditing: %d", textField.tag);
    if (self.view.frame.origin.y==0)
        return;
    
    [UIView animateWithDuration:0.25f
                          delay:0.0f
                        options:UIViewAnimationOptionCurveLinear
                     animations:^{
                         CGRect frame = self.view.frame;
                         frame.origin.y = 0.0f;
                         self.view.frame = frame;
                     }
                     completion:NULL];

}


#pragma mark - UITextView
- (BOOL)textViewShouldBeginEditing:(UITextView *)textView
{
    textView.text = @"";
    textView.textColor = [UIColor blackColor];
    
    [UIView animateWithDuration:0.25f
                          delay:0.0f
                        options:UIViewAnimationOptionCurveLinear
                     animations:^{
                         CGRect frame = self.view.frame;
                         frame.origin.y = -100.0f;
                         self.view.frame = frame;
                     }
                     completion:NULL];
    return YES;
}

- (BOOL)textView:(UITextView *)textView shouldChangeTextInRange:(NSRange)range replacementText:(NSString *)text
{
    NSLog(@"shouldChangeTextInRange: %@", text);
    if ([text isEqualToString:@"\n"]){
        [textView resignFirstResponder];
        return NO;
    }
    
    return YES;
}

- (void)textViewDidEndEditing:(UITextView *)textView
{
    if (textView.text.length==0){
        textView.text = @"Description";
        textView.textColor = [UIColor lightGrayColor];
    }
    
    [UIView animateWithDuration:0.25f
                          delay:0.0f
                        options:UIViewAnimationOptionCurveLinear
                     animations:^{
                         CGRect frame = self.view.frame;
                         frame.origin.y = 0.0f;
                         self.view.frame = frame;
                     }
                     completion:NULL];
}


- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
    NSLog(@"touchesBegan:");
    [super touchesBegan:touches withEvent:event];
    
    [self hidePicker];
    [self.authorField resignFirstResponder];
    [self.titleField resignFirstResponder];
}


- (void)viewDidUnload
{
    [super viewDidUnload];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

@end
