//
//  BRRecordedTracksViewController.m
//  ButterflySDK
//
//  Created by Denny Kwon on 1/18/13.
//  Copyright (c) 2013 ButterflyRadio. All rights reserved.
//

#import "BRRecordedTracksViewController.h"
#import "RadioViewController.h"
#import "BRContentCell.h"
#import "BRGradientBar.h"


@interface BRRecordedTracksViewController ()
@property (retain, nonatomic) UITableView *theTableview;
@property (retain, nonatomic) Station *tempStation;
@property (retain, nonatomic) BRGradientBar *gradientBar;
@property (retain, nonatomic) NSIndexPath *selectedRow;
@end

#define kOffsetY 160.0f

@implementation BRRecordedTracksViewController
@synthesize tempStation;
@synthesize delegate;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        
    }
    return self;
}

- (id)initWithManager:(ButterflyManager *)mgr
{
    self = [super initWithManager:mgr];
    if (self) {
        reloadable = NO;
        Database *db = [Database database];
        
        self.tempStation = [[[Station alloc ] init] autorelease];
        self.tempStation.tracks = [db fetchAllRecordings];
        
    }
    return self;
}

- (void)dealloc
{
    self.delegate = nil;
    self.theTableview = nil;
    self.tempStation = nil;
    self.gradientBar = nil;
    self.selectedRow = nil;
    [super dealloc];
}

- (void)loadView
{
    UIView *view = [self baseViewWithColor:[UIColor colorWithPatternImage:[UIImage imageNamed:@"bg_blue.png"]]];
    CGRect frame = view.frame;
    
    self.gradientBar = [BRGradientBar barWithFrame:CGRectMake(0, kOffsetY, frame.size.width, 0)];
    self.gradientBar.delegate = self;
    self.gradientBar.lblTitle.text = @"Saved";
    
    CGFloat y = self.gradientBar.frame.origin.y+self.gradientBar.frame.size.height;
    UIScrollView *theScrollview = [[UIScrollView alloc] initWithFrame:CGRectMake(0, y, frame.size.width, frame.size.height-y)];
    theScrollview.backgroundColor = [UIColor lightGrayColor];
    theScrollview.alpha = 0.6f;
    theScrollview.contentSize = CGSizeMake(0, 500);
    
    UILabel *lblDescTitle = [[UILabel alloc] initWithFrame:CGRectMake(0.0f, 0.0f, frame.size.width, 30.0f)];
    lblDescTitle.backgroundColor = [UIColor darkGrayColor];
    lblDescTitle.textAlignment = NSTextAlignmentCenter;
    lblDescTitle.text = @"Saved Tracks";
    [theScrollview addSubview:lblDescTitle];
    [lblDescTitle release];

    
    NSString *path = [[NSBundle mainBundle] pathForResource:@"savedtracks" ofType:@"txt"];
    NSError *error = nil;
    NSString *text = [[NSString alloc] initWithContentsOfFile:path encoding:NSUTF8StringEncoding error:&error];
    
    UILabel *lblDescription = [[UILabel alloc] initWithFrame:CGRectZero];
    lblDescription.font = [UIFont fontWithName:kFont size:14.0f];
    lblDescription.backgroundColor = [UIColor clearColor];
    lblDescription.textColor = [UIColor whiteColor];
    lblDescription.lineBreakMode = NSLineBreakByWordWrapping;
    lblDescription.numberOfLines = 0;
    lblDescription.text = text;
    [text release];
    CGSize size = [text sizeWithFont:lblDescription.font constrainedToSize:CGSizeMake(frame.size.width-20.0f, 500.0f) lineBreakMode:lblDescription.lineBreakMode];
    lblDescription.frame = CGRectMake(10.0f, 40.0f, frame.size.width-20.0f, size.height);
    [theScrollview addSubview:lblDescription];
    [lblDescription release];
    theScrollview.contentSize = CGSizeMake(0, lblDescription.frame.size.height+90.0f+self.gradientBar.frame.size.height);
    
    
    [view addSubview:theScrollview];
    [theScrollview release];

    [view addSubview:self.gradientBar];

    
    frame.origin.y = self.gradientBar.frame.origin.y+self.gradientBar.frame.size.height;
    frame.size.height -= frame.origin.y;
    self.theTableview = [[[UITableView alloc] initWithFrame:frame style:UITableViewStylePlain] autorelease];
    self.theTableview.autoresizingMask = UIViewAutoresizingFlexibleHeight;
    self.theTableview.dataSource = self;
    self.theTableview.delegate = self;
    [view addSubview:self.theTableview];

    self.view = view;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    [self setUpBackButton:self action:@selector(exit)];
}



#pragma mark -
- (void)exit
{
    [self.navigationController popViewControllerAnimated:YES];
}

- (void)shiftUp:(NSTimeInterval)duration
{
    [self.gradientBar rotateButton:CGAffineTransformIdentity];
    [UIView animateWithDuration:duration
                          delay:0.0f
                        options:UIViewAnimationOptionCurveLinear
                     animations:^{
                         CGRect frame = self.gradientBar.frame;
                         frame.origin.y = kOffsetY;
                         self.gradientBar.frame = frame;
                         
                         frame = self.theTableview.frame;
                         frame.origin.y = self.gradientBar.frame.origin.y+self.gradientBar.frame.size.height;
                         self.theTableview.frame = frame;
                     }
                     completion:^(BOOL finished){
                         [UIView animateWithDuration:0.07f
                                               delay:0.0f
                                             options:UIViewAnimationOptionCurveLinear
                                          animations:^{
                                              CGRect frame = self.gradientBar.frame;
                                              frame.origin.y += 10.0f;
                                              self.gradientBar.frame = frame;
                                              
                                              frame = self.theTableview.frame;
                                              frame.origin.y += 10.0f;
                                              self.theTableview.frame = frame;
                                          }
                                          completion:^(BOOL fished){
                                              [UIView animateWithDuration:0.07f
                                                                    delay:0.0f
                                                                  options:UIViewAnimationOptionCurveLinear
                                                               animations:^{
                                                                   CGRect frame = self.gradientBar.frame;
                                                                   frame.origin.y = kOffsetY;
                                                                   self.gradientBar.frame = frame;
                                                                   
                                                                   frame = self.theTableview.frame;
                                                                   frame.origin.y = self.gradientBar.frame.origin.y+self.gradientBar.frame.size.height;
                                                                   self.theTableview.frame = frame;
                                                               }
                                                               completion:NULL];
                                          }];
                     }];
    
}

- (void)shiftDown:(NSTimeInterval)duration
{
    [self.gradientBar rotateButton:CGAffineTransformMakeRotation(M_PI)];
    [UIView animateWithDuration:duration
                          delay:0.0f
                        options:UIViewAnimationOptionCurveEaseInOut
                     animations:^{
                         CGRect frame = self.theTableview.frame;
                         frame.origin.y = self.view.frame.size.height;
                         self.theTableview.frame = frame;
                         
                         frame = self.gradientBar.frame;
                         frame.origin.y = self.view.frame.size.height-frame.size.height;
                         self.gradientBar.frame = frame;
                     }
                     completion:^(BOOL finished){
                         [UIView animateWithDuration:0.07f
                                               delay:0.0f
                                             options:UIViewAnimationOptionCurveLinear
                                          animations:^{
                                              CGRect frame = self.theTableview.frame;
                                              frame.origin.y -= 10;
                                              self.theTableview.frame = frame;
                                              
                                              frame = self.gradientBar.frame;
                                              frame.origin.y -= 10;
                                              self.gradientBar.frame = frame;
                                          }
                                          completion:^(BOOL finished){
                                              [UIView animateWithDuration:0.07f
                                                                    delay:0.0f
                                                                  options:UIViewAnimationOptionCurveLinear
                                                               animations:^{
                                                                   CGRect frame = self.theTableview.frame;
                                                                   frame.origin.y = self.view.frame.size.height;
                                                                   self.theTableview.frame = frame;
                                                                   
                                                                   frame = self.gradientBar.frame;
                                                                   frame.origin.y = self.view.frame.size.height-frame.size.height;
                                                                   self.gradientBar.frame = frame;
                                                               }
                                                               completion:NULL];
                                          }];
                     }];
}

#pragma mark - GradientBarDelegate
- (void)btnGradientTapped:(UIButton *)btn
{
    NSLog(@"btnGradientTapped:");
    (self.gradientBar.center.y < 280) ? [self shiftDown:0.2f] : [self shiftUp:0.2f];
}

- (void)gradientBarMoved:(double)diff
{
    NSLog(@"gradientBarMoved: %.2f", diff);
    
    CGPoint ctr = self.theTableview.center;
    ctr.y += diff;
    self.theTableview.center = ctr;
}

- (void)gradientBarStopped
{
    NSLog(@"gradientBarStopped: %.2f", self.gradientBar.center.y);
    (self.gradientBar.center.y > 260) ? [self shiftDown:0.2f] : [self shiftUp:0.2f];
}

- (void)swipe:(double)direction
{
    NSLog(@"SWIPE: %.2f", direction); // if dirction>0, sliding down:
    (direction > 0) ? [self shiftDown:0.15f] : [self shiftUp:0.15f];
}

#pragma mark - UITableViewDataSource
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return self.self.tempStation.tracks.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *cellID = @"ID";
    UITableViewCell *cell = [self.theTableview dequeueReusableCellWithIdentifier:cellID];
    if (cell==nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:cellID] autorelease];
        
        cell.selectionStyle = UITableViewCellSelectionStyleGray;
        cell.textLabel.font = [UIFont fontWithName:@"Heiti SC" size:14.0f];
        cell.textLabel.textColor = kBRRed;
        cell.textLabel.shadowColor = [UIColor whiteColor];
        cell.textLabel.shadowOffset = CGSizeMake(-0.5f, 0.5f);
        cell.textLabel.backgroundColor = [UIColor clearColor];
        
        cell.detailTextLabel.numberOfLines = 2;
        cell.detailTextLabel.font = [UIFont systemFontOfSize:10.0f];
        cell.detailTextLabel.backgroundColor = [UIColor clearColor];

        CGFloat w = 60.0f;
        UILabel *lblDuration = [[UILabel alloc] initWithFrame:CGRectMake(tableView.frame.size.width-w-10, 40.0f, w, 15.0f)];
        lblDuration.textAlignment = NSTextAlignmentRight;
        lblDuration.tag = 1000;
        lblDuration.textColor = kBRRed;
        lblDuration.font = [UIFont fontWithName:@"Heiti SC" size:10.0f];
        lblDuration.text = @"duration";
        
        [cell.contentView addSubview:lblDuration];
        [lblDuration release];
    }
    
    BRTrack *track = (BRTrack *)[self.tempStation.tracks objectAtIndex:indexPath.row];
    cell.textLabel.text = track.name;
    cell.detailTextLabel.text = [NSString stringWithFormat:@"By %@\n%@", track.author, track.date];
    
    UILabel *lblDuration = (UILabel *)[cell.contentView viewWithTag:1000];
    lblDuration.text = [NSString stringWithFormat:@"%d seconds", (int)track.duration];
    cell.imageView.image = [UIImage imageNamed:@"icon_microphone.png"];
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    BRTrack *track = (BRTrack *)[self.tempStation.tracks objectAtIndex:indexPath.row];
    NSLog(@"%@ == %d bytes", track.name, track.fileData.length);

    if ([delegate respondsToSelector:@selector(savedTrackSelected:)])
        [delegate savedTrackSelected:track];
    
    [self.navigationController popViewControllerAnimated:YES];
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return kCellHeight;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    NSLog(@"tableView commitEditingStyle: %d", indexPath.row);
    BRTrack *track = (BRTrack *)[self.tempStation.tracks objectAtIndex:indexPath.row];
    if (!track)
        return;
    
    self.selectedRow = indexPath;
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Are You Sure?" message:@"This will permanently remove the track. This action cannot be undone." delegate:self cancelButtonTitle:@"yes" otherButtonTitles:@"no", nil];
    [alert show];
    [alert release];
}


- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    if (buttonIndex==0){
        BRTrack *track = (BRTrack *)[self.tempStation.tracks objectAtIndex:self.selectedRow.row];
        if (!track)
            return;
        
        [track deleteFromDb];
        [self.tempStation.tracks removeObject:track];
        [self.theTableview deleteRowsAtIndexPaths:@[self.selectedRow] withRowAnimation:UITableViewRowAnimationFade];
        self.selectedRow = nil;
    }
}


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

@end
