//
//  BRStationViewController.m
//  ButterflySDK
//
//  Created by Denny Kwon on 1/11/13.
//  Copyright (c) 2013 ButterflyRadio. All rights reserved.
//

#import "BRStationViewController.h"
#import "BRContentCell.h"
#import "ArticleViewController.h"
#import "BRStationInfoView.h"
#import "BRRecordViewController.h"
#import "SubmitArticleViewController.h"
#import "BRPurchaseViewController.h"
#import "LoginViewController.h"
#import <QuartzCore/QuartzCore.h>

@interface BRStationViewController ()
@property (retain, nonatomic) UILabel *lblDescription;
@property (retain, nonatomic) UIImageView *pointer;
@property (retain, nonatomic) UIBarButtonItem *btnSave;
@property (retain, nonatomic) UITableView *theTableview;
@property (nonatomic) BOOL purchasing;
@property (nonatomic) BOOL hostAuthorized;
@end

@implementation BRStationViewController
@synthesize station;
@synthesize stationImage;
@synthesize admin;

- (id)initWithManager:(ButterflyManager *)mgr
{
    self = [super initWithManager:mgr];
    if (self) {
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(playPremiumTrack)
                                                     name:kRefreshNotification
                                                   object:nil];

        reloadable = NO;
        self.admin = NO;
    }
    return self;
}



- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self
                                                    name:kRefreshNotification
                                                  object:nil];
    
    self.station.delegate = nil;
    self.station = nil;
    self.pointer = nil;
    self.stationImage = nil;
    self.btnSave = nil;
    self.lblDescription = nil;
    self.theTableview = nil;
    [super dealloc];
}

- (void)loadView
{
    reloadable = YES;
    UIView *view = [self baseViewWithColor:kBRYellow];
    CGRect frame = view.frame;
    
    CGFloat padding = 10.0f;
    frame = CGRectMake(0.0f, 0.0f, frame.size.width, frame.size.height);
    
    CGFloat maxDimen = 120.0f;
    stationImage = [[UIImageView alloc] initWithFrame:CGRectMake(padding, padding, maxDimen, maxDimen)];
    self.stationImage.layer.borderColor = [[UIColor blackColor] CGColor];
    self.stationImage.layer.borderWidth = 1.0f;
    if (self.station.imgData){
        UIImage *image = [UIImage imageWithData:self.station.imgData];
        CGSize size = [self formattedSize:image];
        CGRect frame = self.stationImage.frame;
        self.stationImage.frame = CGRectMake(frame.origin.x, frame.origin.y, size.width, size.height);
        self.stationImage.image = image;
    }
    else{
        static NSString *logos[5] = {@"_red.png", @"_green.png", @"_yellow.png", @"_orange.png", @"_blue.png"};
        int i = abs(arc4random())%5;
        NSString *color = logos[i];
        NSString *logo = @"logo";
        logo = [logo stringByAppendingString:color];
        self.stationImage.image = [UIImage imageNamed:logo];
    }
    [view addSubview:self.stationImage];
    
    CGFloat w = 160.0f;
    BRStationInfoView *infoPanel = [[BRStationInfoView alloc] initWithFrame:CGRectMake(frame.size.width-w-padding, padding, w, 100.0f)];
    infoPanel.lblName.text = [NSString stringWithFormat:@"%@ ", self.station.name];
    infoPanel.lblCategory.text = [NSString stringWithFormat:@"category: %@", self.station.category];
    infoPanel.lblTracks.text = ([self.station.tracks containsObject:@"none"]) ? @"No Tracks" : [NSString stringWithFormat:@"%d tracks", self.station.tracks.count];
    infoPanel.lblAdmins.text = [NSString stringWithFormat:@"%d moderators", self.station.admins.count];
    
    [view addSubview:infoPanel];
    [infoPanel release];
    
    
    CGFloat y = padding+maxDimen+padding;
    NSArray *buttons = ([self.station.podcastRss isEqualToString:@"none"]) ? @[@"Tracks", @"Premium"] : @[@"Tracks", @"Premium", @"Podcast"];
    w = (frame.size.width-1)/buttons.count;
    
    for (NSString *buttonTitle in buttons) {
        int index = [buttons indexOfObject:buttonTitle];
        UIButton *btn = [UIButton buttonWithType:UIButtonTypeCustom];
        btn.tag = index+1000;
        btn.frame = CGRectMake(0.5+index*w, y, w, 36.0f);
        btn.titleLabel.font = [UIFont fontWithName:kFont size:13.0f];
        UIImage *bg = ([buttonTitle isEqual:[buttons lastObject]]) ? [UIImage imageNamed:@"bg_button_right.png"] : [UIImage imageNamed:@"bg_button.png"];
        
        [btn setBackgroundImage:bg forState:UIControlStateNormal];
        [btn setTitleColor:[UIColor darkGrayColor] forState:UIControlStateNormal];
        [btn setTitle:buttonTitle forState:UIControlStateNormal];
        [btn addTarget:self action:@selector(btnTapped:) forControlEvents:UIControlEventTouchUpInside];
        [view addSubview:btn];
    }
    y += 29.0f;
    
    UIImage *ptr = [UIImage imageNamed:@"pointer.png"];
    self.pointer = [[[UIImageView alloc] initWithImage:ptr] autorelease];
    self.pointer.frame = CGRectMake(0, 0, 0.50*ptr.size.width, 0.50*ptr.size.height);
    self.pointer.center = CGPointMake(0.5*w, y);
    [view addSubview:self.pointer];
    y += 5.0f;

    UIView *line = [[UIView alloc] initWithFrame:CGRectMake(0, y, frame.size.width, 8)];
    line.backgroundColor = kBRRed;
    [view addSubview:line];
    [line release];
    y += 8.0f;
    
    CGFloat height = 44.0f;
    self.theTableview = [[[UITableView alloc] initWithFrame:CGRectMake(0.0f, y, frame.size.width, frame.size.height-y) style:UITableViewStylePlain] autorelease];
    self.theTableview.dataSource = self;
    self.theTableview.delegate = self;
    self.theTableview.tag = 1000;
    self.theTableview.autoresizingMask = UIViewAutoresizingFlexibleHeight;
    self.theTableview.showsVerticalScrollIndicator = NO;
    UIView *footerView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, self.theTableview.frame.size.width, height)];
    footerView.backgroundColor = [UIColor clearColor];
    self.theTableview.tableFooterView = footerView;
    [footerView release];
    
    [self addPullToRefreshHeader:self.theTableview];
    
    [view addSubview:self.theTableview];

    padding = 10.0f;
    self.bottomToolbar = [[BRToolbar alloc] initWithFrame:CGRectMake(0, view.frame.size.height-height+1, view.frame.size.width, height)];
    self.bottomToolbar.barStyle = UIBarStyleBlack;
    self.bottomToolbar.translucent = YES;
    self.bottomToolbar.autoresizingMask = (UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleTopMargin);
    
    [self.station checkDatabase];
    UIImage *imgHeart = (self.station.saved) ? [UIImage imageNamed:@"toolbarHeartRed.png"] : [UIImage imageNamed:@"toolbarHeart.png"];
    self.btnSave = [self barButtonWithImage:imgHeart tag:1000];
    
    NSArray *barButtons = nil;
    if (self.admin){
        barButtons = @[[self flex], self.btnSave, [self flex], [self barButtonWithImage:[UIImage imageNamed:@"toolbarContact.png"] tag:1002], [self flex], [self barButtonWithImage:[UIImage imageNamed:@"toolbarPost.png"] tag:1003], [self flex]];
    }
    else{
        barButtons = @[[self flex], [self barButtonWithImage:[UIImage imageNamed:@"toolbarSettings.png"] tag:1001], [self flex], self.btnSave, [self flex], [self barButtonWithImage:[UIImage imageNamed:@"toolbarContact.png"] tag:1002], [self flex], [self barButtonWithImage:[UIImage imageNamed:@"toolbarPost.png"] tag:1003], [self flex]];
    }
    
    
    self.bottomToolbar.items = barButtons;
    [view addSubview:self.bottomToolbar];
    
    self.view = view;
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    [self.theTableview deselectRowAtIndexPath:[self.theTableview indexPathForSelectedRow] animated:YES];
    
    if (!self.butterflyMgr.host.loggedIn)
        self.hostAuthorized = NO;
    
    // current user is author of track. it's all good.
    if ([self.station.host isEqualToString:self.butterflyMgr.host.email])
        self.hostAuthorized = YES;
    
    // user is an admin. it's all good.
    if ([self.station.admins containsObject:self.butterflyMgr.host.email])
        self.hostAuthorized = YES;

    [self.theTableview reloadData];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    [self setUpBackButton:self action:@selector(exit)];
    [self setupNowPlayingButton];
    [self setupLoadingIndicator];
    [self setupLoadingIndicator:2 verticalOffset:160.0f];

    self.lblDescription.text = self.station.description;
    
    CGSize size = [self.station.description sizeWithFont:self.lblDescription.font constrainedToSize:CGSizeMake(self.lblDescription.frame.size.width, 1000.0f) lineBreakMode:self.lblDescription.lineBreakMode];
    CGRect frame = self.lblDescription.frame;
    frame.size.height = size.height;
    self.lblDescription.frame = frame;
    
    if (self.station.imgData)
        return;
    
    if ([self.station.image rangeOfString:@"default.jpg"].location!=NSNotFound)
        return;
        
    self.station.delegate = self;
    [self.station fetchImage];
}

- (void)exit
{
    [self.navigationController popViewControllerAnimated:YES];
}

- (void)updateInfo //subclass should always override this!
{
    NSLog(@"%@ - UPDATE INFO", [[self class] description]);
    self.station.delegate = self;
    [self.station getStationInfo];
    
}

- (void)barButtonTapped:(UIBarButtonItem *)btn
{
    NSLog(@"barButtonTapped: %d", btn.tag);
    if (btn.tag==1000){ // save to favorites
        NSLog(@"STATION VIEW CONTROLLER - saveStation");
        if (station.saved){ //remove station from favorites:
            [self.station deleteFromDb];
            
            UIImage *imgHeart = [UIImage imageNamed:@"toolbarHeart.png"];
            UIButton *btn = [UIButton buttonWithType:UIButtonTypeCustom];
            [btn setBackgroundImage:imgHeart forState:UIControlStateNormal];
            btn.frame = CGRectMake(0, 0, imgHeart.size.width, imgHeart.size.height);
            self.btnSave.customView = btn;
            
            [self showAlert:@"Station Removed" message:[NSString stringWithFormat:@"%@ was removed from your favorites.", station.name]];
        }
        else{ // save station to favorites:
            [self.station save];
            UIImage *imgHeart = [UIImage imageNamed:@"toolbarHeartRed.png"];
            UIButton *btn = [UIButton buttonWithType:UIButtonTypeCustom];
            [btn setBackgroundImage:imgHeart forState:UIControlStateNormal];
            btn.frame = CGRectMake(0, 0, imgHeart.size.width, imgHeart.size.height);
            self.btnSave.customView = btn;
            
            [self showAlert:@"Station Saved" message:[NSString stringWithFormat:@"%@ was saved to your favorites.", station.name]];
        }
    }
    if (btn.tag==1001){ // Settings:
        [self showAccount];
    }
    
    if (btn.tag==1002){ // email
        if ([MFMailComposeViewController canSendMail]==NO){
            [self showAlert:@"Error" message:@"Please configure the email account on your device."];
            return;
        }
        
        MFMailComposeViewController *picker = [[MFMailComposeViewController alloc] init];
        picker.navigationBar.barStyle = UIBarStyleBlack;
        picker.mailComposeDelegate = self;
        [picker setToRecipients:self.station.admins];
        [picker setSubject:[NSString stringWithFormat:@"Butterfly Radio: %@", self.station.name]];
        
        NSString *emailBody = [NSString stringWithFormat:@"<html><body><a href='http://www.butterflyradio.com'><img style='width:250px' src='http://www.butterflyradio.com/site/img/logo.png' /></a><br /><br /></body></html>"];
        [picker setMessageBody:emailBody isHTML:YES];
        
        [self presentModalViewController:picker animated:YES];
        [picker release];
        return;
    }
    
    if (btn.tag==1003){ // post track:
        BRRecordViewController *submitTrack = [[BRRecordViewController alloc] initWithManager:self.butterflyMgr];
        submitTrack.station = self.station;
        
        UINavigationController *nav = [[UINavigationController alloc] initWithRootViewController:submitTrack];
        [submitTrack release];
        nav.navigationBar.barStyle = UIBarStyleBlack;
        nav.modalTransitionStyle = UIModalTransitionStyleFlipHorizontal;
        [self presentModalViewController:nav animated:YES];
        [nav release];
        return;
    }
}


#pragma mark -  MFMailComposeViewControllerDelegate
- (void)mailComposeController:(MFMailComposeViewController *)controller didFinishWithResult:(MFMailComposeResult)result error:(NSError *)error
{
    NSLog(@"controller didFinishWithResult:");
    controller.delegate = nil;
    [controller dismissViewControllerAnimated:YES completion:NULL];
}


#pragma mark - Station Delegate
- (void)stationInfoReady:(NSString *)stationId
{
    NSLog(@"stationInfoReady:");
    [self.theTableview reloadData];
    [self stopLoading:self.theTableview];
}

- (void)imageReady:(NSString *)stationId
{
    UIImage *image = [UIImage imageWithData:self.station.imgData];
    CGSize size = [self formattedSize:image];
    CGRect frame = self.stationImage.frame;
    
    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDuration:0.4f];
    [UIView setAnimationTransition:UIViewAnimationTransitionFlipFromLeft forView:self.stationImage cache:YES];
    self.stationImage.frame = CGRectMake(frame.origin.x, frame.origin.y, size.width, size.height);
    self.stationImage.image = image;
    [UIView commitAnimations];
}

- (CGSize)formattedSize:(UIImage *)image
{
    CGFloat max = self.stationImage.frame.size.width;
    CGFloat w = image.size.width;
    CGFloat h = image.size.height;
    double scale;
    if (w > max){
        scale = max/w;
        w = max;
        h *= scale;
    }
    if (h>max){
        scale = max/h;
        h = max;
        w *= scale;
    }
    
    CGSize size = CGSizeMake(w, h);
    return size;
}

- (void)reloadViews:(CGPoint)center
{
    [self.theTableview reloadData];

    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDuration:0.6f];
    [UIView setAnimationTransition:UIViewAnimationTransitionFlipFromLeft forView:self.theTableview cache:YES];
    [self.theTableview setContentOffset:CGPointMake(0.0f, 0.0f) animated:NO];
    self.theTableview.alpha = 1.0f;
    self.pointer.center = center;
    [UIView commitAnimations];
}

- (void)btnTapped:(UIButton *)btn
{
    NSLog(@"btnTapped: %@, tag == %d", btn.titleLabel.text, btn.tag);
    if (btn.tag < 1002){
        int x = btn.tag-1000;
        if (x < 0)
            return;
        
        self.theTableview.tag = btn.tag;
        [self reloadViews:CGPointMake(btn.center.x, self.pointer.center.y)];
        return;
    }
    
    // Podcast Button
    NSLog(@"%@", self.station.podcastRss);
    if (self.station.podcastTracks.count > 0){ // podcast already downloaded:
        self.theTableview.tag = btn.tag;
        [self reloadViews:CGPointMake(btn.center.x, self.pointer.center.y)];
        return;
    }
    
    self.station.delegate = self;
    [self.station downloadPodcast];
    [self.loading show];
}

- (void)podcastTracksReady
{
    NSLog(@"podcastTracksReady");
    [self.loading hide];
    UIButton *btn = (UIButton *)[self.view viewWithTag:1002];
    if (!btn)
        return;
    
    
    self.theTableview.tag = btn.tag;
    [self reloadViews:CGPointMake(btn.center.x, self.pointer.center.y)];
}

- (void)playPremiumTrack
{
    NSLog(@"PLAY PREMIUM TRACK");
    if (!self.loading.hidden)
        [self.loading hide];
    
    if (!self.premiumTrack)
        return;
    
    [self.butterflyMgr playTrack:self.premiumTrack fromStation:self.station];
    [self showRadioWithLoader];
    self.premiumTrack = nil;
}



#pragma mark - UITableViewStuff
- (void)populateCell:(BRContentCell *)cell withTrack:(BRTrack *)track
{
    cell.accessoryType = UITableViewCellAccessoryNone;
    if (track.reply){
        cell.imageView.image = [UIImage imageNamed:@"icon_reply.png"];
        cell.btnReply.hidden = YES;
    }
    else{
        cell.imageView.image = [UIImage imageNamed:@"icon_microphone.png"];
        cell.btnReply.hidden = NO;
    }
    
    cell.textLabel.text = track.name;
    cell.detailTextLabel.text = [NSString stringWithFormat:@"By %@\n%@", track.author, track.date];
    cell.lblPrice.text = (track.value>0) ? [NSString stringWithFormat:@"%d tickets", track.value] : @"";
    
    if (track.value==0)
        return;
    
    //premium track:
    if (self.butterflyMgr.host.loggedIn){
        
        // previously purchased, has access:
        if ([self.butterflyMgr.host.premiumThreads containsObject:track.thread]){
            cell.btnReply.hidden = NO;
            cell.lblAuthorized.hidden = NO;
            cell.lblPrice.hidden = YES;
            return;
        }
        
        // current host is owner or admin of the station:
        if (self.hostAuthorized){
            cell.btnReply.hidden = NO;
            cell.lblAuthorized.hidden = NO;
            cell.lblPrice.hidden = YES;
            return;
        }
        
        // not authorzed
        cell.btnReply.hidden = YES;
        cell.lblAuthorized.hidden = YES;
        cell.lblPrice.hidden = NO;
        return;
    }
    
    // not logged in, hide reply label and show price label:
    cell.btnReply.hidden = YES;
    cell.lblAuthorized.hidden = YES;
    cell.lblPrice.hidden = NO;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    int tableId = tableView.tag;
    if (tableId==1000){ // Tracks
        if ([self.station.tracks containsObject:@"none"]==TRUE || self.station.tracks.count==0)
            return 1;
        return station.tracks.count;
    }
    
    if (tableId==1001){ // Premium
        if ([self.station.premiumTracks containsObject:@"none"]==TRUE || self.station.premiumTracks.count==0)
            return 1;
        return station.premiumTracks.count;
    }

    if (tableId==1002) // Podcasts
        return station.podcastTracks.count;

    //Twitter
    return 40;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    int tableId = tableView.tag;
    
    static NSString *cellID = @"ID";
    BRContentCell *cell = (BRContentCell *)[tableView dequeueReusableCellWithIdentifier:cellID];
    if (cell==nil) {
        cell = [[[BRContentCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:cellID] autorelease];
        [cell setup];
        cell.delegate = self;
    }
    
    [cell clear:@""];
    if (tableId==1000){ // Tracks
        if ([self.station.tracks containsObject:@"none"]==TRUE || self.station.tracks.count==0){
            cell.imageView.image = [UIImage imageNamed:@"icon_microphone.png"];
            [cell clear:@"No Tracks"];
            return cell;
        }

        BRTrack *track = (BRTrack *)[self.station.tracks objectAtIndex:indexPath.row];
        cell.tag = indexPath.row+1000;
        cell.btnReply.hidden = NO;
        [self populateCell:cell withTrack:track];
        return cell;
    }
    
    if (tableId==1001){ // Premium
        if ([self.station.premiumTracks containsObject:@"none"]==TRUE || self.station.premiumTracks.count==0){
            cell.imageView.image = [UIImage imageNamed:@"icon_microphone.png"];
            [cell clear:@"No Premium Tracks"];
            return cell;
        }
        
        BRTrack *track = (BRTrack *)[self.station.premiumTracks objectAtIndex:indexPath.row];
        cell.tag = indexPath.row+1000;
        [self populateCell:cell withTrack:track];
        return cell;
    }
    
    // Podcast:
    if (self.station.podcastTracks.count==0){
        cell.imageView.image = [UIImage imageNamed:@"icon_microphone.png"];
        [cell clear:@"No Podcast"];
        return cell;
    }

    BRTrack *track = (BRTrack *)[self.station.podcastTracks objectAtIndex:indexPath.row];
    cell.tag = indexPath.row+1000;
    [self populateCell:cell withTrack:track];
    cell.btnReply.hidden = YES;
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return kCellHeight;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    int tableId = tableView.tag;
    if (tableId==1000){ // Free Track:
        if ([self.station.tracks containsObject:@"none"]){
            [tableView deselectRowAtIndexPath:[tableView indexPathForSelectedRow] animated:YES];
            return;
        }
        
        BRTrack *track = (BRTrack *)[self.station.tracks objectAtIndex:indexPath.row];
        if (!track)
            return;
        
        if (track.value > 0) // has to be free:
            return;

        NSLog(@"PLAY TRACK: %@", track.name);
        [self.butterflyMgr playTrack:track fromStation:self.station];
        [self showRadioWithLoader];
        return;
    }
    
    if (tableId==1001){ // Premium Track:
        if (self.station.premiumTracks.count==0){
            [tableView deselectRowAtIndexPath:[tableView indexPathForSelectedRow] animated:YES];
            return;
        }
        
        BRTrack *track = (BRTrack *)[self.station.premiumTracks objectAtIndex:indexPath.row];
        if (!track)
            return;
        
        BRTrackAuthorization trackAuthorization = [self trackAuthorizationStatus:track fromStation:self.station];
        if (trackAuthorization==BRTrackAuthorizationAuthorized){
            [self.butterflyMgr playTrack:track fromStation:self.station];
            [self showRadioWithLoader];
            return;
        }
        
        [tableView deselectRowAtIndexPath:[tableView indexPathForSelectedRow] animated:YES];
        self.premiumTrack = track;
        self.purchasing = (trackAuthorization==BRTrackAuthorizationNeedMoreTickets) ? YES : NO;
        return;
    }
    
    // Podcast Track:
    NSLog(@"PLAY PODCAST");
    
    if (self.station.podcastTracks.count==0){
        [tableView deselectRowAtIndexPath:[tableView indexPathForSelectedRow] animated:YES];
        return;
    }
    
    BRTrack *track = (BRTrack *)[self.station.podcastTracks objectAtIndex:indexPath.row];
    if (!track)
        return;

    [self.butterflyMgr playTrack:track fromStation:self.station];
    [self showRadioWithLoader];
}

#pragma mark - ContentCellDelegate
- (void)cellTapped:(int)index
{
    NSLog(@"cellTapped: %d", index);
    if (index < 1000)
        return;
    
    int i = index-1000;
    BRTrack *track = (BRTrack *)[self.station.tracks objectAtIndex:i];
    if (!track)
        return;
    
    BRRecordViewController *submitTrack = [[BRRecordViewController alloc] initWithManager:self.butterflyMgr];
    submitTrack.butterflyMgr = self.butterflyMgr;
    
    if (track.thread != nil){
        NSLog(@"REPLY TO: %@, %@", track.name, track.thread);
        submitTrack.thread = track.thread;
    }
    
    submitTrack.station = self.station;
    UINavigationController *nav = [[UINavigationController alloc] initWithRootViewController:submitTrack];
    nav.navigationBar.barStyle = UIBarStyleBlack;
    [submitTrack release];
    
    submitTrack.modalTransitionStyle = UIModalTransitionStyleFlipHorizontal;
    [self presentViewController:nav animated:YES completion:NULL];
    [nav release];

}

#pragma mark - UIAlertViewDelegate
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    if (buttonIndex==0) {
        if (self.purchasing){
            NSLog(@"PURCHASE !");
            UIImage *img = [self.navigationController screenshot];
            BRPurchaseViewController *purchaseVc = [[BRPurchaseViewController alloc] initWithBackgroundImage:img];
            purchaseVc.butterflyMgr = self.butterflyMgr;
            [self presentModalViewController:purchaseVc animated:NO];
            [purchaseVc release];
            self.purchasing = NO;
        }
        else{
            //user spends tickets to listen to thread.
            //start streaming the track, update profile with authorization for thread, persist to server
            
            NSLog(@"ADD PREMIUM THREAD:");
            if (self.premiumTrack){
                [self.butterflyMgr.host addPremiumThread:self.premiumTrack];
                [self.loading show];
                //wait for kRefresehNotification, if successful - start playback
            }
        }
        return;
    }
    
    self.premiumTrack = nil;
}


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

@end
