//
//  SearchViewController.m
//  butterflyradio
//
//  Created by Denny Kwon on 5/26/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "SearchViewController.h"
#import "BRStationViewController.h"
#import "HistoryCell.h"


@interface SearchViewController ()

@end

@implementation SearchViewController
@synthesize searchResults;
@synthesize cellBg;
@synthesize screen;
@synthesize theTableview;
@synthesize theSearchBar;


- (id)init
{
    self = [super init];
    if (self) {
        [self setup];
    }
    return self;
}

- (id)initWithManager:(ButterflyManager *)mgr
{
    self = [super initWithManager:mgr];
    if (self) {
        [self setup];
    }
    return self;
}

- (void)setup
{
    self.title = @"Search";
    self.cellBg = [UIImage imageNamed:@"bg_cell_station.png"];
    self.tabBarItem.image = [UIImage imageNamed:@"tab_search.png"];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(refresh) name:kThumbnailReadyNotification object:nil];
}

- (void)dealloc
{
    self.theSearchBar = nil;
    self.theTableview = nil;
    self.screen = nil;
    [cellBg release];
    if (searchResults!=nil){
        [searchResults release];
    }
    [super dealloc];
}

- (void)loadView
{
    reloadable = NO;
    CGRect frame = [UIScreen mainScreen].applicationFrame;
    frame.origin.y = 0.0f;
    frame.origin.x = 0.0f;
    UIViewAutoresizing resize = (UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleWidth);
    
    UIView *view = [[UIView alloc] initWithFrame:frame];
    view.autoresizingMask = resize;
    view.backgroundColor = kBlue;
        
    
    theSearchBar = [[UISearchBar alloc] initWithFrame:CGRectMake(0, 0, frame.size.width, 40)];
    self.theSearchBar.barStyle = UIBarStyleBlack;
    self.theSearchBar.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    self.theSearchBar.delegate = self;
    [view addSubview:self.theSearchBar];
    
    frame.origin.y = theSearchBar.frame.size.height;
    frame.size.height -= frame.origin.y;
    theTableview = [[UITableView alloc] initWithFrame:frame style:UITableViewStylePlain];
    self.theTableview.dataSource = self;
    self.theTableview.delegate = self;
    self.theTableview.separatorStyle = UITableViewCellSelectionStyleNone;
    self.theTableview.showsVerticalScrollIndicator = NO;
    self.theTableview.backgroundColor = [UIColor clearColor];
    self.theTableview.autoresizingMask = resize;
    self.theTableview.hidden = YES;
    [view addSubview:self.theTableview];
    
    screen = [[UIView alloc] initWithFrame:frame];
    self.screen.autoresizingMask = (UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight);
    self.screen.backgroundColor = [UIColor blackColor];
    self.screen.hidden = YES;
    self.screen.alpha = 0.6f;
    [view addSubview:self.screen];

    self.view = view;
    [view release];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    [self setupLoadingIndicator];
    [self setupNowPlayingButton];
}

- (void)viewWillAppear:(BOOL)animated
{
    [self.theTableview deselectRowAtIndexPath:[theTableview indexPathForSelectedRow] animated:YES];
}

- (void)search
{
    NSString *term = theSearchBar.text;
    if ([term length]==0){
        [self showAlert:@"Missing Value" message:@"Please fill in the search field."];
        return;
    }
    
    term = [term stringByReplacingOccurrencesOfString:@" " withString:@"+"];
    term = [term lowercaseString];
    
    if (self.request!=nil){
        self.request.delegate = nil;
        [self.request cancel];
    }
    
    NSString *url = [NSString stringWithFormat:@"http://www.butterflyradio.com/api/station?tag=%@", term];
    self.request = [BRNetworkOp operationWithAddress:url parameters:nil];
    [self.request setHttpMethod:@"GET"];
    self.request.delegate = self;
    [self.request sendRequest];
    [self.loading show];
}

- (void)requestData:(NSArray *)pkg; //returns [address, data]
{
    if (!pkg)
        return;
    
    id json = [self.request parseJsonResponse];
    
    [self.loading hide];
    NSDictionary *d = [json objectForKey:@"results"];
    NSString *confirmation = [d objectForKey:@"confirmation"];
    if ([confirmation isEqualToString:@"found"]==NO){
        [self showAlert:@"Not Found" message:@"There were no search results returned."];
        return;
        
    }
    
    NSArray *stations = [d objectForKey:@"stations"];
    self.searchResults = [NSMutableArray array];
    for (int i=0; i<[stations count]; i++){
        NSDictionary *info = [stations objectAtIndex:i];
        Station *station = [[Station alloc] init];
        [station populate:info];
        [self.searchResults addObject:station];
        [station release];
    }
    if (self.searchResults.count>0)
        self.theTableview.hidden = NO;
    
    [self.theTableview reloadData];
}

- (void)refresh //called when station thumbnail is ready
{
    [self.theTableview reloadData];
}


#pragma mark - UITableViewStuff
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [searchResults count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *cellID = @"ID";
    HistoryCell *cell = [self.theTableview dequeueReusableCellWithIdentifier:cellID];
    if (cell==nil){
        cell = [[[HistoryCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:cellID] autorelease];
        [cell setup];
        cell.delegate = self;
    }
    Station *station = (Station *)[searchResults objectAtIndex:indexPath.row];
    cell.nameLabel.text = station.name;
    cell.categoryLabel.text = station.category;
    cell.tag = 1000+indexPath.row;
    
    if (station.ready==FALSE){
        station.delegate = self;
        [station getStationInfo];
        return cell;
    }

    station.delegate = nil;
    NSString *text = ([station.tracks containsObject:@"none"]) ? @"0 Tracks" : [NSString stringWithFormat:@"%d tracks", [station.tracks count]];
    [cell.btnTracks setTitle:text forState:UIControlStateNormal];
    
    text = ([station.articles containsObject:@"none"]) ? @"0 Articles" : [NSString stringWithFormat:@"%d articles", [station.articles count]];
    [cell.btnNews setTitle:text forState:UIControlStateNormal];
    
    [cell.btnAdmins setTitle:[NSString stringWithFormat:@"%d admins", [station.admins count]] forState:UIControlStateNormal];
    if(station.thumbnail==nil){
        [station fetchThumbnail:240];
        return cell;
    }
    
    [cell fillImage:station.thumbnail];
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    Station *station = [searchResults objectAtIndex:indexPath.row];
    if (!station)
        return;
    
    BRStationViewController *stationView = [[BRStationViewController alloc] initWithManager:self.butterflyMgr];
    stationView.station = station;
    [self.navigationController pushViewController:stationView animated:YES];
    [stationView release];
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return cellBg.size.height;
}


#pragma mark - HistoryCellDelegate
- (void)buttonAction:(int)tag title:(NSString *)title
{
    NSLog(@"buttonAction: %@, %d", title, tag);
    if (tag>999){
        tag -= 1000;
        if ([title isEqualToString:@"delete"]){ [self deleteStation:tag]; }
    }
}

- (void)deleteStation:(int)index
{
    Station *station = [searchResults objectAtIndex:index];
    if (station){
        NSLog(@"DELETE STATION: %@", station.unique_id);
        [self.searchResults removeObject:station];
        
        NSIndexPath *indexPath = [NSIndexPath indexPathForRow:index inSection:0];
        [self.theTableview deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
        [self.theTableview reloadData];
    }
}


#pragma mark - StationDelegate
- (void)imageReady:(NSString *)addr
{
    
}

- (void)stationInfoReady
{
    [self.theTableview reloadData];
}




- (void)hideScreen
{
    [self.theSearchBar setShowsCancelButton:FALSE animated:YES];
    [self.navigationController setNavigationBarHidden:FALSE animated:YES];
    [self.theSearchBar resignFirstResponder];

    [UIView animateWithDuration:0.4f
                          delay:0.0f
                        options:UIViewAnimationOptionCurveLinear
                     animations:^{
                         screen.alpha = 0.0f;
                     }
                     completion:NULL];

}

#pragma mark - UISearchBarDelegate
- (void)searchBarTextDidBeginEditing:(UISearchBar *)searchBar
{
    [self.theSearchBar setShowsCancelButton:TRUE animated:YES];
    [self.navigationController setNavigationBarHidden:TRUE animated:YES];
    
    self.screen.alpha = 0.0f;
    self.screen.hidden = NO;
    
    [UIView animateWithDuration:0.4f
                          delay:0.0f
                        options:UIViewAnimationOptionCurveLinear
                     animations:^{
                         self.screen.alpha = 0.6f;
                     }
                     completion:NULL];
}

- (void)searchBarSearchButtonClicked:(UISearchBar *)searchBar
{
    [self hideScreen];
    [self search];
}

- (void)searchBarCancelButtonClicked:(UISearchBar *) searchBar
{
    [self hideScreen];
}

#pragma mark - UIResponder
- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
    NSLog(@"touchesBegan:");
    UITouch *touch = [touches anyObject];
    if ([touch.view isEqual:self.screen]){
        [self hideScreen];
        return;
    }
    
    [super touchesBegan:touches withEvent:event];
    
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
    NSLog(@"touchesMoved:");
    UITouch *touch = [touches anyObject];
    
    if ([touch.view isEqual:self.screen])
        return;

    [super touchesMoved:touches withEvent:event];
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
    NSLog(@"touchesEnded:");
    UITouch *touch = [touches anyObject];
    if ([touch.view isEqual:self.screen])
        return;
    
    [super touchesEnded:touches withEvent:event];
}

- (void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event
{
    NSLog(@"touchesCancelled:");
    UITouch *touch = [touches anyObject];
    if ([touch.view isEqual:self.screen])
        return;
    
    [super touchesCancelled:touches withEvent:event];
}



- (void)viewDidUnload
{
    [super viewDidUnload];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

@end
