//
//  Station.m
//  butterflyradio
//
//  Created by Denny Kwon on 4/3/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "Station.h"

#define kTitleKey       @"Title"
#define kRatingKey      @"Rating"

@implementation Station
@synthesize name;
@synthesize unique_id;
@synthesize description;
@synthesize category;
@synthesize image;
@synthesize tags;
@synthesize tracks;
@synthesize admins;
@synthesize plays;
@synthesize imgData;
@synthesize thumbnail;
@synthesize articles;
@synthesize delegate;
@synthesize host;
@synthesize ready;
@synthesize saved;
@synthesize threadMap;
@synthesize threadArray;
@synthesize adFreq;
@synthesize podcastRss;
@synthesize premiumTracks;
@synthesize request;
@synthesize xmlParser;
@synthesize podcastTracks;

- (id)init
{
    self = [super init];
    if (self){
        getImage = nil;
        getThumb = nil;
        ready = FALSE;
        saved = FALSE;
    }
    return self;
}

+ (Station *)stationWithInfo:(NSDictionary *)info
{
    Station *station = [[Station alloc] init];
    [station populate:info];
    return [station autorelease];
}

- (void)dealloc
{
    self.podcastRss = nil;
    self.premiumTracks = nil;
    if (self.request){
        [self.request cancel];
        self.request.delegate = nil;
    }
    self.request = nil;
    self.xmlParser = nil;
    self.podcastTracks = nil;
    
    [threadArray release];
    [threadMap release];
    [queue cancelAllOperations];
    [name release];
    [unique_id release];
    [description release];
    [category release];
    [image release];
    [tags release];
    [tracks release];
    [admins release];
    [articles release];
    [host release];
    if (queue){
        [queue release];
    }
    if (getImage){
        [getImage release];
    }
    if (getThumb){
        [getThumb release];
    }
    [super dealloc];
}

- (void)getStationInfo
{
    if (self.request != nil){
        self.request.delegate = nil;
        self.request = nil;
    }
    
    NSString *url = [NSString stringWithFormat:@"http://%@/api/station/%@", kUrl, self.unique_id];
    self.request = [BRNetworkOp operationWithAddress:url parameters:nil];
    self.request.delegate = self;
    [self.request setHttpMethod:@"GET"];
    [self.request sendRequest];
}

- (void)cancelUpdate
{
    if (self.request!=nil){
        self.request.delegate = nil;
        [self.request cancel];
        self.request = nil;
    }
    self.delegate = nil;
}

- (void)requestData:(NSArray *)pkg //returns [address, data]
{
    if (pkg==nil)
        return;
    
    NSString *url = [NSString stringWithFormat:@"http://%@/api/station/%@", kUrl, self.unique_id];
    
    if ([self.request.address isEqualToString:url]){ // station info request
        NSDictionary *d = [self.request parseJsonResponse];
        if (d==nil){
            [self.request sendRequest];
            return;
        }
        
        d = [d objectForKey:@"results"];
//        NSLog(@"%@", [d description]);
        NSDictionary *info = [d objectForKey:@"station"];
        [self populate:info];
        [delegate stationInfoReady:self.unique_id];
        self.request.delegate = nil;
        self.request = nil;
        return;
    }
    
    // Podcast Request:
    NSData *rssData = [pkg lastObject];
    if (!self.xmlParser){
        self.xmlParser = [XMLParser parser];
        self.xmlParser.delegate = self;
    }
    
    [self.xmlParser parseData:rssData];
    self.request.delegate = nil;
    self.request = nil;
}


- (void)save
{
//    NSLog(@"STATION - save");
    
    Database *db = [Database database];
    NSDictionary *params = [NSDictionary dictionaryWithObjectsAndKeys:self.unique_id, @"idNum", self.name, @"name", self.host, @"host", self.category, @"category", self.image, @"image", nil];
    [db insertToDB:params];
    self.saved = TRUE;
}

- (void)deleteFromDb
{
    Database *db = [Database database];
    [db deleteFromDB:self.unique_id];
    self.saved = NO;
}

- (void)fetchImage
{
//    NSLog(@"STATION: fetchImage");
    if (getImage)
        return;
    
    if (queue==nil){ queue = [[NSOperationQueue alloc] init]; }
    NSString *url = self.image;
    if ([url rangeOfString:@"default.jpg"].location == NSNotFound)
        url = [url stringByAppendingString:@"=s450"];

    getImage = [[GetImage alloc] initWithTarget:self address:url action:@selector(imageReady:)];
    [queue addOperation:getImage];
}

- (void)imageReady:(NSArray *)pkg
{
//    NSLog(@"STATION: imageReady:");
    if (pkg==nil)
        return;
    
    self.imgData = [pkg objectAtIndex:1];
    [getImage release];
    getImage = nil;
    
    if ([delegate respondsToSelector:@selector(imageReady:)])
        [delegate imageReady:self.unique_id];
}

- (void)fetchThumbnail:(int)dimen
{
//    NSLog(@"STATION: fetchThumbnail");
    if (getThumb)
        return;
    
    if (queue==nil){ queue = [[NSOperationQueue alloc] init]; }
    NSString *url = self.image;
    if ([url rangeOfString:@"default.jpg"].location == NSNotFound)
        url = [url stringByAppendingString:[NSString stringWithFormat:@"=s%d-c", dimen]];

    getThumb = [[GetImage alloc] initWithTarget:self address:url action:@selector(thumbnailReady:)];
    [queue addOperation:getThumb];
}

- (void)thumbnailReady:(NSArray *)pkg
{
//    NSLog(@"STATION: thumbnailReady");
    if (pkg == nil)
        return;
    
    self.thumbnail = [UIImage imageWithData:[pkg objectAtIndex:1]];
    [getThumb release];
    getThumb = nil;
    [[NSNotificationCenter defaultCenter] postNotification:[NSNotification notificationWithName:kThumbnailReadyNotification object:nil]];
    if ([delegate respondsToSelector:@selector(thumbnailDownloaded:)])
        [delegate thumbnailDownloaded:self.unique_id];
}

- (NSString *)adminsString
{
    if ([self.admins containsObject:@"none"])
        return @"none";
    
    NSString *adminList = @"";
    for (NSString *a in self.admins){
        adminList = [adminList stringByAppendingString:a];
        adminList = [adminList stringByAppendingString:@","];
    }
    adminList = [adminList substringToIndex:[adminList length]-1];
    return adminList;
}

- (NSString *)tagsString
{
    if ([self.tags containsObject:@"none"])
        return @"none";
    
    NSString *tagsList = @"";
    for (NSString *a in self.tags){
        tagsList = [tagsList stringByAppendingString:a];
        tagsList = [tagsList stringByAppendingString:@","];
    }
    tagsList = [tagsList substringToIndex:tagsList.length-1];
    return tagsList;
}

- (NSMutableDictionary *)createMap:(NSString *)a
{
    NSMutableDictionary *info = [NSMutableDictionary dictionary];
    NSArray *parts = [a componentsSeparatedByString:@"=="];
    for (NSString *part in parts) {
        NSArray *keyValue = [part componentsSeparatedByString:@"::"];
        if ([keyValue count]==2){
            NSString *key = [keyValue objectAtIndex:0];
            NSString *value = [keyValue objectAtIndex:1];
            [info setObject:value forKey:key];
        }
    }
    return info;
}

- (void)parseTracks:(NSArray *)t
{
    self.tracks = [NSMutableArray array];
    self.premiumTracks = [NSMutableArray array];
    self.threadArray = [NSMutableArray array];
    self.threadMap = [NSMutableDictionary dictionary];

    for (int i=0; i<[t count]; i++) {
        NSString *trackString = [t objectAtIndex:i];
        NSMutableDictionary *trackInfo = [self createMap:trackString];
        
        BRTrack *file = [BRTrack trackWithInfo:trackInfo];

        //////
        Thread *th = (Thread *)[threadMap objectForKey:file.thread];
        if (th==nil){
            th = [[Thread alloc] init];
            th.threadId = file.thread;
            
            [th.thread insertObject:file atIndex:0];
            [threadMap setObject:th forKey:th.threadId];
            [threadArray addObject:th];
            [th release];
        }
        else{
            [th.thread insertObject:file atIndex:0];
        }
        ///////
    }
    
    for (int i=0; i<threadArray.count; i++) {
        Thread *thread = (Thread *)[threadArray objectAtIndex:i];
        for (int k=0; k<thread.thread.count; k++){
            BRTrack *t = (BRTrack *)[thread.thread objectAtIndex:k];
            t.reply = !(k==0);
            if (t.value==0){
                t.fileIndex = [NSString stringWithFormat:@"%d", self.tracks.count]; //have to set the file index here because this is where the tracks array actually populates
                [self.tracks addObject:t];
            }
            else{
                t.fileIndex = [NSString stringWithFormat:@"%d", self.premiumTracks.count];
                [self.premiumTracks addObject:t];
            }
        }
    }
}

- (void)checkDatabase
{
    Database *db = [Database database];
    self.saved = [db checkSaved:self.unique_id];
}

- (void)populate:(NSDictionary *)info
{
    for (NSString *key in [info allKeys]){
        if ([key isEqualToString:@"adFrequency"]){ self.adFreq = [[info objectForKey:key] intValue]; }
        if ([key isEqualToString:@"name"]){ self.name = [info objectForKey:key]; }
        if ([key isEqualToString:@"id"]){ 
            self.unique_id = [info objectForKey:key];
            [self checkDatabase];
        }
        if ([key isEqualToString:@"podcast rss"]){ self.podcastRss = [info objectForKey:key]; }
        if ([key isEqualToString:@"description"]){ self.description = [info objectForKey:key]; }
        if ([key isEqualToString:@"category"]){ self.category = [info objectForKey:key]; }
        if ([key isEqualToString:@"host"]){ self.host = [[info objectForKey:key] lowercaseString]; }
        if ([key isEqualToString:@"image"]){ 
            NSString *img = [info objectForKey:key];
            NSArray *parts = [img componentsSeparatedByString:@"=="];
            self.image = [parts objectAtIndex:0];
        }
        
        if ([key isEqualToString:@"tags"]){ self.tags = [info objectForKey:key]; }
        if ([key isEqualToString:@"tracks"]){
            NSArray *t = [info objectForKey:key];
            if ([t containsObject:@"none"]){
                self.tracks = [NSMutableArray arrayWithArray:t];
            }
            else{
                [self parseTracks:t];
            }
        }
        if ([key isEqualToString:@"admins"]){ self.admins = [info objectForKey:key]; }
        if ([key isEqualToString:@"articles"]){ 
            NSArray *a = [info objectForKey:key];
            if ([a containsObject:@"none"]){
                self.articles = [NSMutableArray arrayWithArray:a];
            }
            else{
                NSMutableArray *array = [NSMutableArray array];
                for (NSString *s in a) {
                    Article *article = [[Article alloc] init];
                    [article parse:s];
                    [array addObject:article];
                    [article release];
                }
                self.articles = array;
                
            }
        }
        if ([key isEqualToString:@"plays"]){ self.plays = [[info objectForKey:key] longLongValue]; }
    }
    ready = TRUE;
}

- (void)downloadPodcast
{
    if (!self.podcastRss)
        return;
    
    if ([self.podcastRss isEqualToString:@"none"])
        return;
    
    if (self.request) // don't send request while another one is still out
        return;
    
    self.request = [BRNetworkOp operationWithAddress:self.podcastRss parameters:nil];
    self.request.delegate = self;
    [self.request setHttpMethod:@"GET"];
    [self.request sendRequest];

}

#pragma mark - XMLParserDelegate
- (void)parserFinished
{
    NSLog(@"PARSER FINISHED: %d ITEMS", self.xmlParser.items.count);
    self.podcastTracks = [NSMutableArray array];
    for (int i=0; i<self.xmlParser.items.count; i++) {
        NSDictionary *podcastInfo = [self.xmlParser.items objectAtIndex:i];
//        NSLog(@"%@", [podcastInfo description]);
        
        NSString *url = [podcastInfo objectForKey:@"link"];
        if ([url rangeOfString:@".mp3"].location==NSNotFound && [url rangeOfString:@".m4a"].location==NSNotFound)
            url = [podcastInfo objectForKey:@"url"];
        
        if ([url hasSuffix:@".mp3"]==YES || [url hasSuffix:@".m4a"]==YES){ //some of the links are malformed (no .mp3 extension) - exclude those
            BRTrack *track = [[BRTrack alloc] init];
            track.fileIndex = [NSString stringWithFormat:@"%d", self.podcastTracks.count];
            track.url = url;
            track.author = self.name;
            track.name = [podcastInfo objectForKey:@"title"];
            NSString *date = [podcastInfo objectForKey:@"pubDate"];
            NSArray *parts = [date componentsSeparatedByString:@" "];
            track.date = (parts.count > 2) ? [NSString stringWithFormat:@"%@ %@, %@", [parts objectAtIndex:2], [parts objectAtIndex:1], [parts objectAtIndex:3]] : date;
            track.description = [podcastInfo objectForKey:@"description"];
            NSString *keywords = [podcastInfo objectForKey:@"itunes:keywords"];
            if (keywords)
                track.tags = [keywords componentsSeparatedByString:@","];
            
            NSString *duration = [podcastInfo objectForKey:@"itunes:duration"]; // example = 1:19:35
            if (duration){
                double time = 0;
                parts = [duration componentsSeparatedByString:@":"];
                for (int i=0; i<parts.count; i++){
                    NSString *t = [parts objectAtIndex:i];
                    int k = i+1;
                    time += pow(60, (parts.count-k))*[t doubleValue];
                }
                track.duration = time;
//                NSLog(@"%@ == %.2f SECONDS", track.name, track.duration);
                double m = track.duration/60.0f;
                track.mins = (int)m;
            }
            
            [self.podcastTracks addObject:track];
            [track release];
        }
    }
    
    [self.delegate podcastTracksReady];
}

@end
