//
//  BRGradientBar.m
//  ButterflySDK
//
//  Created by Denny Kwon on 1/19/13.
//  Copyright (c) 2013 ButterflyRadio. All rights reserved.
//

#import "BRGradientBar.h"

@interface BRGradientBar ()
@property (nonatomic) CGPoint start;
@property (nonatomic) CGFloat originalY;
@property (nonatomic) NSTimeInterval startTime;
@end


@implementation BRGradientBar
@synthesize lblTitle;
@synthesize delegate;
@synthesize btnToggle;
@synthesize moveable;


- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        self.moveable = YES;
        
        UIImage *imgGradient = [UIImage imageNamed:@"gradientBar.png"];
        UIImageView *gradientBar = [[UIImageView alloc] initWithImage:imgGradient];
        gradientBar.frame = CGRectMake(0, 0, frame.size.width, imgGradient.size.height);
        frame.size.height = imgGradient.size.height;
        self.frame = frame;
        [self addSubview:gradientBar];
        [gradientBar release];
        
        self.lblTitle = [[[UILabel alloc] initWithFrame:CGRectMake(8, 5, 230, frame.size.height-10)] autorelease];
        self.lblTitle.backgroundColor = [UIColor clearColor];
        self.lblTitle.textColor = [UIColor whiteColor];
        self.lblTitle.text = @"Title Label";
        self.lblTitle.font = [UIFont fontWithName:@"Heiti SC" size:18.0f];
        [self addSubview:self.lblTitle];
        
        self.btnToggle = [UIButton buttonWithType:UIButtonTypeCustom];
        [self.btnToggle setBackgroundImage:[UIImage imageNamed:@"BRBtnToggle.png"] forState:UIControlStateNormal];
        self.btnToggle.frame = CGRectMake(self.frame.size.width-45, 0.5*(frame.size.height-35), 35, 35);
        [self.btnToggle addTarget:self.delegate action:@selector(btnGradientTapped:) forControlEvents:UIControlEventTouchUpInside];
        [self addSubview:self.btnToggle];
        
        self.originalY = self.center.y;
    }
    return self;
}

- (id)initWithFrame:(CGRect)frame background:(UIColor *)bg
{
    self = [super initWithFrame:frame];
    if (self) {
        self.moveable = YES;
        self.backgroundColor = bg;
        
        self.lblTitle = [[[UILabel alloc] initWithFrame:CGRectMake(8, 5, 230, frame.size.height-10)] autorelease];
        self.lblTitle.backgroundColor = [UIColor clearColor];
        self.lblTitle.textColor = [UIColor whiteColor];
        self.lblTitle.text = @"Title Label";
        self.lblTitle.font = [UIFont fontWithName:@"Heiti SC" size:18.0f];
        [self addSubview:self.lblTitle];
        
        self.btnToggle = [UIButton buttonWithType:UIButtonTypeCustom];
        [self.btnToggle setBackgroundImage:[UIImage imageNamed:@"BRBtnToggle.png"] forState:UIControlStateNormal];
        self.btnToggle.frame = CGRectMake(self.frame.size.width-45, 0.5*(frame.size.height-35), 35, 35);
        [self.btnToggle addTarget:self.delegate action:@selector(btnGradientTapped:) forControlEvents:UIControlEventTouchUpInside];
        [self addSubview:self.btnToggle];
        
        self.originalY = self.center.y;
    }
    return self;
}

+ (BRGradientBar *)barWithFrame:(CGRect)frame
{
    BRGradientBar *bar = [[BRGradientBar alloc] initWithFrame:frame];
    return [bar autorelease];
}

+ (BRGradientBar *)barWithFrame:(CGRect)frame background:(UIColor *)bg
{
    BRGradientBar *bar = [[BRGradientBar alloc] initWithFrame:frame background:bg];
    return [bar autorelease];
}

- (void)dealloc
{
    self.lblTitle = nil;
    self.delegate = nil;
    self.btnToggle = nil;
    [super dealloc];
}

- (void)rotateButton:(CGAffineTransform)orientation
{
    //    self.btnToggle.transform = orientation;
    
    [UIView animateWithDuration:0.25f
                          delay:0.0f
                        options:UIViewAnimationOptionCurveLinear
                     animations:^{
                         self.btnToggle.transform = orientation;
                     }
                     completion:NULL];
}


#pragma mark - TouchResponder
- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
    if (!moveable)
        return;
    
    UITouch *t = [touches anyObject];
    self.start = [t locationInView:self.superview];
    NSLog(@"touchesBegan: %.2f, %.2f", self.start.x, self.start.y);
    
    self.startTime = [[NSDate date] timeIntervalSince1970];
    
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
    if (!moveable)
        return;
    
    NSLog(@"touchesMoved:");
    UITouch *t = [touches anyObject];
    CGPoint point = [t locationInView:self.superview];
    
    CGPoint ctr = self.center;
    double delta = (point.y - self.start.y);
    ctr.y += delta;
    
    if (ctr.y < self.originalY)
        return;
    
    self.center = ctr;
    
    self.start = point;
    [self.delegate gradientBarMoved:delta];
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
    if (!moveable)
        return;

    NSLog(@"touchesEnded:");
    NSTimeInterval endTime = [[NSDate date] timeIntervalSince1970];
    if (endTime-self.startTime < 0.2f){ // swipe
        NSLog(@"SWIPE");
        UITouch *t = [touches anyObject];
        CGPoint point = [t locationInView:self.superview];
        [self.delegate swipe:(point.y-self.center.y)];
        return;
    }
    
    
    [self.delegate gradientBarStopped];

}

- (void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event
{
    if (!moveable)
        return;

    NSLog(@"touchesCancelled:");
    [self.delegate gradientBarStopped];

}

/*
 // Only override drawRect: if you perform custom drawing.
 // An empty implementation adversely affects performance during animation.
 - (void)drawRect:(CGRect)rect
 {
 // Drawing code
 }
 */

@end
