//
//  LoadingIndicator.m
//  butterflyradio
//
//  Created by Denny Kwon on 5/5/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "LoadingIndicator.h"

@implementation LoadingIndicator
@synthesize titleLabel;
@synthesize darkScreen;
@synthesize progress;
@synthesize spinner;

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        self.autoresizingMask = UIViewAutoresizingFlexibleWidth;
        
        CGFloat width = 160.0f;
        UIView *background = [[UIView alloc] initWithFrame:CGRectMake(0.5*(frame.size.width-width), 100, width, 100)];
        
        self.darkScreen = background;
        [background release];
        
        self.darkScreen.autoresizingMask = (UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin);;
        self.darkScreen.layer.cornerRadius = 10.0f;
        self.darkScreen.backgroundColor = [UIColor blackColor];
        self.darkScreen.alpha = 0.7f;
        [self addSubview:self.darkScreen];
        
        titleLabel = [[UILabel alloc] initWithFrame:CGRectMake(0.5*(frame.size.width-width), 110, width, 35)];
        self.titleLabel.autoresizingMask = (UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin);
        self.titleLabel.text = @"Loading...";
        self.titleLabel.font = [UIFont fontWithName:@"Helvetica" size:18.0f];
        self.titleLabel.textAlignment = UITextAlignmentCenter;
        self.titleLabel.backgroundColor = [UIColor clearColor];
        self.titleLabel.textColor = [UIColor whiteColor];
        [self addSubview:self.titleLabel];
        
        width = 20.0f;
        spinner = [[UIActivityIndicatorView alloc] initWithFrame:CGRectMake(0.5*(frame.size.width-width), 150, width, width)];
        self.spinner.autoresizingMask = (UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin);
        [self addSubview:self.spinner];
        
        
        progress = [[UIProgressView alloc] initWithProgressViewStyle:UIProgressViewStyleDefault];
        CGFloat w = self.darkScreen.frame.size.width-30;
        self.progress.frame = CGRectMake(0.5*(self.frame.size.width-w), 180, w, 10);
        [self addSubview:self.progress];
        progress.hidden = YES;
    }
    return self;
}

- (id)initWithFrame:(CGRect)frame loadingIndicator:(int)color verticalOffset:(CGFloat)y
{
    self = [super initWithFrame:frame];
    if (self) {
        UIImage *imgCircle = nil;
        if (color==0)
            imgCircle = [UIImage imageNamed:@"loadingBlue.png"];

        if (color==1)
            imgCircle = [UIImage imageNamed:@"loadingRed.png"];

        if (color==2)
            imgCircle = [UIImage imageNamed:@"loadingPurple.png"];

        UIImageView *cirlce = [[UIImageView alloc] initWithImage:imgCircle];
        cirlce.center = CGPointMake(0.5f*frame.size.width, y);
        [self addSubview:cirlce];
        [cirlce release];
        
        self.spinner = [[[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleWhite] autorelease];
        self.spinner.center = cirlce.center;
        self.spinner.autoresizingMask = (UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin);
        [self addSubview:self.spinner];
    }
    return self;
}



- (void)dealloc
{
    self.darkScreen = nil;
    [titleLabel release];
    [spinner release];
    [progress release];
    [super dealloc];
}

- (void)show
{
    [self.spinner startAnimating];
    if (!self.hidden)
        return;
    
    self.alpha = 0.0f;
    self.hidden = NO;
    
    [UIView animateWithDuration:0.4f
                          delay:0.0f
                        options:UIViewAnimationOptionCurveLinear
                     animations:^{
                         self.alpha = 1.0f;
                     }
                     completion:NULL];
}

- (void)hide
{
    [UIView animateWithDuration:0.4f
                          delay:0.0f
                        options:UIViewAnimationOptionCurveLinear
                     animations:^{
                         self.alpha = 0.0f;
                     }
                     completion:^(BOOL finisehd){
                         self.hidden = YES;
                         [self.spinner stopAnimating];
                     }];
}


/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect
{
    // Drawing code
}
*/

@end
