//
//  XMLParser.m
//  matarese
//
//  Created by Denny Kwon on 12/9/12.
//  Copyright (c) 2012 com.thegridmedia. All rights reserved.
//

#import "XMLParser.h"

@implementation XMLParser
@synthesize parser;
@synthesize items;
@synthesize open;
@synthesize currentItem;
@synthesize currentElement;
@synthesize delegate;

+ (XMLParser *)parser
{
    XMLParser *p = [[XMLParser alloc] init];
    return [p autorelease];
}

- (void)dealloc
{
    self.parser = nil;
    self.currentItem = nil;
    self.currentElement = nil;
    self.items = nil;
    [super dealloc];
}

- (void)parseData:(NSData *)data
{
    if (self.parser){
        self.parser.delegate = nil;
        self.parser = nil;
    }
    
    self.items = nil;
    self.open = NO;
    parser = [[NSXMLParser alloc] initWithData:data];
    self.parser.delegate = self;
    [self.parser parse];
}

#pragma mark - NSXMLParserDelegate
- (void)parserDidStartDocument:(NSXMLParser *)parser
{
    //NSLog(@"parserDidStartDocument:");
    self.items = [NSMutableArray array];
}

- (void)parserDidEndDocument:(NSXMLParser *)parser
{
    //NSLog(@"parserDidEndDocument: %d ITEMS", self.items.count);
    
    if ([self.delegate respondsToSelector:@selector(parserFinished)])
        [self.delegate parserFinished];
}

- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict
{
    //NSLog(@"parser didStartElement: %@, %@", elementName, [attributeDict description]);
    
    if ([elementName isEqualToString:@"item"])
        self.currentItem = [NSMutableDictionary dictionary];

    
    if (self.currentItem)
        self.currentElement = elementName;
    
    if (!attributeDict)
        return;
    
    for (NSString *key in attributeDict.allKeys){
        NSString *value = [attributeDict objectForKey:key];
        [self.currentItem setObject:value forKey:key];
    }


}

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string
{
    //NSLog(@"parser foundCharacters: %@", string);
    if (!self.currentElement)
        return;
    
    NSMutableString *value = (NSMutableString *)[self.currentItem objectForKey:self.currentElement];
    if (!value){
        value = [NSMutableString string];
        [self.currentItem setObject:value forKey:self.currentElement];
    }
    
    [value appendString:string];
}

- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName
{
    //NSLog(@"parser didEndElement: %@", elementName);
    
    if ([elementName isEqualToString:@"item"]){
        [self.items addObject:self.currentItem];
        self.currentItem = nil;
    }
    
    if (self.currentItem)
        self.currentElement = nil;


}



@end
